#!/bin/sh

### ====== print help  ========== ###
case "$@" in
  -h|--help|-help|"")
    echo "The help below is for ./makenemo but almost all examples and all options apply to concepts.build!"
    echo "---------------------------------------------------------------------------------------"
    ./makenemo -h "all"
    exit
esac



echo "************** Compiling CONCEPTS **************"
echo ""

### ====== argument handling  ========== ###

# Default argument values
make_jobs_arg=''
make_output_arg=''
verbose_setting=''
debug_setting=''
clean_config=false

# Get arguments needed for coupling libraries
# NOTE: we don't use shift because we want to send almost all arguments to makenemo
i=0
for arg in "$@" ; do
  i=$((i+1))
  if [ "${arg}" == "-m" ]; then
    next_arg="arch"
    continue
  elif [ "${next_arg}" == "arch" ]; then
    environment=${arg}
    next_arg=''
  elif [ "${arg}" == "-n" ]; then
    next_arg="config"
    continue
  elif [ "${next_arg}" == "config" ]; then
    nemo_config=${arg}
    next_arg=''
  elif [ "${arg}" == "-t" ]; then
    next_arg="blddir"
    continue
  elif [ "${next_arg}" == "blddir" ]; then
    blddir=${arg}
    next_arg=''
  elif [ "${arg}" == "-O" ]; then
    make_output=true
    j=$((i-1))
    k=$((i+1))
    set -- "${@:1:$j}" "${@:$k:$#}"
  elif [ "${arg}" == "-j" ]; then
    next_arg="jobs"
    continue
  elif [ "${next_arg}" == "jobs" ]; then
    jobs=${arg}
    next_arg=''
    make_parallel=true
  elif [[ ${arg} == *"VERBOSE"* ]]; then
    verbose_setting=${arg}
    j=$((i-1))
    k=$((i+1))
    set -- "${@:1:$j}" "${@:$k:$#}"
  elif [[ ${arg} == *"DEBUG"* ]]; then
    debug_setting=${arg}
    j=$((i-1))
    k=$((i+1))
    set -- "${@:1:$j}" "${@:$k:$#}"
  elif [ "${arg}" == "clean" ]; then
    clean_config=true
  fi
done

if [ "${make_ouput}" = true ]; then
make_output_arg=-O
fi
if [ "${make_parallel}" = true ]; then
make_jobs_arg="-j ${jobs}"
fi

### ====== compilation  ========== ###

current_dir=`pwd`
export TREE_ROOT=`git rev-parse --show-toplevel`

# First compile coupling librairies
if [ "${clean_config}" = false ]; then
  echo "********* Compiling coupling libraries *********"

  cd ${TREE_ROOT}/coupling/atm-ocn/
  components="itf_flx_5.0.rc1 cpl_lib"
  ./script/build ${environment} ${make_jobs_arg} ${make_output_arg} ${components} ${verbose_setting} ${debug_setting} || exit 1
fi

cd ${current_dir}

# Load NEMO compilation environment (.env file) -- inspired from make_xios
# NOTE: we do this now so that EC_ARCH is defined
# NOTE: this is also in makenemo so that calling makenemo directly also works
nb=$( find ${TREE_ROOT}/NEMOGCM/ARCH -name arch-${environment}.env -print | wc -l )
if [ $nb -eq 0 ]; then # no env file found for chosen arch
    echo "Warning !!!"
    echo "Environment file arch-${environment}.env not found"
    echo "EXITING..."
    exit 1
fi
env_file=$( find ${TREE_ROOT}/NEMOGCM/ARCH -name arch-${environment}.env -print )
source ${env_file}

# Call NEMO compiling script
my_args=$(printf " %s\n" "$@" | tr '\n' ' ' | sed "s#\s\+\-t\s\+${blddir}##g" | sed "s/\s\+clean\b//g")

if [ -z "${blddir}" ] ; then
  # Force compiling destination directory if not defined
  bld_dest="${HOME}/data/${TRUE_HOST}/CONCEPTS/build.${EC_ARCH}"
  mkdir -p ${bld_dest}
  blddir=${current_dir}/build.${environment}.${TRUE_HOST}
  ln -sfn ${bld_dest} ${blddir}
fi

echo ""
echo "************** Compiling NEMO-CICE *************"
echo "************** In following dir:   *************"
echo "==> ${blddir}"

if [ "${clean_config}" = true ]; then
  set -x
  ./makenemo ${my_args} -t ${blddir} clean || { { set +x; } 2> /dev/null; exit 1; }
else
  set -x
  ./makenemo ${my_args} -t ${blddir}       || { { set +x; } 2> /dev/null; exit 1; }
  { set +x; } 2> /dev/null
  # Add information to the BLD directory about the status of the branch
  echo "BRANCH INFORMATION:"  > ${blddir}/${nemo_config}/BLD/bin/nemo_branch.txt
  git describe --dirty       >> ${blddir}/${nemo_config}/BLD/bin/nemo_branch.txt
  echo "STATUS INFORMATION:" >> ${blddir}/${nemo_config}/BLD/bin/nemo_branch.txt
  git status                 >> ${blddir}/${nemo_config}/BLD/bin/nemo_branch.txt
fi
