MODULE diahbar
   !!======================================================================
   !!                       ***  MODULE  diahba  ***
   !! HABR diagnostics
   !!======================================================================
   !! History :  3.5  !  2013-10  (R. Bourdalle-Badie)  Original code
   !!----------------------------------------------------------------------
#if defined key_diahbar   || defined key_esopa
   !!----------------------------------------------------------------------
   !!   'key_diahbar' :                           activate barotropic diagnotics
   !!----------------------------------------------------------------------
   !!   dia_hbar      : barotropic diagnostics
   !!   dia_hbar_init : initialisation of barotropic diagnostics
   !!----------------------------------------------------------------------
   USE oce            ! ocean dynamics and active tracers 
   USE dom_oce        ! ocean space and time domain
   USE eosbn2         ! equation of state                (eos_bn2 routine)
   USE par_oce
   USE sol_oce         ! ocean elliptic solver
   USE solpcg          ! preconditionned conjugate gradient solver
   USE solsor          ! Successive Over-relaxation solver
   USE solver
#if ! defined key_rpne
   USE fsbmat
#endif
   USE par_kind
   USE in_out_manager
   USE lib_mpp        ! distribued memory computing library
   USE lbclnk         ! ocean lateral boundary conditions (or mpp link)
   USE iom            ! I/O manager library
   USE timing         ! preformance summary
   USE wrk_nemo       ! working arrays
   USE restart

   IMPLICIT NONE
   PRIVATE

#if ! defined key_rpne
   PUBLIC   dia_hbar       ! routine called in step.F90 module
   PUBLIC   dyn_hpa        ! routine called in step.F90 module
   PUBLIC   dia_hbar_init  ! routine called in opa.F90 module
   PUBLIC   dia_hbar_alloc ! routine called in nemogcm.F90 module
   PUBLIC   dia_trdfs      ! routine called by dia_hbar
   PUBLIC   dia_trdfs_ini  ! routine called by dia_hbar_init

   INTEGER                     :: nn_max_hbar      != 40000
   INTEGER, PUBLIC             :: nn_write_hbar    != 0
   REAL(wp)                    :: rn_eps_hbar      != 1.e-6
   REAL(wp)                    :: rn_sor_hbar      != 1.999
   LOGICAL, PUBLIC             :: ln_dia_trdfs
   LOGICAL, PUBLIC             :: ln_temporal_mean != .FALSE.
   !rbb CHARACTER(LEN=jp_oce_lenmax), PUBLIC  :: cn_path_hbar ='./'
   CHARACTER(LEN=256), PUBLIC  :: cn_path_hbar     !='./'
   !
   LOGICAL, PUBLIC, PARAMETER                          :: lk_diahbar = .TRUE. 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: hbar
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: utrd,zfubar
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: vtrd,zfvbar

!##################  for cmc_sam2 and rpne
#else
   INTEGER                     :: nn_max_hbar      != 40000
   INTEGER                     :: nn_write_hbar    != 0   Frequency to update Hbar
   REAL(wp)                    :: rn_eps_hbar      != 1.e-6
   REAL(wp)                    :: rn_sor_hbar      != 1.999
   LOGICAL                     :: ln_dia_trdfs
   LOGICAL                     :: ln_temporal_mean != .FALSE.
   !rbb CHARACTER(LEN=jp_oce_lenmax), PUBLIC  :: cn_path_hbar ='./'
   CHARACTER(LEN=256)          :: cn_path_hbar     !='./'
   !
   LOGICAL,          PARAMETER                         :: lk_diahbar = .TRUE.
   REAL(wp),         ALLOCATABLE,  DIMENSION(:,:)      :: hbar
   REAL(wp),         ALLOCATABLE,  DIMENSION(:,:)      :: zfubar
   REAL(wp),         ALLOCATABLE,  DIMENSION(:,:)      :: zfvbar
   REAL(wp),         ALLOCATABLE,  DIMENSION(:,:) :: zfubar_mean,zfvbar_mean
   REAL(wp),         ALLOCATABLE,  DIMENSION(:,:)      :: gcxfs, gcxbfs   ! add them to distinguish dynspg_flt.
   integer                                             :: numsol_hbar     ! add to monitor solver for hbar

   logical,parameter          :: diahbar_print_cmc = .FALSE.   ! if .F., don't print message
#if defined key_cmc_sam2
   integer                    :: il_spinup_diahbar = -1        !                : no spin up
                                                               ! positive number: the end step of spin-up in cold restart
#endif

   Public ::  dia_hbar, hbar, lk_diahbar,  nn_write_hbar
!##################for cmc_sam2 and rpne
#endif

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: diabar.F90 3294 2012-01-28 16:44:18Z rblod $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   FUNCTION dia_hbar_alloc()
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE dia_bar_alloc  ***
      !!----------------------------------------------------------------------
      INTEGER :: dia_hbar_alloc
      !!----------------------------------------------------------------------
      !
#if defined key_rpne
     ALLOCATE( zfubar(jpi,jpj) , zfvbar(jpi,jpj), hbar(jpi,jpj),                                 STAT=dia_hbar_alloc )
#else
     ALLOCATE( zfubar(jpi,jpj) , zfvbar(jpi,jpj), hbar(jpi,jpj), utrd(jpi,jpj) , vtrd(jpi,jpj) , STAT=dia_hbar_alloc )
#endif
      !
      IF( lk_mpp             )   CALL mpp_sum ( dia_hbar_alloc )
      IF( dia_hbar_alloc /= 0 )   CALL ctl_warn('dia_hbar_alloc: failed to allocate arrays')
      !
   END FUNCTION dia_hbar_alloc

   SUBROUTINE dia_hbar( kt )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE dia_hbar  ***
      !!
      !! ** Purpose :   compute and output some barotropic diagnostics
      !!----------------------------------------------------------------------
      !
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index
      !
      INTEGER  ::   ji, jj, jk        ! dummy loop arguments
      !
      REAL(wp) :: zua, zva
      REAL(wp) :: zmin , zmax
#if defined key_rpne
      INTEGER :: il_dia_hbar_alloc
#endif
      !               
      REAL(wp), POINTER, DIMENSION(:,:)   :: zhu , zhv          ! 2D workspace
      REAL(wp), POINTER, DIMENSION(:,:)   :: zdiv               ! 2D workspace
      REAL(wp), POINTER, DIMENSION(:,:)   :: zwx, zwy, zwu, zwv ! 2D workspace
      !!--------------------------------------------------------------------
      IF( nn_timing == 1 )   CALL timing_start('dia_hba')
 
      CALL wrk_alloc( jpi , jpj , zhu , zhv, zdiv)
      CALL wrk_alloc( jpi , jpj , zwx, zwy, zwu, zwv)

      ! 0. Initialisation
      IF( kt == nit000 ) THEN
         CALL dia_hbar_init
#if ! defined key_rpne
         utrd(:,:) = 0.e0
         vtrd(:,:) = 0.e0
#endif
#if defined key_rpne
         IF ( ln_dia_trdfs.AND.(nn_write_hbar/=1) ) THEN
            ALLOCATE( zfubar_mean(jpi,jpj) , zfvbar_mean(jpi,jpj), STAT=il_dia_hbar_alloc )
            IF( il_dia_hbar_alloc /= 0 )   CALL ctl_warn('dia_hbar: failed to allocate arrays')
            zfubar_mean(:,:) = 0.e0
            zfvbar_mean(:,:) = 0.e0
         ENDIF
#endif
      ENDIF
      zfubar(:,:) = 0.e0
      zfvbar(:,:) = 0.e0
      zhu(:,:)    = 0.e0
      zhv(:,:)    = 0.e0
      zdiv(:,:)   = 0.e0

#if ! defined key_rpne
      IF( MOD(kt-nit000+1,nn_write_hbar)==0. )THEN
#endif

      ! 1. compute fUbar, fVbar, hUbar, hVbar 
      !                                                ! ===============
      DO jk = 1, jpkm1                                 ! Horizontal slab
         !                                             ! ===============
         zwx(:,:) = e2u(:,:) * fse3u(:,:,jk) * un(:,:,jk)
         zwy(:,:) = e1v(:,:) * fse3v(:,:,jk) * vn(:,:,jk)
#if ! defined key_vvl
         IF(jk == 1) THEN
            zwx(:,:) = zwx(:,:) + e2u(:,:) * sshn(:,:) * un(:,:,jk)
            zwy(:,:) = zwy(:,:) + e1v(:,:) * sshn(:,:) * vn(:,:,jk)
         ENDIF
#endif

         DO jj = 2, jpjm1
            DO ji = 2, jpim1   ! vector opt.
               zua= 0.25/e1u(ji,jj)*( zwy(ji,jj-1)+zwy(ji+1,jj-1)+zwy(ji ,jj)+zwy(ji+1,jj  ) ) ! V at U point
               zva=-0.25/e2v(ji,jj)*( zwx(ji-1,jj)+zwx(ji-1,jj+1)+zwx(ji ,jj)+zwx(ji  ,jj+1) ) ! U at V point
               zua = zua*( ff(ji  ,jj-1)+ff(ji,jj) )*0.5  ! Coriolis term at U point
               zva = zva*( ff(ji-1,jj  )+ff(ji,jj) )*0.5  ! Coriolis term at V point

               zfubar(ji,jj)  = zfubar(ji,jj) + zua *umask(ji,jj,jk)   ! SUM for compute f*Ubar*H
               zfvbar(ji,jj)  = zfvbar(ji,jj) + zva *vmask(ji,jj,jk)   ! SUM for compute f*Vbar*H
            END DO
         END DO
      END DO

      ! Lateral boundary conditions 
      CALL lbc_lnk( zfubar , 'U', -1. )
      CALL lbc_lnk( zfvbar , 'V', -1. )
! FD debug      zmin=MINVAL(zfubar);zmax=MAXVAL(zfubar);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zfubar3 ",zmin,zmax
! FD debug      zmin=MINVAL(zfvbar);zmax=MAXVAL(zfvbar);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zfvbar3 ",zmin,zmax

#if ! defined key_rpne
     CALL iom_put("fubar",zfubar)
     CALL iom_put("fvbar",zfvbar)
#endif

#if  defined key_rpne
     IF ( ln_dia_trdfs.AND.(nn_write_hbar/=1) ) THEN
        zfubar_mean(:,:)=zfubar_mean(:,:)+zfubar(:,:)
        zfvbar_mean(:,:)=zfvbar_mean(:,:)+zfvbar(:,:)
     ENDIF
 
     IF( MOD(kt-nit000+1,nn_write_hbar)==0. )THEN

        IF ( ln_dia_trdfs.AND.(nn_write_hbar/=1) ) THEN
           zfubar(:,:)=zfubar_mean(:,:)/real(nn_write_hbar)
           zfvbar(:,:)=zfvbar_mean(:,:)/real(nn_write_hbar)
           zfubar_mean(:,:) = 0.0
           zfvbar_mean(:,:) = 0.0
        ENDIF
#endif
     ! Barotropic height computation
     ! ------------------------------
#if defined key_cmc_sam2
     if ( ln_dia_trdfs  .and.  kt > il_spinup_diahbar )  CALL dia_trdfs( kt )
#else
     IF( ln_dia_trdfs ) CALL dia_trdfs( kt )         
#endif

     ENDIF
     !
     CALL wrk_dealloc( jpi , jpj , zhu , zhv, zdiv)
     CALL wrk_dealloc( jpi , jpj , zwx, zwy, zwu, zwv)
     !
     IF( nn_timing == 1 )   CALL timing_stop('dia_hba')
      !
   END SUBROUTINE dia_hbar

  SUBROUTINE dia_trdfs ( kt )
    !!---------------------------------------------------------------------
    !!
    !!                       ROUTINE diatrdfs
    !!
    !!  Purpose :
    !!  --------
    !!	Inversion de la free surface à partir de l'equation de la dynamique
    !!     tous les nwri_trdfs pas de temps.
    !!
    !!   Method :
    !!   -------
    !!	cf equipe R&D Mercator + Compte-rendu reunion free surface du
    !!        13 / 02 / 2003.
    !!
    !!   Modifications :
    !!   --------------
    !!       original      : 03-07 (N. Ferry)
    !!----------------------------------------------------------------------
    !! * Arguments
#if defined key_rpne
! just in case different length scale  used for hbar and dynspg_flt
! now, share same sol_mat with dynspg_flt
    use solmat, only : z2rdt => rdt2_flt_hbar
#endif

    INTEGER, INTENT( in ) ::   kt     ! ocean time-step index

    !! * Local declarations
    INTEGER ::  ktask, kindic, kcpu
    INTEGER :: ji, jj, jk, it, nmaxeta
    REAL(wp) :: epseta, soreta, zbtd, zgcb, zgwgt,ze1e2t, zpsmea
    REAL(wp) :: zcu,zcv,zvau,zuav,zfact1
    REAL(wp) :: zmin,zmax

    REAL(wp), POINTER, DIMENSION(:,:)   :: gcxeta, gcxbeta, gcdprceta, gcdmateta
    REAL(wp), POINTER, DIMENSION(:,:)   :: zwu, zwv
    REAL(wp), POINTER, DIMENSION(:,:)   :: ztmp
    REAL(wp), POINTER, DIMENSION(:,:,:) :: gcpeta
  
    !!----------------------------------------------------------------------
      IF( nn_timing == 1 )   CALL timing_start('dia_trrdfs')

    ! 1. Initialization
    ! -----------------
    !define time axis
    it= kt-nit000+1
#if defined key_rpne
    if ( diahbar_print_cmc )  then
#endif
    IF(lwp)WRITE(numout,*)"diatrdfs kt ", kt ; CALL FLUSH(numout)
#if defined key_rpne
    endif
#endif
 

      CALL wrk_alloc( jpi , jpj      , gcxeta, gcxbeta, gcdprceta, gcdmateta )
      CALL wrk_alloc( jpi , jpj      , zwu, zwv, ztmp                        )
      CALL wrk_alloc( jpi , jpj , 4  , gcpeta                                )

      !backup for eta of the solver solution (if dynspg_flt activated, else
      !gcx,... =0) 
      DO jj=1,jpj
         DO ji=1,jpi
#if defined key_rpne  && defined key_dynspg_flt
! NB. same solver for both hbar and dynspg_flt.
!     we can test different length scale (z2rdt) for hbar, later.
            gcxeta   (ji, jj)   = gcx   (ji,jj)
            gcxbeta  (ji, jj)   = gcxb  (ji,jj)
#endif

            !gcxeta   (ji,jj)   = gcx   (ji,jj)
            !gcxbeta  (ji,jj)   = gcxb  (ji,jj)
            !gcpeta   (ji,jj,1) = gcp   (ji,jj,1)
            !gcpeta   (ji,jj,2) = gcp   (ji,jj,2)
            !gcpeta   (ji,jj,3) = gcp   (ji,jj,3)
            !gcpeta   (ji,jj,4) = gcp   (ji,jj,4)
            !gcdprceta(ji,jj)   = gcdprc(ji,jj)
            !gcdmateta(ji,jj)   = gcdmat(ji,jj)
            ! initialisation for the new solveur :
            !gcp   (ji,jj,1) = gcpfs   (ji,jj,1)
            !gcp   (ji,jj,2) = gcpfs   (ji,jj,2)
            !gcp   (ji,jj,3) = gcpfs   (ji,jj,3)
            !gcp   (ji,jj,4) = gcpfs   (ji,jj,4)
            !gcdprc(ji,jj)   = gcdprcfs(ji,jj)
            !gcdmat(ji,jj)   = gcdmatfs(ji,jj)
         ENDDO
      ENDDO

      ! change the precision and the parameters of the solver :
      epseta          = rn_eps
      rn_eps          = rn_eps_hbar
      soreta          = rn_sor
      rn_sor          = rn_sor_hbar
      nmaxeta = nn_nmax
      nn_nmax    = nn_max_hbar
      IF(lwp)WRITE(numout,*) 'precion du solveur trdfs : ', rn_eps
      IF(lwp)WRITE(numout,*) 'max iter du solveur trdfs : ', nn_nmax


!      zmin=MINVAL(zfubar);zmax=MAXVAL(zfubar);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zfubar ",zmin,zmax
!      zmin=MINVAL(zfvbar);zmax=MAXVAL(zfvbar);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zfvbar ",zmin,zmax


      ! 2. Elliptic inversion :
      ! -----------------------------------------------------
      !coriolis
      DO jj = 1,jpj
         DO ji = 1,jpi
            zwu(ji,jj) = zfubar(ji,jj)
            zwv(ji,jj) = zfvbar(ji,jj)
         ENDDO
      ENDDO

      ! 2.1 computation of the divergence of vertical intergation
      ! 2.1.1 Apply boundary conditions to zwu, zwv
      CALL lbc_lnk( zwu , 'U', -1. )
      CALL lbc_lnk( zwv , 'V', -1. )

      ! 2.1.2. Horizontal fluxes
      DO jj = 1, jpj
         DO ji = 1, jpi
               zwu(ji,jj) = e2u(ji,jj) * zwu(ji,jj)
               zwv(ji,jj) = e1v(ji,jj) * zwv(ji,jj)
         END DO
      END DO
!      zmin=MINVAL(zwu);zmax=MAXVAL(zwu);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zwu ",zmin,zmax
!      zmin=MINVAL(zwv);zmax=MAXVAL(zwv);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"zwv ",zmin,zmax

      ! 2.1.3. horizontal divergence
      DO jj = 2, jpjm1
         DO ji = 2, jpim1
            ztmp(ji,jj) =   zwu(ji,jj) - zwu(ji-1,jj)   &
                          + zwv(ji,jj) - zwv(ji  ,jj-1)
         END DO
      END DO

#if defined key_rpne
!share same sol_mat with dynspg_flt,
!let hbar in unit of meter, so that we can use gcx and gcxb in GIOPSv2.0 here
            ztmp(:,:) = ztmp(:,:) * ( z2rdt*z2rdt )
#endif

!      zmin=MINVAL(ztmp);zmax=MAXVAL(ztmp);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"ztmp ",zmin,zmax

      ! 2.1.4. Lateral boundary conditions on zdiv
      CALL lbc_lnk( ztmp ,'T', 1. )
!      zmin=MINVAL(ztmp);zmax=MAXVAL(ztmp);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"ztmp ",zmin,zmax
!      zmin=MINVAL(gcdprc);zmax=MAXVAL(gcdprc);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"gcdprc ",zmin,zmax

      ! 2.2 Right hand side of the elliptic equation and first guess
      DO jj = 2, jpjm1
         DO ji = fs_2, fs_jpim1 ! vector opt
            !   ... r.h.s of the elliptic equation, i.e. the divergence of the 
            !       the vertically integrated contribution to 
            !       the free surface budget
            !zgcb =  - gcb(ji,jj)
            zgcb =  - ztmp(ji,jj)
            gcb(ji,jj) = gcdprc(ji,jj) * zgcb
            !   ... First guess of the free surface term number jn
#if defined key_rpne
            gcx   (ji,jj) = 2. * gcxfs(ji,jj) - gcxbfs(ji,jj)   ! Ansatz for solver
            gcxbfs(ji,jj) =      gcxfs(ji,jj)                   ! keept it for next iteration of hbar
#else
            gcx (ji,jj) =  gcxb(ji,jj)
#endif
         END DO
      END DO
!      zmin=MINVAL(gcb);zmax=MAXVAL(gcb);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"gcb ",zmin,zmax
!      zmin=MINVAL(gcx);zmax=MAXVAL(gcx);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"gcx ",zmin,zmax
      !CALL iom_put("bmask",bmask)
      !CALL iom_put("gcdprcfs",gcdprcfs)
      !CALL iom_put("gcdmatfs",gcdmatfs)
      !CALL iom_put("gcb",gcb)
      !CALL iom_put("gcx",gcx)
      !CALL iom_put("gcr",gcr)
      !CALL iom_put("gcdmat",gcdmat)
      !CALL iom_put("ztmp",ztmp)


      ! 2.3 Relative precision (computation on one processor)
      rnorme =0.
      rnorme = SUM( gcb(:,:) * gcdmat(:,:) * gcb(:,:) )
      IF( lk_mpp )   CALL mpp_sum( rnorme )   ! sum over the global domain
      epsr = rn_eps * rn_eps * rnorme
      ncut = 0
      ! if rnorme is 0, the solution is 0, the solver isn't called
      IF( rnorme == 0.e0 ) THEN
            gcx(:,:) = 0.e0
            res   = 0.e0
            niter = 0
            ncut  = 999
       ENDIF

      ! 2.4 Evaluate the free surface trend
      ! Iterarive solver for the elliptic equation (except IF sol.=0)
      ! (output in gcx with boundary conditions applied)
      kindic = 0
      IF( ncut == 0 ) THEN
         IF    ( nn_solv == 1 ) THEN ; CALL sol_pcg( kindic ) ! diagonal preconditioned conjuguate gradient
         ELSEIF( nn_solv == 2 ) THEN ; CALL sol_sor( kindic ) ! successive-over-relaxation
         ELSE                                               ! e r r o r in nn_solv namelist parameter
            IF(lwp) WRITE(numout,cform_err)
            IF(lwp) WRITE(numout,*) ' diatrdfs : e r r o r, nn_solv = 1 or 2' 
            IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~                not = ', nn_solv
            nstop = nstop + 1
         ENDIF
      ENDIF
      IF(lwp)WRITE(numout,*) 'niter',niter,'res',res,'rn_eps',rn_eps,'kindic',kindic

      ! 2.5 compute the ocean mean sea surface
      ze1e2t = 0.
      zpsmea = 0.
      DO jj = 1, jpj
         DO ji = 1, jpi
            ze1e2t = ze1e2t + &
                 !rbb e1t(ji,jj)*e2t(ji,jj)*tmask(ji,jj,1)
                 e1t(ji,jj)*e2t(ji,jj)*tmask_i(ji,jj)
            zpsmea = zpsmea + gcx(ji,jj)* &
                 !rbb e1t(ji,jj)*e2t(ji,jj)*tmask(ji,jj,1)
                 e1t(ji,jj)*e2t(ji,jj)*tmask_i(ji,jj)
         END DO
      END DO
      IF( lk_mpp )   CALL mpp_sum( ze1e2t )   ! sum over the global domain
      IF( lk_mpp )   CALL mpp_sum( zpsmea )   ! sum over the global domain
      zpsmea = zpsmea / ze1e2t
      
!      zmin=MINVAL(gcx);zmax=MAXVAL(gcx);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"gcx ",zmin,zmax
      ! 2.6 spatial average substracted :
      DO jj = 1, jpj
         DO ji = 1, jpi
            hbar(ji,jj) = (gcx(ji,jj)-zpsmea)*tmask(ji,jj,1)
            !... and backup solution du solveur de free surface trend :
#if defined key_rpne
            gcxfs(ji,jj) = gcx      (ji,jj)     ! keep it for next iteration of hbar
#else
            gcxb (ji,jj) = gcx      (ji,jj)
#endif
         END DO
      END DO
!      zmin=MINVAL(hbar);zmax=MAXVAL(hbar);CALL mpp_min(zmin);CALL mpp_max(zmax);IF(lwp)WRITE(numout,*)"hbar ",zmin,zmax

      !3. Restore the solver solution n case of flt / eta :      
      ! --------------------------------------------------

#if defined key_rpne
       IF(lwp) WRITE(numsol_hbar,9200) kt, niter, res, SQRT(epsr)/eps       ! Solver
       IF(lwp) call flush(numsol_hbar)
9200  FORMAT('it:', i8, ' iter:', i8, ' r: ',e18.10, ' b: ',e18.10 )
#if defined key_dynspg_flt
       DO jj=1,jpj
       DO ji=1,jpi
            gcx   (ji,jj)     = gcxeta   (ji,jj)
            gcxb  (ji,jj)     = gcxbeta  (ji,jj)
       ENDDO
       ENDDO
      rn_eps  = epseta
      rn_sor  = soreta
      nn_nmax = nmaxeta
#endif
#endif

     ! save hbar 
     CALL iom_put("hbar", hbar)

#if defined key_rpne
     IF( lrst_oce )  CALL dia_hbar_rst( kt, 'WRITE' )
#else
     IF( kt==nitend )CALL dia_hbar_rst( kt, 'WRITE' )
#endif

     CALL wrk_dealloc( jpi , jpj      , gcxeta, gcxbeta, gcdprceta, gcdmateta )
     CALL wrk_dealloc( jpi , jpj      , zwu, zwv, ztmp                        )
     CALL wrk_dealloc( jpi , jpj , 4  , gcpeta                                )

    IF( nn_timing == 1 )   CALL timing_stop('dia_trdfs')

   END SUBROUTINE dia_trdfs



   SUBROUTINE dyn_hpa( kt )
     !!----------------------------------------------------------------------
     !!                    ***  ROUTINE dyn_hpa  ***
     !!
     !! ** Purpose :   compute and output the hydrostatic pressure anomaly at some level (bottom)
     !!     and convert it into sea level anomaly (~dynamic height)
     !!----------------------------------------------------------------------
     !
     INTEGER, INTENT( in ) ::   kt    ! ocean time-step index
     !
     INTEGER  ::   ji, jj, jk         ! dummy loop arguments
     REAL(wp) :: zcoef0, zcoef1, zuap ! temporary scalars
     REAL(wp) :: zcoef2, zcoef3, zvap !    "         "
     !
     REAL(wp), POINTER, DIMENSION(:,:)   :: zhpa     ! 2D workspace
     !!--------------------------------------------------------------------
     IF( nn_timing == 1 )   CALL timing_start('dia_hpa')
 
     CALL wrk_alloc( jpi , jpj , zhpa)

     ! 0. Initialisation
     zhpa(:,:) = 0.e0
     zcoef0 = - 0.5
     zuap   = 0.e0
     zvap   = 0.e0

     ! 1. Surface value
     ! ----------------
     DO jj = 2, jpjm1
        DO ji = 2, jpim1   ! vector opt.
           zcoef1 = zcoef0 * (fse3w(ji,jj,1)+sshn(ji,jj) )
           ! hydrostatic pressure gradient
           zhpa(ji,jj) = zcoef1 * rhd(ji,jj,1) * tmask(ji,jj,1)
        END DO
     END DO

     ! 2. interior value (2=<jk=<jpkm1)
     ! -----------------
     DO jk = 2, jpkm1
        DO jj = 2, jpjm1
           DO ji = 2, jpim1   ! vector opt.
              zcoef1 = zcoef0 * fse3w(ji,jj,jk)
              ! hydrostatic pressure gradient
              zhpa(ji,jj) = zhpa(ji,jj)   &
                 &           + zcoef1 * (  rhd(ji,jj,jk)   * tmask(ji,jj,jk)   &
                 &                       + rhd(ji,jj,jk-1) * tmask(ji,jj,jk)  )
           END DO
        END DO
     END DO

     CALL lbc_lnk( zhpa, 'T', 1.)

     CALL iom_put("dynhpa",zhpa)
     !
     CALL wrk_dealloc( jpi , jpj , zhpa)
     !
     IF( nn_timing == 1 )   CALL timing_stop('dia_hpa')
      !

   END SUBROUTINE dyn_hpa


   SUBROUTINE dia_hbar_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_hba_init  ***
      !!                   
      !! ** Purpose :   initialization for HBAR diagnostic computation
      !!----------------------------------------------------------------------
      INTEGER  ::   inum
      INTEGER  ::   ik
      INTEGER  ::   ji, jj, jk  ! dummy loop indices
      REAL(wp) ::   zztmp  
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('dia_hba_init')
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'dia_hba : Barotropic diagnostic & '
      IF(lwp) WRITE(numout,*) 'hydrostatic pressure anomaly in z-coordinate case'
      CALL FLUSH(numout)
      !
      IF( dia_hbar_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'dia_hbar_init : unable to allocate arrays' )
!      IF( sol_oce_alloc()  > 0 )   CALL ctl_stop('agrif sol_oce_alloc: allocation of arrays failed')

#if ! defined key_dynspg_flt
      CALL solver_init( nit000 )
#endif

      ! initialisation of hbar computation
      ! ----------------------------------
      CALL dia_trdfs_ini  

      IF( nn_timing == 1 )   CALL timing_stop('dia_hba_init')
      !
   END SUBROUTINE dia_hbar_init

#if defined key_rpne

   SUBROUTINE dia_hbar_rst( kt, cdrw )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE ts_rst  ***
      !!
      !! ** Purpose : Read or write filtered free surface arrays in restart file
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in) ::   kt     ! ocean time-step
      CHARACTER(len=*), INTENT(in) ::   cdrw   ! "READ"/"WRITE" flag
      integer         :: ierr
      !!----------------------------------------------------------------------
      IF(lwp)WRITE(numout,*)"dia_hbar_rst "
      !
      IF( TRIM(cdrw) == 'READ' ) THEN
         IF( iom_varid( numror, 'gcxfs', ldstop = .FALSE. ) > 0 ) THEN
! Caution : extra-hallow
! gcx and gcxb are defined as: DIMENSION(1-jpr2di:jpi+jpr2di,1-jpr2dj:jpj+jpr2dj)
            IF(lwp)WRITE(numout,*)"dia_hbar_rst read "
            CALL iom_get( numror, jpdom_autoglo, 'gcxfs' , gcxfs (1:jpi,1:jpj) )
            CALL iom_get( numror, jpdom_autoglo, 'gcxbfs', gcxbfs(1:jpi,1:jpj) )
            IF( neuler == 0 )   gcxbfs(:,:) = gcxfs (:,:)
         ELSE
            gcxfs (:,:) = 0.e0
            gcxbfs(:,:) = 0.e0
         ENDIF
         IF( iom_varid( numror, 'HBAR', ldstop = .FALSE. ) > 0 ) THEN
            IF(lwp) WRITE(numout,*) "dia_hbar_rst read:    hbar "
            CALL iom_get( numror, jpdom_autoglo, 'HBAR', hbar (1:jpi,1:jpj) )
         ELSE
            hbar (:,:) = 0.e0
         ENDIF
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN
! Caution : extra-hallow
! gcx and gcxb are defined as: DIMENSION(1-jpr2di:jpi+jpr2di,1-jpr2dj:jpj+jpr2dj)
         CALL iom_rstput( kt, nitrst, numrow, 'gcxfs' , gcxfs (1:jpi,1:jpj) )
         CALL iom_rstput( kt, nitrst, numrow, 'gcxbfs', gcxbfs(1:jpi,1:jpj) )
         CALL iom_rstput( kt, nitrst, numrow, 'HBAR',   hbar  (1:jpi,1:jpj) )
      ENDIF
      !
   END SUBROUTINE dia_hbar_rst

#else

   SUBROUTINE dia_hbar_rst( kt, cdrw )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE ts_rst  ***
      !!
      !! ** Purpose : Read or write filtered free surface arrays in restart file
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in) ::   kt     ! ocean time-step
      CHARACTER(len=*), INTENT(in) ::   cdrw   ! "READ"/"WRITE" flag
      !!----------------------------------------------------------------------
      IF(lwp)WRITE(numout,*)"dia_hbar_rst "
      !
      IF( TRIM(cdrw) == 'READ' ) THEN
         IF( iom_varid( numror, 'gcx', ldstop = .FALSE. ) > 0 ) THEN
! Caution : extra-hallow
! gcx and gcxb are defined as: DIMENSION(1-jpr2di:jpi+jpr2di,1-jpr2dj:jpj+jpr2dj)
            IF(lwp)WRITE(numout,*)"dia_hbar_rst read "
            CALL iom_get( numror, jpdom_autoglo, 'gcx' , gcx (1:jpi,1:jpj) )
            CALL iom_get( numror, jpdom_autoglo, 'gcxb', gcxb(1:jpi,1:jpj) )
            IF( neuler == 0 )   gcxb(:,:) = gcx (:,:)
         ELSE
            gcx (:,:) = 0.e0
            gcxb(:,:) = 0.e0
         ENDIF
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN
! Caution : extra-hallow
! gcx and gcxb are defined as: DIMENSION(1-jpr2di:jpi+jpr2di,1-jpr2dj:jpj+jpr2dj)
         CALL iom_rstput( kt, nitrst, numrow, 'gcx' , gcx (1:jpi,1:jpj) )
         CALL iom_rstput( kt, nitrst, numrow, 'gcxb', gcxb(1:jpi,1:jpj) )
      ENDIF
      !
   END SUBROUTINE dia_hbar_rst

#endif

   SUBROUTINE dia_trdfs_ini 
      !!---------------------------------------------------------------------
      !!
      !!                       ROUTINE diatrdfs_ini
      !!
      !!  Purpose :
      !!  --------
      !!  Initialization of parameter for diatrdfs routine.
      !!
      !!   Method :
      !!   -------
      !!
      !!   Modifications :
      !!   --------------
      !!       original      : 03-07 (N. Ferry)
      !!----------------------------------------------------------------------

      INTEGER :: it, nmaxeta, ios
      REAL(wp) :: epseta
#if defined key_rpne
      INTEGER :: ierr
#endif

      NAMELIST/namhbar/nn_max_hbar, rn_eps_hbar, rn_sor_hbar, ln_temporal_mean, cn_path_hbar, nn_write_hbar,ln_dia_trdfs

      !!----------------------------------------------------------------------
      !
      ! Read Namelist namhbar
      REWIND( numnam_ref )              ! Namelist namhbar in reference namelist
      READ  ( numnam_ref, namhbar, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namhbar in reference namelist', lwp )

      REWIND( numnam_cfg )              !  Namelist namhbar in configuration namelist
      READ  ( numnam_cfg, namhbar, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namhbar in configuration namelist', lwp )
      IF( lwm) WRITE ( numond, namhbar )

      ! Initialize nn_write_hbar if no value in the namelist, default value is nwrite
      IF( nn_write_hbar == 0 ) nn_write_hbar = nwrite
#if defined key_cmc_sam2
      if ( .not. ln_rstart )   il_spinup_diahbar=nn_write    ! in one day-nstep, specified by in_out_manager
#endif

      IF( lwp )THEN
         WRITE(numout,*)
         WRITE(numout,*) 'dia_trdfs : hydrostatic pressure anomaly'
         WRITE(numout,*)'~~~~~~~~~~~                              '
         WRITE(numout,*)' Maximum iteration of the solver = ', nn_max_hbar
         WRITE(numout,*)' precision of the solver         = ', rn_eps_hbar
         WRITE(numout,*)' sor parameter for solsor        = ', rn_sor_hbar
         WRITE(numout,*)' Active the temporal mean        = ', ln_temporal_mean
         WRITE(numout,*)' directory of Stathbar outputs   = ', TRIM(cn_path_hbar)
         WRITE(numout,*)' value of nn_write_hbar          = ', nn_write_hbar
         WRITE(numout,*)' compute hbar online             = ', ln_dia_trdfs
#if defined key_cmc_sam2
         if ( .not. ln_rstart ) &
         WRITE(numout,*)' spin up hbar in cold restart    = ', il_spinup_diahbar
#endif
      ENDIF
      CALL FLUSH(numout)

       it= 1   ! in fact = kt-nit000+1 but we are in the first step
       epseta          = rn_eps
       eps             = rn_eps_hbar
       nmaxeta = nn_nmax
       nn_nmax    = nn_max_hbar
!       IF(lwp)WRITE(numout,*) 'precion du solveur trdfs : ', eps
!       IF(lwp)WRITE(numout,*) 'sor parameter for solsor : ', sor
!       IF(lwp)WRITE(numout,*) 'max iter du solveur trdfs : ', nn_nmax

#if ! defined key_rpne
       CALL fsb_mat
#endif
       eps  = epseta
       nn_nmax = nmaxeta

#if defined key_rpne
       ! we rename all gcx and gcxb as gcxfs and gcxbfs"
       IF(lwp) WRITE(numout,*) " cmc_sam2:: we specify local-variables: gcxfs and gcxbfx for HBAR"
       ALLOCATE( gcxfs(jpi,jpj), gcxbfs(jpi,jpj), STAT= ierr )
#endif
       IF( ln_rstart )THEN
          CALL rst_read_open
          CALL dia_hbar_rst( nit000, 'READ' )
#if defined key_cmc_sam2
       else   
          hbar  (:,:)=0.0
          gcxfs (:,:)=0.0
          gcxbfs(:,:)=0.0
#endif
       ENDIF
#if defined key_rpne
       IF(lwp) CALL ctl_opn( numsol_hbar, 'solver_hbar.stat', 'REPLACE', 'FORMATTED', 'SEQUENTIAL', -1, numout, lwp )
#endif

   END SUBROUTINE dia_trdfs_ini

#else
   !!----------------------------------------------------------------------
   !!   Default option :                                         NO diabar
   !!----------------------------------------------------------------------
   LOGICAL, PUBLIC, PARAMETER :: lk_diahbar = .FALSE.   ! coupled flag
CONTAINS
   SUBROUTINE dia_hbar_init    ! Dummy routine
   END SUBROUTINE dia_hbar_init
   SUBROUTINE dia_hbar( kt )   ! Empty routine
      INTEGER ::   kt
      WRITE(*,*) 'dia_hbar: You should not have seen this print! error?', kt
   END SUBROUTINE dia_hbar
!
   SUBROUTINE dyn_hpa( kt )   ! Empty routine
      INTEGER ::   kt
      WRITE(*,*) 'dyn_hpa: You should not have seen this print! error?', kt
   END SUBROUTINE dyn_hpa
#endif

   !!======================================================================
END MODULE diahbar
