MODULE sbcisf_surge
  ! 2020/10/01
  ! module to hold ISF (ice shelf) data required in different parts of NEMO
  ! curren
  
  !!======================================================================
  !!                       ***  MODULE  sbcisf  ***
  !! Surface module :   surface boundary conditions, ice shelf
  !!======================================================================
  !! History :  1.0  !   2020-10  (P. Wang, O. Huziy)  Original code from 
  !!                    Pengcheng refactored into a separate module
  !!----------------------------------------------------------------------


  USE iom            ! I/O manager library
  USE dom_oce        ! ocean space and time domain

  
  IMPLICIT NONE
  
  REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:), PRIVATE ::  isf_mask_
  
  PUBLIC sbc_isf_get_mask
  
CONTAINS

   SUBROUTINE sbc_isf_get_mask( isf_mask, isf_mask_file )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE sbc_blk_clio  ***
      !!                   
      !! ** Purpose :   get isf mask, read from file only once, to avoid reading 
      !!                the mask more than once
      !!
      !! ** Method  : save a field in a module variable for reuse when needed
      !! ** Action  : 
      !!      
      !!----------------------------------------------------------------------
    
      LOGICAL, INTENT(IN) :: isf_mask_file ! .true. if isf file is used
      REAL(wp), DIMENSION(:,:), ALLOCATABLE, INTENT(INOUT) ::  isf_mask
      
      INTEGER   ::   inum  ! read mask file  
      
      
      IF (.not. isf_mask_file) THEN
        RETURN
      ENDIF

    
      IF ( .not. ALLOCATED(isf_mask_)) THEN
        
        ALLOCATE(isf_mask_(jpi, jpj))
        
        CALL iom_open( 'isf_mask.nc', inum )
        CALL iom_get ( inum, jpdom_data, 'isf_mask', isf_mask_)
        CALL iom_close( inum )
        
        IF(lwp) WRITE(numout,*)' PW: Read isf_mask file'
      ENDIF
      
      IF (.not. ALLOCATED(isf_mask)) THEN
        ALLOCATE(isf_mask(jpi, jpj))
      ENDIF
      
      
      isf_mask(:, :) = isf_mask_(:, :)
    
    END SUBROUTINE sbc_isf_get_mask

END MODULE sbcisf_surge
