MODULE sbctide
   !!======================================================================
   !!                       ***  MODULE  sbctide  ***
   !! Initialization of tidal forcing
   !!======================================================================
   !! History :  9.0  !  2007  (O. Le Galloudec)  Original code
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain
   USE phycst
   USE daymod
   USE dynspg_oce
   USE tideini
   !
   USE iom
   USE in_out_manager  ! I/O units
   USE ioipsl          ! NetCDF IPSL library
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)

   IMPLICIT NONE
   PUBLIC

#if defined key_rpne
   !Modifications from J Chanut
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:) ::  amp_pot, phi_pot
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:) ::  amp_load, phi_load
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)   ::  pot_astro
   LOGICAL, PRIVATE, SAVE :: first_time_in_sbctide = .true. ! private to this module
#if defined key_surge
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)   :: u_tpxo, u_tpxo2, v_tpxo, v_tpxo2, pot_mask, salf
#endif

#else 
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)   ::  pot_astro
#endif


#if defined key_tide
   !!----------------------------------------------------------------------
   !!   'key_tide' :                                        tidal potential
   !!----------------------------------------------------------------------
   !!   sbc_tide            : 
   !!   tide_init_potential :
   !!----------------------------------------------------------------------

   LOGICAL, PUBLIC, PARAMETER ::   lk_tide  = .TRUE.
   !!!TK 20180206 
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:) ::  umtoa, umtop,vmtoa,vmtop

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.5 , NEMO Consortium (2013)
   !! $Id: sbctide.F90 5215 2015-04-15 16:11:56Z nicolasmartin $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE sbc_tide( kt )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE sbc_tide  ***
      !!----------------------------------------------------------------------      
      INTEGER, INTENT( in ) ::   kt     ! ocean time-step
      INTEGER               ::   jk,inum     ! dummy loop index
      !!----------------------------------------------------------------------

#if defined key_rpne
      IF (first_time_in_sbctide) THEN
        nstep_tide_upt = NINT(rday/rdt)
        IF ((nitend-nit000+1)*rdt .lt. rday) THEN
          nstep_tide_upt = nitend-nit000+1
          if (lwp) write(numout,*) '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
          if (lwp) write(numout,*) '!!WARNING: Run for less than 24h, RESTART WILL NOT VALIDATE'
          if (lwp) write(numout,*) '!!         WITH LONGER RUNS BECAUSE OF TIDE PARAM UPDATES'
          if (lwp) write(numout,*) '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
        ENDIF
        if (lwp) write(numout,*) 'Will update tide pars at each steps = ', nstep_tide_upt
        first_time_in_sbctide = .false.
      ENDIF
!froy - now working for restart runs of less than 24h
      IF ( mod(kt - nit000, nstep_tide_upt) == 0 ) THEN
#else
      IF( nsec_day == NINT(0.5_wp * rdttra(1)) ) THEN      ! start a new day
#endif         !
         IF( kt == nit000 ) THEN
            ALLOCATE( amp_pot(jpi,jpj,nb_harmo),                      &
               &      phi_pot(jpi,jpj,nb_harmo), pot_astro(jpi,jpj)   )
   !!!TK 20180206 
            ALLOCATE( umtoa(jpi,jpj,nb_harmo),vmtoa(jpi,jpj,nb_harmo), vmtop(jpi,jpj,nb_harmo),  &
               &      umtop(jpi,jpj,nb_harmo), u_tpxo(jpi,jpj) ,v_tpxo(jpi,jpj) ,&
               &      pot_mask(jpi,jpj), salf(jpi,jpj),u_tpxo2(jpi,jpj),v_tpxo2(jpi,jpj))
          u_tpxo (:,:) = 0.e0
          u_tpxo2 (:,:) = 0.e0
          v_tpxo (:,:) = 0.e0
          v_tpxo2 (:,:) = 0.e0
          pot_mask (:,:) = 0.e0
          umtoa (:,:,:) = 0.e0
          umtop (:,:,:) = 0.e0
          vmtoa (:,:,:) = 0.e0
          vmtop (:,:,:) = 0.e0
          CALL iom_open('SAL.nc',inum)
          CALL iom_get (inum, jpdom_data, 'salf',salf) ! 
          CALL iom_close(inum)

          CALL iom_open('TPXO8_uv',inum)
          CALL iom_get (inum, jpdom_data, 'uM2Amp',umtoa(:,:,1)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uM2Phi',umtop(:,:,1)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vM2Amp',vmtoa(:,:,1)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vM2Phi',vmtop(:,:,1)) ! bfrcoef2d is used as tmp array
              IF (nb_harmo .gt. 1) THEN
          CALL iom_get (inum, jpdom_data, 'uS2Amp',umtoa(:,:,2)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uS2Phi',umtop(:,:,2)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vS2Amp',vmtoa(:,:,2)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vS2Phi',vmtop(:,:,2)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uO1Amp',umtoa(:,:,3)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uO1Phi',umtop(:,:,3)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vO1Amp',vmtoa(:,:,3)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vO1Phi',vmtop(:,:,3)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uK1Amp',umtoa(:,:,4)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uK1Phi',umtop(:,:,4)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vK1Amp',vmtoa(:,:,4)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vK1Phi',vmtop(:,:,4)) ! bfrcoef2d is used as tmp array
              ENDIF
              IF (nb_harmo .gt. 5) THEN
          CALL iom_get (inum, jpdom_data, 'uQ1Amp',umtoa(:,:,5)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uQ1Phi',umtop(:,:,5)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vQ1Amp',vmtoa(:,:,5)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vQ1Phi',vmtop(:,:,5)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uP1Amp',umtoa(:,:,6)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uP1Phi',umtop(:,:,6)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vP1Amp',vmtoa(:,:,6)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vP1Phi',vmtop(:,:,6)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uN2Amp',umtoa(:,:,7)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uN2Phi',umtop(:,:,7)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vN2Amp',vmtoa(:,:,7)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vN2Phi',vmtop(:,:,7)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uK2Amp',umtoa(:,:,8)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'uK2Phi',umtop(:,:,8)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vK2Amp',vmtoa(:,:,8)) ! bfrcoef2d is used as tmp array
          CALL iom_get (inum, jpdom_data, 'vK2Phi',vmtop(:,:,8)) ! bfrcoef2d is used as tmp array
              ENDIF
          !CALL iom_get (inum, jpdom_data, 'Msk',pot_mask) ! bfrcoef2d is used as tmp array

          CALL iom_close(inum)
          CALL iom_open('TPXO8_Mask',inum)
          CALL iom_get (inum, jpdom_data, 'Msk',pot_mask) ! bfrcoef2d is used as tmp array
          CALL iom_close(inum)
!!!          WRITE(numout,*) ' TPXO8 nudging implemented by TK '
        ENDIF
!!!        IF (ln_read_tide_elev    ) CALL tide_init_elevation
!!!        IF (ln_read_tide_trans   ) CALL tide_init_transport
!!!      ENDIF
         amp_pot(:,:,:) = 0._wp
         phi_pot(:,:,:) = 0._wp
         pot_astro(:,:) = 0._wp
         CALL tide_harmo( omega_tide, v0tide, utide, ftide, ntide, nb_harmo )
         !
         kt_tide = kt
         !
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'sbc_tide : Update of the components and (re)Init. the potential at kt=', kt
            WRITE(numout,*) '~~~~~~~~ '
            DO jk = 1, nb_harmo
               WRITE(numout,*) Wave(ntide(jk))%cname_tide, utide(jk), ftide(jk), v0tide(jk), omega_tide(jk)
            END DO
         ENDIF
         !
         IF( ln_tide_pot )   CALL tide_init_potential
         !
      ENDIF
      !
   END SUBROUTINE sbc_tide


   SUBROUTINE tide_init_potential
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE tide_init_potential  ***
      !!----------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk   ! dummy loop indices
      REAL(wp) ::   zcons, ztmp1, ztmp2, zlat, zlon, ztmp, zamp, zcs   ! local scalar
      !!----------------------------------------------------------------------

      DO jk = 1, nb_harmo
         zcons = 0.7_wp * Wave(ntide(jk))%equitide * ftide(jk)
         DO ji = 1, jpi
            DO jj = 1, jpj
               ztmp1 =  amp_pot(ji,jj,jk) * COS( phi_pot(ji,jj,jk) )
               ztmp2 = -amp_pot(ji,jj,jk) * SIN( phi_pot(ji,jj,jk) )
               zlat = gphit(ji,jj)*rad !! latitude en radian
               zlon = glamt(ji,jj)*rad !! longitude en radian
               ztmp = v0tide(jk) + utide(jk) + Wave(ntide(jk))%nutide * zlon
               ! le potentiel est composé des effets des astres:
               IF    ( Wave(ntide(jk))%nutide == 1 )  THEN  ;  zcs = zcons * SIN( 2._wp*zlat )
               ELSEIF( Wave(ntide(jk))%nutide == 2 )  THEN  ;  zcs = zcons * COS( zlat )**2
               ELSE                                         ;  zcs = 0._wp
               ENDIF
               ztmp1 = ztmp1 + zcs * COS( ztmp )
               ztmp2 = ztmp2 - zcs * SIN( ztmp )
               zamp = SQRT( ztmp1*ztmp1 + ztmp2*ztmp2 )
               amp_pot(ji,jj,jk) = zamp
               phi_pot(ji,jj,jk) = ATAN2( -ztmp2 / MAX( 1.e-10_wp , zamp ) ,   &
                  &                        ztmp1 / MAX( 1.e-10_wp,  zamp )   )
            END DO
         END DO
      END DO
      !
   END SUBROUTINE tide_init_potential

#else
  !!----------------------------------------------------------------------
  !!   Default case :   Empty module
  !!----------------------------------------------------------------------
  LOGICAL, PUBLIC, PARAMETER ::   lk_tide = .FALSE.
CONTAINS
  SUBROUTINE sbc_tide( kt )      ! Empty routine
    INTEGER         , INTENT(in) ::   kt         ! ocean time-step
    WRITE(*,*) 'sbc_tide: You should not have seen this print! error?', kt
  END SUBROUTINE sbc_tide
#endif

  !!======================================================================
END MODULE sbctide
