MODULE semilag_adv

!FRoy OK ???? (Check if my interpretation of gdept is OK below
! (new names gdept_0 ==> gdept_1d , gdept ==> gdept_0)
! (new names gdepw_0 ==> gdepw_1d , gdepw ==> gdepw_0)
! (new names e3t_0 ==> e3t_1d , e3t ==> e3t_0)

   USE par_oce          ! ocean parameters
   USE dom_oce          ! ocean space and time domain
   USE oce              ! ocean dynamics and tracers variables
   USE in_out_manager   ! I/O manager
   USE lib_mpp          ! distributed memory computing library

   IMPLICIT NONE
   PRIVATE

   PUBLIC traject_sld, upwnd3_new , upwnd1, expstep

   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, &
      DIMENSION(:,:,:) ::  xd, yd, zd      ! Trajectories, stored as displacement-per-cell
 
   INTEGER, PUBLIC :: trajectory_kt = -1   ! Timestep for which trajectories are valid

   LOGICAL, PUBLIC :: super_noisy

   INTEGER, PUBLIC :: Tsl_iter=80 ! max iteration in computing trajectories
   INTEGER, PUBLIC :: Tsl_cut =40 ! Iteration at which termination with a confirmed in-domain foot is acceptable
      
      ! Paramaterized diagnostic indices
   INTEGER, PUBLIC :: print_tdiag = 0, print_vdiag=0, print_diag = 0
   INTEGER, PUBLIC, PARAMETER :: diag_i = 18, diag_j = 203, diag_k = 49

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"

CONTAINS

!copyright (C) 2001  MSC-RPN COMM  %%%MC2%%%
!***s/r slag3d -- Upstream interpolation: Semi-Lagrangian Scheme
!*

!---------------------------------------------------------------------
!
! compute upstream trajectories
!
!---------------------------------------------------------------------

      subroutine traject_sld( kt, dt_here )
      implicit none
! arguments
      INTEGER, INTENT(in) :: kt
      REAL(wp) :: dt_here ! Timestep to use

!*
!*AUTHORs    M. Desgagne & C. Girard
!*           P. Pellerin: Solid Object
!*
!*OBJECT
!*
!*    *******************************************************************
!*    *                                                                 *
!*    *               ANDRE ROBERT  SEMI-LAGRANGIAN SCHEME              *
!*    *                                                                 *
!*    *                            CALCULATES                           *
!*    *                                                                 *
!*    *                       TRAJECTORIES: x , y , z                   *
!*    *                                      d   d   d                  *
!*    *                                                                 *
!*    *    1. MID-POINT trajectories                                    *
!*    *                                                                 *
!*    *               ___________________________________               *
!*    *              |                                   |              *
!*    *              |                                   |              *
!*    *              | dr/2  =   [ x (r), y (r), z (r) ] |              *
!*    *              |              d      d      d      |              *
!*    *              |___________________________________|              *
!*    *                                                                 *
!*    *                                                                 *
!*    *             FUNCTIONS OF UPSTREAM WIND ux , vy , wz             *
!*    *                                          d    d    d            *
!*    *                                                                 *
!*    *        ux (r)  = u0 (r-dr/2)                                    *
!*    *          d         d                     _____x                 *
!*    *                                             _y                  *
!*    *                       where     u0  =  [ u0*s  ] * dt/dx        *
!*    *                                   d                             *
!*    *                                                                 *
!*    *                                                                 *
!*    *        vy (r)  = v0 (r-dr/2)                                    *
!*    *          d         d                     _____y                 *
!*    *                                             _x                  *
!*    *                       where     v0  =  [ v0*s  ] * dt/dy        *
!*    *                                   d                             *
!*    *                                                                 *
!*    *                                                                 *
!*    *        wz (r)  = w0 (r-dr/2)                                    *
!*    *          d         d                                            *
!*    *                                           __z                   *
!*    *                       where     w0  =  [  w0   ] * dt/dz        *
!*    *                                   d                             *
!*    *                                                                 *
!*    *      FOR STRAIGTH TRAJECTORIES: x  = ux , y  = vy , z  = wz     *
!*    *                                  d     d   d     d   d     d    *
!*    *                                                                 *
!*    *      FOR GREAT CIRCLE TRAJECTORIES    cf    gc_itraj            *
!*    *                                                                 *
!*    *                                                                 *
!*    *    2. DOUBLING the DISPLACEMENTS                                * 
!*    *                                                                 * 
!*    *               ___________________________________               *
!*    *              |                                   |              *
!*    *              |                                   |              *
!*    *              |  [ x (r), y (r), z (r) ]  =  dr   |              *
!*    *              |     d      d      d               |              *
!*    *              |___________________________________|              *
!*    *   
!*    *                                                                 * 
!*    *      FOR GREAT CIRCLE TRAJECTORIES    cf    gc_ddisp            * 
!*    *                                                                 * 
!*    ******************************************************************* 
!* 
!*ARGUMENTS 
!*     _________________________________________________________________ 
!*    |         |                                                 |     |
!*    |  NAME   | DESCRIPTION                                     | I/O |
!*    |---------|-------------------------------------------------|-----|
!*    |         |                                                 |     |
!*    |    xd   | lagrangian displacement along X                 |  o  |
!*    |    yd   | lagrangian displacement along  Y                |  o  |
!*    |    zd   | lagrangian displacement along   Z               |  o  |
!*    |         |                                                 |     |
!*    |    w0   | vertical motion in Z-coord at time t    (work)  |     |
!*    |    wm   | vertical motion in Z-coord at time t-dt (work)  |     |
!*    |         |                                                 |     |
!*    |   u0d   | velocity at departure point along X     (work)  |     |
!*    |   v0d   | velocity at departure point along  Y    (work)  |     |
!*    |   w0d   | velocity at departure point along   Z   (work)  |     |
!*    |         |                                                 |     |
!*    |   uxd   | upstream velocity along X               (work)  |     |
!*    |   vyd   | upstream velocity along  Y              (work)  |     |
!*    |   wzd   | upstream velocity along   Z             (work)  |     |
!*    |         |                                                 |     |
!*    |_________|_________________________________________________|_____|
!*
      REAL(wp), dimension(jpi,jpj,jpk) :: u0d,v0d,w0d, &
                                          uo, vo, wo, &
                                          zu0,zum,zv0,zvm,zw0,zwm, &
                                          fxx,fyy,fzz 

      REAL(wp) :: dxn, dyn, dzn, vol, vil, uop, vop, wop, uip, vip, wip, tmag, ee
      REAL(wp) :: u0p, v0p, w0p, pscale
      REAL(wp) :: uoh, voh, woh, uih, vih, wih, uimask, vimask, wimask
      REAL(wp) :: u0new, v0new, w0new
      INTEGER  :: ji, jj, jk, id, jd, iff, jf, iter, kp1, km1
      INTEGER :: maxiter, maxji, maxjj, maxjk, itertotal, numcells
!*
      xd = 0; yd = 0; zd = 0;
      uo = 0; vo = 0; wo = 0;
      u0d = 0.0_wp
      v0d = 0.0_wp
      w0d = 0.0_wp
      maxiter = 0
      itertotal = 0
      numcells = 0
      maxji = 0
      maxjj = 0
!*
!*    1A) Compute vertical motion in coordinate space
!*

      ! csubich: For creation of a useful advection trend, we want to advect the ``before'' velocities
      ! to the ``after'' timestep.  Unlike a conventional semi-Lagrangian method where fluid parcels
      ! keep their identity within this step, we will compute a time-difference as a phantom trend.
      
      ! This is compatible with the very-small-CFL limit of the traditional flux-form advection, and
      ! implementing it this way means that the remainder of the NEMO tracer-manipulating code can
      ! be indifferent to whether advection is done via the semi-Lagrangian method or a finite-volume
      ! method.
      
      ! Compute an initial displacement guess by scaling velocities by the timestep.

      ! This guess accounts for staggering via a simple average of neighbouring points,
      ! since a t-point is located between u-points and v-points.  This averaging
      ! deliberately does not take the mask into account, since solid boundaries
      ! that have zero perpendicular velocity exist at the velocity points.  That is to
      ! say, if the t-cell is adjacent to a boundary in the first dimension, the best
      ! first-guess for u-velocity is 1/2 that of the "right" u-point.

      uo(2:jpi,:,:) = 0.5*(un(1:jpi-1,:,:)+un(2:jpi,:,:))
      vo(:,2:jpj,:) = 0.5*(vn(:,1:jpj-1,:)+vn(:,2:jpj,:))
      
      ! Similarly, the first guess on w is taken via averaging the up-and-down neighbours.
      ! The grid is aligned vertically such that w(:,:,1) is the surface, with t(:,:,1) being
      ! located between w(:,:,1) and w(:,:,2).  The bottom index is masked by construction of
      ! the grid.

      wo(:,:,1:jpk-1) = 0.5*(wn(:,:,1:jpk-1)+wn(:,:,2:jpk))
      u0d = uo*dt_here
      v0d = vo*dt_here
      w0d = wo*dt_here
      do jk = 1,jpkm1
         xd(2:jpi,:,jk) = u0d(2:jpi,:,jk)/e1t(2:jpi,:)
         yd(:,2:jpj,jk) = v0d(:,2:jpj,jk)/e2t(:,2:jpj)
      end do

      ! Limit initial trajectories to 0.5 cell-lengths in lateral directions, to prevent
      ! first guesses from "skipping past" boundaries
      do ji=1,jpi
         do jj=1,jpj
            do jk=1,jpk
               if (abs(xd(ji,jj,jk)) > 0.5 .or. abs(yd(ji,jj,jk)) > 0.5) then
                  ee = sqrt(xd(ji,jj,jk)**2 + yd(ji,jj,jk)**2)
                  u0d(ji,jj,jk) = 0.5*u0d(ji,jj,jk)/ee
                  v0d(ji,jj,jk) = 0.5*v0d(ji,jj,jk)/ee
                  w0d(ji,jj,jk) = 0.5*w0d(ji,jj,jk)/ee
                  xd(ji,jj,jk) = u0d(ji,jj,jk)/e1t(ji,jj)
                  yd(ji,jj,jk) = v0d(ji,jj,jk)/e2t(ji,jj)
               end if
            end do
         end do
      end do
      ! Note of caution: the z-coordinate is ordered such that positive velocity means movement
      ! towards the surface and lower corodinate values. 

      ! This initial displacement also accounts for partial cells, by constructing an offset based
      ! on the cell's actual depth (gdept_0(i,j,k)) versus the background cell depth (gdept_1d(k)).
      ! These resulting displacements are always with reference to the background cell structure;
      ! this transformation is undone during the vertical interpolation step to again refer to the
      ! local cell structure.
      do ji = 1,jpi
        do jj = 1,jpj 
            zd(ji,jj,1:jpk-1) = (-w0d(ji,jj,1:jpk-1) + gdept_1d(1:jpk-1) - gdept_0(ji,jj,1:jpk-1))/ e3t_1d(1:jpk-1)
         end do
      end do
      if (print_tdiag /= 0) then
         write (numout,*) 'Initial tracer velocities'
         write (numout,'(3E12.4)') uo(diag_i,diag_j,diag_k), vo(diag_i,diag_j,diag_k), wo(diag_i,diag_j,diag_k)
         write (numout,*) 'Initial tracer displacements'
         write (numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
         write (numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
         print_diag = 0
      endif

      ! Now, compute trajectories on a per-cell basis
      do jk = 1,jpk-1
         do jj = 2,jpj-1
            do ji = 2,jpi-1
               ! If this cell is masked, then the trajectories are irrelevant
               if (tmask(ji,jj,jk) == 0.0) cycle

               numcells = numcells + 1

               ! Otherwise, interpolate to find the "foot" velocity and adjust the candidate trajectory
               ! until convergence

               ! Save origin velocities
               uoh = uo(ji,jj,jk)
               voh = vo(ji,jj,jk)
               woh = wo(ji,jj,jk)

               do iter = 1, Tsl_iter

                  ! Directly upwind the tracer point onto the staggered u-grid.  Here, we
                  ! make use of the "offset" functionality.

                  ! In the NEMO grid used, the tracers are located between velocity
                  ! points, such that the grid goes:
                  !    t1 u1 t2 u2 t3 u3 [etc]
                  ! So, to find out what we expect u to be at the t2 point, for example,
                  ! we need to "really" ask for u(1.5), on its native grid.  This introduces
                  ! a half-cell offset to the negative, represented here by a positive
                  ! parameter value, since interpolation is done at the point (ii-xd-offset_x).
         
                  if (print_tdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                     !super_noisy = .true.
                  endif
                  call upwnd1 ( uih, uimask, un, umask, ji, xd(ji,jj,jk), jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.5, 0.0, 0.0, &
                  &                'zero_mask', 'noflux_between', 'noflux_between')
                  
                  ! Upwinding of V works similarly
                  call upwnd1 ( vih, vimask, vn, vmask, ji, xd(ji,jj,jk), jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.0, 0.5, 0.0, &
                  &                 'noflux_between','zero_mask', 'noflux_between')
                  
                  ! For W, the offset changes.  Now, t2 lies between w2 and w3, so
                  ! we're asking from the plus-half-point rather than the minus-half-
                  ! point.
                  !if (print_tdiag /= 0) print_diag = 1
                  call upwnd1 ( wih, wimask, wn, tmask, ji, xd(ji,jj,jk), jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.0, 0.0, -0.5, &
                  &                 'noflux_between', 'noflux_between', 'zero_mask')
                  ! Implementation reminder: w-velocity is "negative" in grid terms, compared
                  ! to u-velocity and v-velocity; upwards w points to *lower* k indices

                  if (uimask == 0 .or. vimask == 0 .or. wimask == 0) then
                     uih = 0
                     vih = 0
                     wih = 0
                  endif

                  super_noisy = .false.

                  if (iter > Tsl_cut .and. uimask == 1.0 .and. vimask == 1.0 .and. wimask == 1.0) then
                     if (print_tdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then 
                        write(numout,*) 'Ending with confirmed in-domain point @ iter', iter
                     endif
                     exit
                  endif

                  if (print_tdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                     write (numout,*) 'Revised tracer velocities'
                     write (numout,'(3E12.4)') uih, vih, wih
                     print_diag = 0
                  endif

                  ! Preserve prior physical displacements for delta calculation 
                  u0p = u0d(ji,jj,jk); v0p = v0d(ji,jj,jk); w0p = w0d(ji,jj,jk);

                  call expstep(dt_here, uoh, voh, woh, uih, vih, wih, u0p, v0p, w0p, u0new, v0new, w0new)

!                  if (iter > (4*Tsl_iter/5)) then
!                     u0d(ji,jj,jk) = 0.9375*u0p + 0.0625*u0new
!                     v0d(ji,jj,jk) = 0.9375*v0p + 0.0625*v0new
!                     w0d(ji,jj,jk) = 0.9375*w0p + 0.0625*w0new
                  if (iter > (3*Tsl_cut/4)) then
                     u0d(ji,jj,jk) = 0.875*u0p + 0.125*u0new
                     v0d(ji,jj,jk) = 0.875*v0p + 0.125*v0new
                     w0d(ji,jj,jk) = 0.875*w0p + 0.125*w0new
                  elseif (iter > (2*Tsl_cut/4)) then
                     u0d(ji,jj,jk) = 0.75*u0p + 0.25*u0new
                     v0d(ji,jj,jk) = 0.75*v0p + 0.25*v0new
                     w0d(ji,jj,jk) = 0.75*w0p + 0.25*w0new
                  elseif (iter > (Tsl_cut/4)) then
                     u0d(ji,jj,jk) = 0.5*u0p + 0.5*u0new
                     v0d(ji,jj,jk) = 0.5*v0p + 0.5*v0new
                     w0d(ji,jj,jk) = 0.5*w0p + 0.5*w0new
                  else
                     u0d(ji,jj,jk) = u0new
                     v0d(ji,jj,jk) = v0new
                     w0d(ji,jj,jk) = w0new
                  endif


                  ! Recalculate cell displacements
                  xd(ji,jj,jk) = u0d(ji,jj,jk)/e1t(ji,jj)
                  yd(ji,jj,jk) = v0d(ji,jj,jk)/e2t(ji,jj)
                  zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)

                  if (print_tdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                     write (numout,*) 'Revised tracer displacements', iter
                     write (numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
                     write (numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
                     write (numout,*) 'Delta'
                     write (numout,'(1E12.4)') ((u0d(diag_i,diag_j,diag_k)-u0p)**2 + &
                                                (v0d(diag_i,diag_j,diag_k)-v0p)**2 + &
                                                (w0d(diag_i,diag_j,diag_k)-w0p)**2)**0.5
                     print_diag = 0
                  endif

                  if ((u0d(ji,jj,jk)-u0p)**2/e1v(ji,jj)**2 + &
                      (v0d(ji,jj,jk)-v0p)**2/e2v(ji,jj)**2 + &
                      (w0d(ji,jj,jk)-w0p)**2/e3t_1d(jk)**2 < (0.001)**2) then
                      u0d(ji,jj,jk) = u0p
                      v0d(ji,jj,jk) = v0p
                      w0d(ji,jj,jk) = w0p
                      xd(ji,jj,jk) = u0d(ji,jj,jk)/e1t(ji,jj)
                      yd(ji,jj,jk) = v0d(ji,jj,jk)/e2t(ji,jj)
                      zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)
                     exit
                  endif
               end do
               if (iter > maxiter) then
                  maxiter = iter
                  maxji = ji
                  maxjj = jj
                  maxjk = jk
               endif

               itertotal = itertotal + iter
            end do
         end do
      ENDDO ! end of interation loop

      if (.true. .or. print_tdiag /= 0) then
         write (numout,*) ' Maximum of ', maxiter, ' iterations at cell'
         write (numout,*) maxji, maxjj, maxjk
         write (numout,*) itertotal, 'trajectory iterations in', numcells
      endif

      end subroutine traject_sld


! Author: C. Subich
! upwnd3_sld:
!   Calculates, at each output grid cell, values given by tricubic
!   interpolation of the input field at displacements (xd,yd,zd).
!
! Input parameters:
!   f_in: Field to be interpolated
!   f_mask: In/out of domain mask for f
!   (xd,yd,zd): Displacements in x, y, and z directions
!   (offset_x,offset_y,offset_z): Fixed offsets in each dimension
!         to apply to the given displacements; this allows 
!         interpolating between different staggered grids without 
!         otherwise averaging, such as finding velocities at t-points
!   (bdy_x,bdy_y,bdy_z): Boundary condition type, by dimension:
!      'zero_mask': Field is zero *at* masked points, typical
!         for fluid velocities along their respective directions
!      'noflux_between': Field has zero derivative *between* 
!         a masked point and the first interior point, for 
!         staggered Neumann boundaries
!      'zero_between': Field is zero between a masked point and
!         the first interior point, for staggered Dirichlet boundaries
! Output parameter:
!   f_out: Interpolated field

      subroutine upwnd3_new( f_out, f_in, f_mask, xd, yd, zd, &
      &                      offset_x, offset_y, offset_z,    &
      &                      bdy_x, bdy_y, bdy_z, do_limit, toler )

      implicit none

      ! Arguments

      real(wp), intent(in), dimension(jpi, jpj, jpk) :: f_in, f_mask, xd, yd, zd
      real(wp), intent(out), dimension(jpi, jpj, jpk) :: f_out

      character(len=*), intent(in) :: bdy_x, bdy_y, bdy_z

      real(wp), intent(in) :: offset_x, offset_y, offset_z, toler

      logical, intent(in) :: do_limit ! Whether interpolation should limit to prevent overshoots

      ! Local variables

      ! In principle, each output point is based on a weighted sum of up to
      ! 4^3 = 64 original points; conversely this means that each on-grid
      ! point of the source array will be accessed on average 64 times.

      ! We can save some of this effort by precomputing derivatives along one
      ! dimension.  This reduces work for the Hermite-spline formulation along
      ! that dimension, which needs gridpoint values and derivatives at the
      ! ends of an interval.

      ! For a few reasons, we'll do this precomputation in z:

      !  *) First, we want to ensure not just a high-ish-order interpolation in
      !     z, but also that the interpolating function is C(1) at the gridpoints.
      !     This is key to ensuring that small vertical motions don't upset 
      !     conservation or cause anomalous transport.
      !  *) Second, the z-direction is special because the grid stretches in this
      !     dimension.  The e1t and e2t scale factors are almost constant, but 
      !     e3t varies.  This means that "interpolate by grid index" is less
      !     useful in z, so handling this specially allows for general "grid index"
      !     interpolation in x and y
      !  *) Third, the z-direction has partial cells near the boundary (key_zps).
      !     This requires a significant amount of special-case code to handle.

      real(wp), dimension(jpi, jpj, jpk) :: f_z

      ! Working array to hold values for 2D interpolation, at points f(i,j,z-zd)
      ! that are probably off-grid with respect to z
      real(wp), dimension(4,4) :: f2d_xy, f2dmask_xy

      ! Working array to hold values for 1D interpolation, at points f(x-xd,j,z-zd)
      real(wp), dimension(4) :: f1d_y, f1dmask_y

      ! Interpolation values: value and derivative at the ''left'' and ''right'' points
      real(wp) :: val_l, val_r, grad_l, grad_r

      ! Helper variables for coefficients, to help correct rounding error
      real(wp) :: coefl, coefr

      real(wp) :: out_mask

      ! Loop variables
      integer :: ji, jj, jk, jl, jm
      
      ! Derived index variables
      integer :: ix, iy, iz, iz_g
      ! Derived local offsets
      real(wp) :: lox, loy, loz, loz_g
      real(wp) :: source_z

      ! Temporaries for second-derivative calculation
      real(wp) :: dz1, dz2, f0, f1, f2

      integer noisy

      noisy = 0

      f_z(:,:,:) = 0.0_wp

      ! Check that we have been given valid boundary conditions

      if (bdy_x /= 'zero_mask' .and. bdy_x /= 'zero_between' &
    &      .and. bdy_x /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_x not valid', bdy_x)
      end if
      if (bdy_y /= 'zero_mask' .and. bdy_y /= 'zero_between' &
    &      .and. bdy_y /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_y not valid', bdy_y)
      end if
      if (bdy_z /= 'zero_mask' .and. bdy_z /= 'zero_between' &
    &      .and. bdy_z /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_z not valid', bdy_z)
      end if

      if (print_diag > 0) then
         write(numout,*) 'Called upwnd3_new with diagnostic point'
         write(numout,*) diag_i, diag_j, diag_k
      end if

         
      ! Compute first derivtive in z
      do jk=1,jpk ! Avoid top layer
         do jj=1,jpj
            do ji=1,jpi
               ! If the point is masked, then we do not want to precompute
               ! the first derivative -- we'll use one-sided stencils as
               ! necessary in the interpolation itself
               if (f_mask(ji,jj,jk) == 0.0) then
                  if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                     write(numout,*) 'Point is masked'
                  end if
                  cycle
               end if

               if (jk == 1) then
                  ! For the top layer, assume a no-flux boundary condition and create a corresponding
                  ! ghost point for the purposes of derivative computation
                  f0 = f_in(ji,jj,jk)
                  dz1 = 2*gdept_0(ji,jj,jk)
               else
                  f0 = f_in(ji,jj,jk-1)
                  dz1 = gdept_0(ji,jj,jk) - gdept_0(ji,jj,jk-1)
               endif
               f1 = f_in(ji,jj,jk)
               dz2 = gdept_0(ji,jj,jk+1) - gdept_0(ji,jj,jk)


               ! If the next-lower point is masked, we'll need to work with
               ! the boundary condition. (We don't need to check points above
               ! by construction of the NEMO grid: no underwater caves)
               if (f_mask(ji,jj,jk+1) == 0.0) then
                  if (bdy_z == 'noflux_between') then
                     ! No flux at the halfway point between cells means that
                     ! to locally second-order the first in-boundary point
                     ! has a phantom value equal to this point.
                     f2 = f1
                  elseif (bdy_z == 'zero_between') then
                     ! Taking on zero at the halfway point between cells means
                     ! that locally the ghost point takes on the negative of
                     ! this point
                     f2 = -f1
                  elseif (bdy_z == 'zero_mask') then
                     ! Taking on zero at masked values means that f_in(masked)=0,
                     ! obviously.
                     f2 = 0
                  end if
               else ! f_mask(ji,jj,jk) /= 0
                  ! No ghost point necessary
                  f2 = f_in(ji,jj,jk+1)
               end if


               !f_zz(ji,jj,jk) = 2.0/(dz1*dz2*(dz1+dz2))*((f0-f1)*dz2 + (f2-f1)*dz1)
               f_z(ji,jj,jk) = -f0*dz2/(dz1*(dz1+dz2)) +  &
                                f1*(dz2-dz1)/(dz1*dz2) +  &
                                f2*dz1/(dz2*(dz1+dz2))
               ! F. Roy (Marker to find this piece of code rapidly)
               ! Mild limiting of f_z for interior maxima
               ! NOTE: this step is not described in the methods paper, which is written
               ! based on the NEMO3.1 implementation.
               !!! FIXME: THIS CODE IS COMMENTED OUT.  DECIDE IF VERTICAL LIMITING IS DESIRABLE
               !!!        AND KEEP IT, OR UNDESIRABLE AND DELETE IT
               if (jk > 1 .and. & ! f_mask(ji,jj,jk+1) > 0.0 .and. &  -- extend limiting to bottom level
                   (((f1 <= f0) .and. (f1 <= f2)) .or. & 
                    ((f1 >= f0) .and. (f1 >= f2)))) then
                  ! Limit so that if f1 is an interior minimum, the computed first drivative
                  ! has a magnitude no greater than that of the smallest one-sided derivative
                  if (abs(f_z(ji,jj,jk)) > min(abs((f0-f1)/dz1),abs((f2-f1)/dz2))) then
                     f_z(ji,jj,jk) = sign(min(abs((f0-f1)/dz1),abs((f2-f1)/dz2)),f_z(ji,jj,jk))
                  end if
               end if

               if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                  write(numout,*) 'Depths'
                  write(numout,*) gdept_0(ji,jj,jk-1), gdept_0(ji,jj,jk), gdept_0(ji,jj,jk+1)
                  write(numout,*) 'dz'
                  write(numout,*) dz1, dz2
                  write(numout,*) 'Function values'
                  write(numout,*) f0, f1, f2
                  write(numout,*) 'fz', f_z(ji,jj,jk)
               endif
            end do
         end do
      end do



      ! Now, loop over each output cell, skipping the lateral boundaries
      do jk=1,jpk-1
         do jj=2,jpj-1
            do ji=2,jpi-1

            ! If our output cell is masked for the tracer field, we can skip it.
            ! This applies to the tracer mask because grid staggering suggests that
            ! we'll ask for velocities "at" adjacent, in-boundary points with
            ! mock trajectories that lie within the domain.
            
            if (tmask(ji,jj,jk) == 0.0) then
               cycle 
            endif

            if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
               write(numout,'(A,3E11.2)') 'upwnd3_new', xd(ji,jj,jk), yd(ji,jj,jk), zd(ji,jj,jk)
            endif

            ! (ji,jj,jk) is our *output* cell.  Step one here will be to
            ! compute our *input* "cell", which is really a floating-point,
            ! off-grid location

            ! As a reminder, offset_{x,y,z} are universal for-this-interpolation
            ! offsets to apply to the local xd/yd/zd displacements, in order to
            ! aid in interpolating between grid.
            lox = real(ji,wp) - xd(ji,jj,jk) - offset_x
            loy = real(jj,wp) - yd(ji,jj,jk) - offset_y
            ! To account for partial vertical steps, loz_g is here specified using
            ! the *global* e3t_1d value; this will have to be rescaled based on
            ! the local value for vertical interoplation
            !loz_g = real(jk,wp) - zd(ji,jj,jk) - offset_z

            ! Clamp loz_g to not rise above the top boundary.  This corresponds to an index of
            ! 0.5 for the u, v, and t grids, and to accommodate the w-grid we will apply the
            ! offset afterwards.
            !if (real(jk,wp) - zd(ji,jj,jk) < 0.5) then
            !   loz_g = 0.5 - offset_z
            !endif


            ! Now, lox/loy/loz contain a full real location.  We want to turn this
            ! into an integer cell and a floating-point displacement less than one
            ! cell.
            !write (6,'(3I4,3F6.2)') ji, jj, jk, lox, loy, loz
            ix = int(lox)
            iy = int(loy)
            lox = lox - real(ix,wp)
            loy = loy - real(iy,wp)

            ! Handling the z-offset is tricky with respect to the stretched grid.  The values we have here
            ! in the form of loz_g are cell offsets based on the e3t of the global level at the destination
            ! cell.

            ! Compute the absolute level we're interpolating from
            ! For this, note that the input cell-based displacement is scaled by the grid factors of the
            ! appropriate reference t-cell (that is to say, e3t_1d).  However, the *global* offset is scaled
            ! by the *local* grid, as it is used to transfer between the (u/v/t)-levels and the w-levels.
            ! In turning this global cell-displacement into a physical depth, the use of the w-depths is
            ! most appropriate
            source_z = gdept_1d(jk) - e3t_1d(jk)*zd(ji,jj,jk) - (gdepw_0(ji,jj,jk+1)-gdepw_0(ji,jj,jk))*offset_z

            ! Clamp this by the surface height, including any offset
            source_z = max(source_z,gdepw_1d(1)-offset_z*(gdepw_1d(2)-gdepw_1d(1)))

            ! Construct a preliminary integer "source cell", which will be exact if e3t were uniform:
            iz_g = min(jpk-1,max(1,int(real(jk,wp) - zd(ji,jj,jk) - offset_z)))

            ! Correct this by checking whether source_z is above this level
            do while (source_z < gdept_1d(iz_g))
               iz_g = iz_g - 1
               if (iz_g < 1) exit
            enddo

            ! Now correct for source_z being below iz_g+1.  We don't need any special correction
            ! for the very bottom level because it is wholly masked.
            do while (iz_g < jpk)
               if (source_z <= gdept_1d(iz_g+1)) exit
               iz_g = iz_g + 1
            enddo

            ! Finally, the local offset is given by the dividing the partial-cell difference by
            ! the true layer thickness
            if (iz_g == 0) then
               loz_g = 0.5 + source_z / (2*gdept_1d(1))
            else
               loz_g = (source_z - gdept_1d(iz_g))/(gdept_1d(iz_g+1)-gdept_1d(iz_g))
            endif

            ! At this point, iz_g contains the global level most closely above the interpolation
            ! point, and loz_g contains the offset in the range [0,1) (save at the bottom level)
            ! corresponding to the fractional layer thickness between iz_g and iz_g+1 where the
            ! interpolation lies.
            
            if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
               write (numout,'(3I4,3E10.3,1F8.2)') ix, iy, iz_g, lox, loy, loz_g, source_z
            endif

            ! Do not clamp interpolation   
            if (iz_g < 1) then
               ! Interpolating at the top boundary is a bit special, since there is no
               ! kk=0 point on the arrays.

               ! This interpolation is possible because of the ghost point -- the phantom point
               ! above the surface has the same value as the first point and the opposite derivative.

               iz = iz_g
               loz = loz_g

               do jl=-1,2
                  do jm=-1,2
                     ! Check to see if we're still in the computational domain
                     if (ix + jl < 1 .or. ix + jl > nlci .or. &
     &                   iy + jm < 1 .or. iy + jm > nlcj .or. (iz < 0)) then
                        f2d_xy(jl+2,jm+2) = 0.0_wp
                        f2dmask_xy(jl+2,jm+2) = 0.0_wp
                        cycle 
                     end if
                     ! Check to see if we're on land
                     if (f_mask(ix+jl,iy+jm,1) == 0.0) then
                        f2d_xy(jl+2,jm+2) = 0.0_wp
                        f2dmask_xy(jl+2,jm+2) = 0.0_wp
                        cycle
                     else
                        ! Not on land. 
                        f2dmask_xy(jl+2,jm+2) = 1.0_wp

                        ! The full Hermite interpolation form reduces greatly here, into a parabola
                        ! with the gridpoint value at 0 and 1, and appropriate derivatives
                        val_r = f_in(ix+jl,iy+jm,1)
                        grad_r = f_z(ix+jl,iy+jm,1)*(2*gdept_1d(1)) ! Rescale derivative by layer thickness
                        f2d_xy(jl+2,jm+2) = val_r  + loz*(loz-1)*grad_r
                        if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k .and. &
                                             ix + jl == diag_i .and. iy + jm == diag_j) then
                           write(numout,*) '-- surface vertical semilag'
                           write(numout,*) f_in(ix+jl,iy+jm,1), f_z(ix+jl,iy+jm,1), f2d_xy(jl+2,jm+2)
                           write(numout,*) val_r, grad_r
                        end if
                     end if
                  end do
               end do
            else ! If we're not at the top (iz > 0)
                        
            ! Interpolation will proceed along z, then x, then y, so we first want
            ! to compute f(ix-1:ix+2,iy-1:iy+2,iz+loz), storing these intermediate
            ! results into f2d_xy

            do jl = -1,2
               do jm = -1,2

                  ! If we're interpolating off-grid, that's not good -- mark the cell
                  ! as masked.  Ideally, this will not occur for an interior point, but
                  ! it may be a concern for high Courant numbers combined with small halo
                  ! regions.
                  if (ix + jl < 1 .or. ix + jl > nlci .or. &
     &                iy + jm < 1 .or. iy + jm > nlcj) then
                     f2d_xy(jl+2,jm+2) = 0.0_wp
                     f2dmask_xy(jl+2,jm+2) = 0.0_wp
                     cycle 
                  end if
                 
                  ! The global z-displacement is given with reference to the background level
                  ! structure.  In the presence of partial cells, this may not reflect the
                  ! grid we actually have.  Rescaling is a three-step process:

                  ! First, the cell-length displacement from the background above-level is
                  ! turned into a physical displacement from the background above-level
                  loz = loz_g*(gdept_1d(iz_g+1)-gdept_1d(iz_g))

                  ! Next, this is adjusted by any difference in elevation between the
                  ! local above-level and the background
                  loz = loz + (gdept_1d(iz_g)-gdept_0(ix+jl,iy+jm,iz_g))

                  ! Finally, this is re-scaled into a cell-fraction based on e3w
                  if (loz < 0) then
                     ! This can happen if this partial cell is thicker than the
                     ! background levels, where a location that is globally between
                     ! the level and bathymetry is here between the bottom level and
                     ! the one above
                     iz = iz_g-1
                     loz = loz + gdept_0(ix+jl,iy+jm,iz+1) - gdept_0(ix+jl,iy+jm,iz)
                     !iz = iz_g
                     !loz = 0.0
                  elseif (loz > (gdept_0(ix+jl,iy+jm,iz_g+1)-gdept_0(ix+jl,iy+jm,iz_g))) then
                     ! This will happen if we're interpolating into a partial cell,
                     ! where a displacement between the bottom background level and
                     ! the one above is physically located between the local bottom
                     ! level and the boundary
                     iz = iz_g+1
                     if (iz > jpk-1) then
                        ! Going past the bottom boundary
                        f2d_xy(jl+2,jm+2) = 0.0_wp
                        f2dmask_xy(jl+2,jm+2) = 0.0_wp
                        cycle
                     endif
                     loz = loz - (gdept_0(ix+jl,iy+jm,iz)-gdept_0(ix+jl,iy+jm,iz-1))
                     !iz = iz_g
                     !loz = 1.0
                  else ! No change necessary
                     iz = iz_g
                  endif
                  ! Now, undo this scaling to convert loz back to a fractional level
                  ! displacement, using the proper levels.
                  dz1 = gdept_0(ix+jl,iy+jm,iz+1) - gdept_0(ix+jl,iy+jm,iz)
                  loz = loz / dz1

                  ! Now, the core of this interpolation is the the cubic Hermite spline.
                  ! The off-grid value is based on a weighted sum of the "left" and "right"
                  ! points, along with the slope of the curve at those points.  The weights
                  ! are given by cubic polynomials in x (0<=x<=1)

                  ! To perform this interpolation, we need the value and derivative at the
                  ! two points that bracket our 1D interpolation target.  We'll call those
                  ! the ''left'' and ''right'' points for simplicity, even though the
                  ! variables will be re-used in each dimension

                  ! One final scaling is necessary -- the derivatives precomputed in f_z
                  ! are in physical units, as (stuff)/(meter), and we will interpolate
                  ! based on a layer fraction.  This requires dividing f_z by the precomputed
                  ! dz1 layer thickness.


                  if (f_mask(ix+jl,iy+jm,iz) == 1.0 .and. f_mask(ix+jl,iy+jm,iz+1) == 1.0) then
                     ! Default case: both appropriate cells are un-masked
                     f2dmask_xy(jl+2,jm+2) = 1.0_wp
                     val_l = f_in(ix+jl,iy+jm,iz)
                     val_r = f_in(ix+jl,iy+jm,iz+1)

                     grad_l = f_z(ix+jl,iy+jm,iz) * dz1
                     grad_r = f_z(ix+jl,iy+jm,iz+1) * dz1
                  elseif(f_mask(ix+jl,iy+jm,iz) == 0.0) then
                     ! Based on the NEMO grid that has water-over-bottom, if the top point is
                     ! masked then the bottom point must also be masked and we're living inside
                     ! the boundary
                     f2d_xy(jl+2,jm+2) = 0.0_wp
                     f2dmask_xy(jl+2,jm+2) = 0.0_wp
                     cycle
                  elseif(f_mask(ix+jl,iy+jm,iz) == 1.0 .and. f_mask(ix+jl,iy+jm,iz+1) == 0.0) then
                     ! If only the bottom point is masked, our behaviour depends on the BC type

                     ! But these do not change
                     val_l = f_in(ix+jl,iy+jm,iz)
                     grad_l = f_z(ix+jl,iy+jm,iz) * dz1 
                     if (bdy_z == 'zero_mask') then
                        ! With the zero-at-mask boundary type, the bottom point has a zero value.
                        val_r = 0.0_wp

                        ! Additionally, to avoid supurious reversals in w-velocity, limit the 
                        ! effective 'left' gradient:
                        if (val_l >= 0) then
                           grad_l = max(grad_l,-2*val_l)
                        else 
                           grad_l = min(grad_l,-2*val_l)
                        endif

                        ! This avoids a problem whereby the reconstructed vertical velocity could 
                        ! be positive at a near-boundary t-point, despite the above w-velocity
                        ! being negative; this positive w-velocity would in turn lead to
                        ! modest extrapolation of tracer values and spurious maxima.  This grad_l
                        ! is chosen so that at its limited value, grad_r = 0 implied by the BC.

                        ! And the derivative at the boundary is calculated by extrapolating
                        ! val_l/grad_l to the 0-boundary via a quadratic:
                        ! (f = f0 + x f0' + A x^2 -> f(1)=0=f0 + f0' + A -> A = -(f0+f0')
                        !   -> f'(1) = f0' + 2A = -2f0 - f0'
                        grad_r = -2*val_l - grad_l
                     elseif (bdy_z == 'noflux_between') then
                        ! With no flux between cells, then if we're further than half a cell
                        ! from the 'left' (top) point we're actually inside the boundary; we
                        ! should mask this cell
                        if (loz > 0.5) then
                           ! Experiment: Limit partial-cell extrapolation to the boundary proper,
                           ! so that information can still be exchanged from shallow neighoburs
                           ! to a deeper cell
                           ! loz = 0.5
                           f2d_xy(jl+2,jm+2) = 0.0_wp
                           f2dmask_xy(jl+2,jm+2) = 0.0_wp
                           cycle
                        endif
                        ! Otherwise, we're looking at a mirroring boundary condition, and we
                        ! can use the same one-sided stencil as above
                        val_r = val_l
                        grad_r = -grad_l ! The first derivative has odd symmetry
                     elseif (bdy_z == 'zero_between') then
                        ! With zero between cells, we have odd symmetry.  Otherwise, the logic
                        ! proceeds for noflux_between
                        if (loz > 0.5) then
                           loz = 0.5
                           !f2d_xy(jl+2,jm+2) = 0.0_wp
                           !f2dmask_xy(jl+2,jm+2) = 0.0_wp
                           !cycle
                        endif
                        val_r = -val_l
                        grad_r = grad_l ! Even symmetry on first derivative
                     else
                        call ctl_stop('upwnd3_sld: e r r o r, bdy_z not valid', bdy_z)
                     endif
                  endif

                  
                  ! Now, we have val_l, val_r, grad_l, and grad_r all set consistently
                  ! with the boundary conditions; we can actually interpolate.

                  ! The hermite basis functions for this interpolation are
                  ! (2 x**3 - 3 x**2 + 1), multiplying the left value
                  ! (-2 x**3 + 3 x**2), multiplying the right value
                  ! (x**3 - 2 x**2 + 1) , multiplying the left gradient
                  ! (x**3 - x**2), multiplying the right gradient

                  ! If we wish to slope-limit the interpolation to prevent introducing
                  ! overshoots and undershoots, this is the place to add the limiter;
                  ! it will modify the gradient factors.

                  f2dmask_xy(jl+2,jm+2) = 1.0_wp ! We're in-domain
                  coefr = -2.0*loz**3 + 3.0*loz**2
                  coefl = 1-coefr
                  f2d_xy(jl+2,jm+2) = coefl * val_l  + &
    &                                 coefr * val_r  + &
    &                                 ( 1.0 * loz**3 - 2.0 * loz**2 + loz) * grad_l + &
    &                                 ( 1.0 * loz**3 - 1.0 * loz**2      ) * grad_r
                  if (f_mask(ix+jl,iy+jm,iz+1) == 0.0 .and. &
                         (f2d_xy(jl+2,jm+2) > max(val_l,val_r) .or. & 
                          f2d_xy(jl+2,jm+2) < min(val_l,val_r))) then
                     ! Limiting: if the value would overshoot near the bottom boundary, apply linear interpolation
                     ! Modification Sept 14 2016: limit only cells substantially deeper than shallowest neighbour
                     if ((ix+jl == 1 .or. ix+jl == jpi .or. iy+jm == 1 .or. iy+jm == jpj) .or. &
                         (e3t_0(ix+jl,iy+jm,iz) > 1.75*min(e3t_0(ix+jl-1,iy+jm,iz),e3t_0(ix+jl+1,iy+jm,iz)) .or. &
                           e3t_0(ix+jl,iy+jm,iz) > 1.75*min(e3t_0(ix+jl,iy+jm-1,iz),e3t_0(ix+jl,iy+jm+1,iz)))) then
                        f2d_xy(jl+2,jm+2) = val_l + loz*(val_r-val_l)
                     end if
                  end if
                  if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k .and. &
                                       ix + jl == diag_i .and. iy + jm == diag_j) then
                     write(numout,*) '-- depth vertical semilag'
                     write(numout,*) val_l, val_r, loz
                     write(numout,*) grad_l, grad_r
                     write(numout,*) f2d_xy(jl+2,jm+2)
                     write(numout,*) '-- grid depths'
                     write(numout,*) gdept_0(diag_i,diag_j,diag_k:diag_k+1)
                     write(numout,*) gdept_0(diag_i,diag_j,iz:iz+1)
                     write(numout,*) e3t_0(diag_i,diag_j,diag_k:diag_k+1)
                  end if

               end do ! End loop over jm
            end do    ! End loop over jl
            endif ! End if iz < 1

            ! Now, we've evaluated up to 16 points, interpolating in z.  We repeat the
            ! process in one of the other directions; let's pick x.
            if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
               write(numout,'(4E14.5)') f2d_xy
            endif


            ! Loop over y values, interpolate in x

            do jm=1,4
               f1d_y(jm) = hermite_1d(f2d_xy(:,jm),f2dmask_xy(:,jm),bdy_x,lox,f1dmask_y(jm),noisy,do_limit,toler)
            end do
            if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                write (numout,*) 'After x interpolation'
                write(numout,'(4E15.5)') f1d_y
            endif

            ! Interpolate resulting 1D array in y
            f_out(ji,jj,jk) = hermite_1d(f1d_y,f1dmask_y,bdy_y,loy,out_mask,noisy,do_limit,toler)

            if (print_diag > 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
               write(numout,'(1E13.5, 1F3.0)') f_out(ji,jj,jk), out_mask
            endif

            end do ! Loop over ji
         end do ! Loop over jj
      end do ! Loop over jk

end subroutine

function hermite_1d(f1d, f1d_mask, bdy_type, offset, out_mask, noisy, do_limit, toler)
   ! The 1D interpolation is shared between the x and y directions, so it should
   ! be broken out into its own function.

   implicit none

   ! Parameters
   real(wp) :: hermite_1d ! Output interpolated value
   real(wp), intent(out) :: out_mask ! In/out of domain 
   real(wp), dimension(4), intent(in) :: f1d, f1d_mask ! Function values and domain mask
   real(wp), intent(in) :: offset ! Offset for interpolation, 0<off<1
   character(len=*), intent(in) :: bdy_type ! Boundary type, as per upwnd3
   integer, intent(inout) :: noisy
   logical, intent(in) :: do_limit ! Whether the method should limit overshoots
   real(wp), intent(in) :: toler   ! Tolerance on Hermite overshoots (do_limit = .true.)

   ! Local variables
   real(wp), dimension(4) :: f ! Copy of f1d, including any boundary mirroring
   real(wp) :: val_l, val_r   ! Function values at f(2) and f(3), respecting BCs
   real(wp) :: grad_l, grad_r ! Derivatives at f(2) and f(3), respecting BCs
   real(wp) :: coefl, coefr ! Left-right coefficients
   real(wp) :: f12, f21, f23, f32, f34, f43
   integer :: noisyhere

   ! Our input array of values is given by f1d.  Along this line, the
   ! target of our interpolation lies between jj=2 and jj=3.

   ! If both of these values are off-grid, we're done and go home early.

   noisyhere = 0
   if (noisy /= 0) then
      noisyhere = 1
      noisy = 0
      write(numout,*) 'hermite_1d'
      if (maxval(abs(f1d)) > 0.1) then
         write(numout,'(A, 4F7.2)') 'interpolating', f1d
      else
         write(numout,'(A, 4E10.2)') 'interpolating', f1d
      endif
      write(numout,'(A, 4F4.1)') 'mask', f1d_mask
      write(numout,*) 'offset', offset
   endif

   if (f1d_mask(2) == 0.0 .and. f1d_mask(3) == 0.0) then
      out_mask = 0.0
      hermite_1d = 0.0
      return
   elseif (f1d_mask(2) == 0.0 .and. offset < 0.5 .and. &
   &       (bdy_type == 'zero_between' .or. bdy_type == 'noflux_between')) then
      ! If we're more than halfway to the left point and it is masked and
      ! we have between-cell boundary conditions, we're actually off-grid.
      out_mask = 0.0
      hermite_1d = 0.0
      return
   elseif (f1d_mask(3) == 0.0 .and. offset > 0.5 .and. &
   &       (bdy_type == 'zero_between' .or. bdy_type == 'noflux_between')) then
      ! Symmetric to the previous case, at the right boundary
      out_mask = 0.0
      hermite_1d = 0.0
      return
   endif

   ! Now, we get to calculate the function and its derivative at the
   ! left and right points.  The stencil for the full gradient is
   !    1/6 * [-2, -3, 6, -1] at the left point, and
   !    1/6 * [ 1, -6, 3,  2] at the right point.
   ! These stencils apply when there is no masking, but will obviously
   ! need to be modified for boundaries.  We can handle much of the work
   ! via symmetry

   ! Start by copying input values to our local array
   f(:) = f1d(:)

   if (f1d_mask(2) == 1.0 .and. f1d_mask(1) == 0.0) then
      ! If the leftmost point is masked, account for BCs by appropriate symmetry
      if (bdy_type == 'zero_mask') then
         f(1) = 0.0
      elseif (bdy_type == 'noflux_between') then
         f(1) = f(2) ! Even symmetry
      elseif (bdy_type == 'zero_between') then
         f(1) = -f(2) ! Odd symmetry
      else
         call ctl_stop('hermite_1d: e r r o r, invalid boundary type', bdy_type)
      endif
   endif
   if (f1d_mask(3) == 1.0 .and. f1d_mask(4) == 0.0) then
      ! Repeat symmetrically for the right point
      if (bdy_type == 'zero_mask') then
         f(4) = 0.0
      elseif (bdy_type == 'noflux_between') then
         f(4) = f(3) ! Even symmetry
      elseif (bdy_type == 'zero_between') then
         f(4) = -f(3) ! Odd symmetry
      else
         call ctl_stop('hermite_1d: e r r o r, invalid boundary type', bdy_type)
      endif
   endif

   ! Now, assign left and right values and gradients
   if (f1d_mask(2) == 1.0 .and. f1d_mask(3) == 1.0) then
      ! Easy case: nothing's masked
      val_l = f(2)
      val_r = f(3)
      grad_l = 1.0/6.0*(-2.0 * f(1) - 3.0 * f(2) + 6.0 * f(3) - 1.0 * f(4))
      grad_r = 1.0/6.0*( 1.0 * f(1) - 6.0 * f(2) + 3.0 * f(3) + 2.0 * f(4))
      !grad_l = 1.0/2.0*(1.0 * f(3) - 1.0 * f(1))
      !grad_r = 1.0/2.0*(1.0 * f(4) - 1.0 * f(2))

      if (noisyhere /= 0) then
         write (numout,*) '***'
         write (numout,*) f(1), f(2), f(3), f(4)
         write (numout,*) grad_l, grad_r
      endif
      ! Slope limiting: check for a local maximum
      if (do_limit) then
         f21=f(2)-f(1)
         f12=-f21
         f23=f(2)-f(3)
         f32=-f23
         f34=f(3)-f(4)
         f43=-f34
         if (f21 > toler .and. f23 > toler) then
            ! f(2) is a local maximum
            grad_l = min(grad_l,0.0) ! Suppress left-gradient > 0
         elseif (f12 > toler .and. f32 > toler) then
            ! Local minimum
            grad_l = max(grad_l,0.0) ! Suppress left-gradient < 0
         endif
         if (f32 > toler .and. f34 > toler) then
            ! f(3) is a local maximum
            grad_r = max(grad_r,0.0)
         elseif (f23 > toler .and. f43 > toler) then
            ! f(3) is a local minimum
            grad_r = min(grad_r,0.0)
         endif
!         if (f(1) < f(2) .and. f(3) < f(2)) then
!            ! f(2) is a local maximum
!            grad_l = min(grad_l,0.0) ! Suppress left-gradient > 0
!         elseif (f(1) > f(2) .and. f(3) > f(2)) then
!            ! Local minimum
!            grad_l = max(grad_l,0.0) ! Suppress left-gradient < 0
!         endif
!         if (f(2) < f(3) .and. f(4) < f(3)) then
!            ! f(3) is a local maximum
!            grad_r = max(grad_r,0.0)
!         elseif (f(2) > f(3) .and. f(4) > f(3)) then
!            ! f(3) is a local minimum
!            grad_r = min(grad_r,0.0)
!         endif
      endif
      if (noisyhere /= 0) then
         write (numout,*) grad_l, grad_r
         write (numout,*) '***'
      endif
   elseif (f1d_mask(2) == 0.0) then
      ! Left point is masked, but the right isn't (see check for both
      ! at the top of the function), so find its value by appropriate
      ! symmetry and compute the gradient using symmetry-extended values
      val_r = f(3)
      if (bdy_type == 'zero_mask') then
         ! f(1) = -f(3), f(2) = 0.0
         val_l = 0.0
         !grad_l = f(3)
         !grad_r = 1.0/2.0*f(4)
         grad_l = 1.0/6.0*(8.0 * f(3) - f(4))
         grad_r = 1.0/3.0*(f(3) + f(4))
      elseif (bdy_type == 'noflux_between') then
         ! f(1) = f(4), f(2) = f(3)
         val_l = val_r
         grad_l = 0.5*(f(3) - f(4))
         grad_r = 0.5*(f(4) - f(3))
         grad_l = 0.0
         grad_r = 0.0
      elseif (bdy_type == 'zero_between') then
         ! f(1) = -f(4), f(2) = -f(3); the first derivative has even symmetry
         val_l = -val_r
         !grad_l = 1.0/2.0*(f(3)+f(4))
         !grad_r = 1.0/2.0*(f(4)+f(3))
         grad_l = 1.0/6.0*(9.0 * f(3) + f(4))
         grad_r = 1.0/6.0*(9.0 * f(3) + f(4))
      else
         call ctl_stop('hermite_1d: e r r o r, invalid boundary type', bdy_type)
      endif
   elseif (f1d_mask(3) == 0.0) then
      ! The right-point equivalent of the above case.
      val_l = f(2)
      if (bdy_type == 'zero_mask') then
         val_r = 0.0
         grad_l = 1.0/3.0*(-1.0 * f(1) - 1.0 * f(2))
         grad_r = 1.0/6.0*( f(1) - 8.0 * f(2))
         !grad_l =  1.0/2.0*(-f(1))
         !grad_r = f(2)
      elseif (bdy_type == 'noflux_between') then
         val_r = val_l
         grad_l = 0.5*(f(2) - f(1))
         grad_r = 0.5*(f(1) - f(2))
         grad_l = 0.0
         grad_r = 0.0
      elseif (bdy_type == 'zero_between') then
         ! f(3) = 0, f(4) = -f(2)
         val_r = -val_l
         grad_l = 1.0/3.0 * (-f(1) + f(2))
         grad_r = 1.0/6.0 * (f(1) - 8.0 * f(2))
      else 
         call ctl_stop('hermite_1d: e r r o r, invalid boundary type', bdy_type)
      endif
   endif

   ! If we're slope-limiting, do so here.

   ! Now, we have values and gradients, so complete the interpolation.
   if (noisyhere /= 0) then
      write(numout,'(4E10.2)') val_l, val_r, grad_l, grad_r
   endif
   ! Precompute coefficients for val-left and val-right to help
   ! ensure they add to 1
   coefr = -2.0*offset**3 + 3.0*offset**2
   coefl = 1-coefr
   hermite_1d = coefl * val_l  + &
 &              coefr * val_r  + &
 &              ( 1.0 * offset**3 - 2.0 * offset**2 + offset) * grad_l + &
 &              ( 1.0 * offset**3 - 1.0 * offset**2         ) * grad_r

      out_mask = 1.0
end function

! upwnd1 : Provide for *LINEAR* interpolation of a field at a single point,
! given by floating-point coordinates (xx-xd, yy-yd, zz-zd), and respecting offsets and
! boundary conditions as in upwnd3.

! This function is intended to provide for fast, low-order, local interpolation
! for computing trajectories, and to serve this purpose it applies to only
! a single provided point per invocation -- this permits local variations in
! the number of iterations to compute trajectories.
subroutine upwnd1( f_out, mask_out, f_in, f_mask, xx, xd, yy, yd, zz, zd, &
   &                      offset_x, offset_y, offset_z,    &
   &                      bdy_x, bdy_y, bdy_z )

      implicit none

      ! Arguments

      real(wp), intent(in), dimension(jpi, jpj, jpk) :: f_in, f_mask
      integer, intent(in) :: xx, yy, zz
      real(wp), intent(in) :: xd, yd, zd
      real(wp), intent(out) :: f_out, mask_out

      character(len=*), intent(in) :: bdy_x, bdy_y, bdy_z

      real(wp), intent(in) :: offset_x, offset_y, offset_z

      ! Local variables

      ! As with upwnd3, the interpolation proceeds in z, then x/y.  Linear
      ! interpolation, however, requires keeping only the single-nearest neighbours
      ! to the off-grid point, not a 4x4 neighbourhood
      real(wp), dimension(2,2):: f2d, f2d_mask
      real(wp), dimension(2):: f1d, f1d_mask

      ! Derived index variables
      integer :: ix, iy, iz, iz_g
      ! Derived local offsets
      real(wp) :: lox, loy, loz, loz_g
      real(wp) :: source_z
      ! Temporary variables for interpolation:
      real(wp) :: val_l, val_r, f_bdy, sfactor
      ! Loop variables
      integer :: ji, jj
      ! Convenience in-loop variables
      integer :: lix, liy

      if (bdy_x /= 'zero_mask' .and. bdy_x /= 'zero_between' &
    &      .and. bdy_x /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_x not valid', bdy_x)
      end if
      if (bdy_y /= 'zero_mask' .and. bdy_y /= 'zero_between' &
    &      .and. bdy_y /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_y not valid', bdy_y)
      end if
      if (bdy_z /= 'zero_mask' .and. bdy_z /= 'zero_between' &
    &      .and. bdy_z /= 'noflux_between') then
         call ctl_stop('upwnd3_sld: e r r o r, bdy_z not valid', bdy_z)
      end if

      ! The local floating-point x, y indices are straightforward given
      ! the inputs and global offsets:

      lox = xx - xd - offset_x
      loy = yy - yd - offset_y

      ! And their integer versions are equally trivial:

      ix = floor(lox)
      iy = floor(loy)

      ! Allowing us to reuse lox/loy for the fractional cell offset
      lox = lox - real(ix,wp)
      loy = loy - real(iy,wp)

      ! z-interpolation is special because of the effects of partial cells and/or variable
      ! volume.  As such, we want an absolute depth compared to the input zz (source coordinate)
      ! and xd (normalized by e3), with a correction based on the w-grid for any z-offset.

      source_z = gdept_1d(zz) - e3t_1d(zz)*zd - (gdepw_0(xx,yy,zz+1)-gdepw_0(xx,yy,zz))*offset_z
      ! Starting guess for 
      iz_g = min(max(zz - floor(zd+offset_z),1),jpk-1)

      if (super_noisy) then
         write (numout,*) 'interpolating from ', (xx-xd-offset_x), (yy-yd-offset_y)
      endif

      f2d = 0.0
      f2d_mask = 0.0
      do ji=0,1
         do jj=0,1
            ! Sanity-checking for being on-grid
            if ((ix+ji) < 1 .or. (ix+ji) > nlci) then
               f2d(ji+1,jj+1) = 0.0
               f2d_mask(ji+1,jj+1) = 0.0
               cycle
            endif
            if ((iy+jj) < 1 .or. (iy+jj) > nlcj) then
               f2d(ji+1,jj+1) = 0.0
               f2d_mask(ji+1,jj+1) = 0.0
               cycle
            endif
            ! Now, interpolate in the vertical to fill the f2d array

            ! The first step here is to -find- the appropriate vertical coordinate,
            ! that being the index iz such that source_z lies between the levels iz
            ! and iz+1 at (ix+ji,iy+jj)

            iz = iz_g
            do while (iz >= 1)
               if (source_z >= gdept_0(ix+ji,iy+jj,iz)) exit
               iz = iz-1
            enddo
            do while (iz <= jpk-1)
               if (source_z <= gdept_0(ix+ji,iy+jj,iz+1)) exit
               iz = iz+1
            enddo


            ! Special-casing if iz=0, giving an implicit extrapolate-to-the-surface-as-constant
            if (iz == 0) then
               f2d_mask(ji+1,jj+1) = f_mask(ix+ji,iy+jj,1)
               f2d(ji+1,jj+1) = f_in(ix+ji,iy+jj,1)
               cycle
            endif

            ! Check if the top point is masked; if so then the bottom point must also be masked
            ! and we're inevitably out-of-fluid
            if (f_mask(ix+ji,iy+jj,iz) == 0.0) then
               f2d_mask(ji+1,jj+1) = 0.0
               f2d(ji+1,jj+1) = 0.0
               cycle
            endif
            
            loz = (source_z-gdept_0(ix+ji,iy+jj,iz))/(gdept_0(ix+ji,iy+jj,iz+1)-gdept_0(ix+ji,iy+jj,iz))

            ! Copy over left=top value and right=bottom value, correcting for vertical boundary
            ! condition if masked
            val_l = f_in(ix+ji,iy+jj,iz)
            if (f_mask(ix+ji,iy+jj,iz+1) /= 0.0) then
               val_r = f_in(ix+ji,iy+jj,iz+1)
            else ! bottom point masked
               if (bdy_z == 'zero_mask') then
                  val_r = 0.0
               elseif (loz > 0.5) then
                  f2d(ji+1,jj+1) = 0.0
                  f2d_mask(ji+1,jj+1) = 0.0
                  cycle
               elseif (bdy_z == 'zero_between') then
                  val_r = -f_in(ix+ji,iy+jj,iz)
               elseif (bdy_z == 'noflux_between') then
                  val_r = f_in(ix+ji,iy+jj,iz)
               else 
                  call ctl_stop('upwnd1: e r r o r, invalid boundary type', bdy_z)
               endif
            endif

            ! Now, perform linear interpolation.  We've already ensured we're in-domain.
            f2d_mask(ji+1,jj+1) = 1.0
            f2d(ji+1,jj+1) = (1-loz)*val_l + loz*val_r
         enddo
      enddo

      ! Interpolate in x, then y based on lox/loy

      if (super_noisy) then
         write (numout,'(2E14.5)') f2d
         write (numout,'(2F7.3)') f2d_mask
      endif

      if (bdy_y == 'zero_mask' .and. bdy_x == 'noflux_between' .and. loy > 0) then
         ! If we're interpolating on a v-grid and have precisely one masked point,
         ! x-then-y interpolation will give a discontinuous result near lox=0.5

         ! In these cases, we want to escape from the normal interpolation and do something
         ! a bit fancier, that interpolates linearly along the cell edges (to preserve continuity there)
         ! but has a logarithmic form near the boundary itself to match the BCs.

         if (f2d_mask(1,1) == 0.0 .and. f2d_mask(1,2) == 1.0 .and. f2d_mask(2,1) == 1.0 .and. f2d_mask(2,2) == 1.0) then
            ! Left corner masked -- boundary goes from 0 to lox==0.5 at loy==0

            sfactor = max(loy,2*(lox-0.5))
            f_out = (loy)*((1-lox)*f2d(1,2)+lox*f2d(2,2)) + (1-loy)*lox*f2d(2,1)
            f_out = (0.5+0.5*(lox-0.5)/((lox-0.5)**2+loy**2)**0.5)*(1-sfactor)*f2d(2,1)+sfactor*f_out
            mask_out = 1.0
            return
         elseif (f2d_mask(1,1) == 1.0 .and. f2d_mask(1,2) == 0.0 .and. f2d_mask(2,1) == 1.0 .and. f2d_mask(2,2) == 1.0) then
            ! Left corner masked -- boundary goes from 0 to lox=0.5 at loy==1
            sfactor = max(1-loy,2*(lox-0.5))
            f_out = loy*lox*f2d(2,2)+(1-loy)*(lox*f2d(2,1)+(1-lox)*f2d(1,1))
            f_out = (0.5+0.5*(lox-0.5)/((lox-0.5)**2+(1-loy)**2)**0.5)*(1-sfactor)*f2d(2,2)+sfactor*f_out
            mask_out = 1.0
            return
         elseif (f2d_mask(1,1) == 1.0 .and. f2d_mask(1,2) == 1.0 .and. f2d_mask(2,1) == 0.0 .and. f2d_mask(2,2) == 1.0) then
            ! Right corner masked, boundary goes from lox=0.5 to 1 at loy==0
            sfactor = max(loy,2*(0.5-lox))
            f_out = (loy)*((1-lox)*f2d(1,2)+lox*f2d(2,2)) + (1-loy)*(1-lox)*f2d(1,1)
            f_out = (0.5-0.5*(lox-0.5)/((lox-0.5)**2+loy**2)**0.5)*(1-sfactor)*f2d(1,1)+sfactor*f_out
            mask_out = 1.0
            return
         elseif (f2d_mask(1,1) == 1.0 .and. f2d_mask(1,2) == 1.0 .and. f2d_mask(2,1) == 1.0 .and. f2d_mask(2,2) == 0.0) then
            ! Right corner masked, boundary goes from lox=0.5 to 1 at loy==1
            sfactor = max(1-loy,2*(0.5-lox))
            f_out = loy*(1-lox)*f2d(1,2)+(1-loy)*(lox*f2d(2,1)+(1-lox)*f2d(1,1))
            f_out = (0.5-0.5*(lox-0.5)/((lox-0.5)**2+(1-loy)**2)**0.5)*(1-sfactor)*f2d(1,2)+sfactor*f_out
            mask_out = 1.0
            return
         endif
      endif

      do jj=0,1
         f1d_mask(jj+1) = 1.0 ! Initially assume we're in-domain
         if (f2d_mask(1,jj+1) == 0.0 .and. f2d_mask(2,jj+1) == 0.0) then
            ! Point is masked
            f1d_mask(jj+1) = 0.0
            f1d(jj+1) = 0.0
            cycle
         endif

         if (f2d_mask(1,jj+1) /= 0.0) then ! Left point is not masked
            val_l = f2d(1,jj+1)
         elseif (bdy_x == 'zero_mask') then ! Left point is masked, right is not
            val_l = 0.0
            ! Signal that we're close to a boundary via the mask variable -- see discussion at final interpolation
            f1d_mask(jj+1) = lox
         elseif (lox < 0.5) then ! Inside boundary
            f1d(jj+1) = 0.0
            f1d_mask(jj+1) = 0.0
            cycle
         elseif (bdy_x == 'zero_between') then
            val_l = -f2d(2,jj+1)
         elseif (bdy_x == 'noflux_between') then
            val_l = f2d(2,jj+1)
         else 
            call ctl_stop('upwnd1: e r r o r, invalid boundary type', bdy_x)
         endif

         if (f2d_mask(2,jj+1) /= 0.0) then ! Right point is not masked
            val_r = f2d(2,jj+1)
         elseif (bdy_x == 'zero_mask') then ! Right point is masked, left is not
            val_r = 0.0
            ! And indicate distance to the boundary
            f1d_mask(jj+1) = -(1-lox)
         elseif (lox > 0.5) then ! Inside boundary
            f1d(jj+1) = 0.0
            f1d_mask(jj+1) = 0.0
            cycle
         elseif (bdy_x == 'zero_between') then
            val_r = -f2d(1,jj+1)
         elseif (bdy_x == 'noflux_between') then
            val_r = f2d(1,jj+1)
         else 
            call ctl_stop('upwnd1: e r r o r, invalid boundary type', bdy_x)
         endif

         ! Perform linear interpolation
         !f1d_mask(jj+1) = 1.0
         f1d(jj+1) = (1-lox)*val_l + lox*val_r
      enddo

      if (super_noisy) then
         write (numout,'(2E14.5)') f1d
         write (numout,'(2F7.3)') f1d_mask
      endif
      ! Interpolation in y
      if (f1d_mask(1) == 0.0 .and. f1d_mask(2) == 0.0) then
         mask_out = 0.0
         f_out = 0.0
         return
      endif

      if (f1d_mask(1) /= 0) then ! Left is not masked
         val_l = f1d(1)
      elseif (bdy_y == 'zero_mask') then ! Left is masked; right is not
         val_l = 0.0
      elseif (loy < 0.5) then ! In boundary
         mask_out = 0.0
         f_out = 0.0
         return
      elseif (bdy_y == 'zero_between') then
         val_l = -f1d(2)
      elseif (bdy_y == 'noflux_between') then
         mask_out = 1.0
         f_out = f1d(2)
         return
      else
         call ctl_stop('upwnd1: e r r o r, invalid boundary type', bdy_y)
      endif
      
      if (f1d_mask(2) /= 0) then ! Right is not masked
         val_r = f1d(2)
      elseif (bdy_y == 'zero_mask') then ! Right is masked, left is not
         val_r = 0.0
      elseif (loy > 0.5) then ! In boundary
         mask_out = 0.0
         f_out = 0.0
         return
      elseif (bdy_y == 'zero_between') then
         val_r = -f1d(1)
      elseif (bdy_y == 'noflux_between') then
         mask_out = 1.0
         f_out = f1d(1)
         return
      else
         call ctl_stop('upwnd1: e r r o r, invalid boundary type', bdy_y)
      endif

      ! This ultimately bilinear interpolation doesn't work correctly in the presence
      ! of an x-boundary with zero_mask, because on the NEMO grid the physical boundary
      ! extends halfway into the cell along the y-dimension:

      ! x--o
      ! x  |     
      ! |  |    
      ! o--o       

      ! Not only does this mean that x-then-y interpolation returns different results
      ! than y-then-x interpolation, but it also means that the zero-at-boundary condition
      ! does not strictly hold.  This plays hell with trajectory calculations.

      ! To fix this, the above code signals fractional mask values when x-interpolating
      ! in the presence of a zero-at-mask boundary.  We can now check this here:

      ! First, perform ordinary bilinear interpolation
      mask_out = 1.0
      f_out = (1-loy)*val_l + loy*val_r

      if (bdy_x == 'zero_mask' .and. bdy_y == 'noflux_between') then
         if (f1d_mask(1) == 0 .or. f1d_mask(2) == 0 .or. f1d_mask(1) == f1d_mask(2)) then
            ! In these cases, there's no correction necessary -- the top and bottom interpolations
            ! are either in-boundary or see the same boundaries.
            return
         endif
         ! Implement corrections for boundaries intruding halfway through cells at
         ! corners
         if (f1d_mask(1) /= 1.0 .and. f1d_mask(2) == 1.0) then
            ! Correct the output: assume that near the singularity
            ! the function transitions between its zero and not-zero
            ! values as cos(theta), then weight this to the far-field
            ! by distance from the boundary-wall

            if (f1d_mask(1) > 0) then 
               ! Left boundary has the wall, so we need to transition between 0
               ! at f2d(1,1) to the value of f2d(1,2)
               sfactor = max(lox,2*(loy-0.5))
               f_out = (0.5+0.5*(loy-0.5)/((loy-0.5)**2+lox**2)**0.5)*f2d(1,2)*(1-sfactor)+sfactor*f_out
            else 
               ! Right boundary has the wall, so the transition is between 0 at f2d(2,1)
               ! and the value of f2d(2,2)
               sfactor = max(1-lox,2*(loy-0.5))
               f_out = (0.5+0.5*(loy-0.5)/((loy-0.5)**2+(1-lox)**2)**0.5)*f2d(2,2)*(1-sfactor) + sfactor*f_out
            endif
         elseif (f1d_mask(1) == 1.0 .and. f1d_mask(2) /= 1.0) then
            ! Same, for walls in the second half of the cell
            if (f1d_mask(2) > 0) then
               ! Transition between 0 at f2d(1,2) to the value of f2d(1,1)
               sfactor = max(lox,2*(0.5-loy))
               f_out = (0.5-0.5*(loy-0.5)/((loy-0.5)**2+lox**2)**0.5)*f2d(1,1)*(1-sfactor)+sfactor*f_out
            else
               ! Transition between 0 at f2d(2,2) to f2d(2,1)
               sfactor = max(1-lox,2*(0.5-loy))
               f_out = (0.5-0.5*(loy-0.5)/((loy-0.5)**2+(1-lox)**2)**0.5)*f2d(2,1)*(1-sfactor) + (sfactor)*f_out
            endif
         elseif (f1d_mask(1) /= f1d_mask(2)) then
            ! In this case, we have an ugly setup where the top and bottom halves of
            ! the cell have different left/right boundaries.  This can happen in the
            ! case of a channel that looks like a Tetris z-block.  
            if (f1d_mask(1) > 0) then
               ! The walls are at the top left and bottom right of the cell:
               f_out = (0.5+0.5*(loy-0.5)/((loy-0.5)**2+lox**2)**0.5)*f2d(1,2)*(1-lox)+ &
                       (0.5-0.5*(loy-0.5)/((loy-0.5)**2+(1-lox)**2)**0.5)*f2d(2,1)*lox 
            else ! Walls at bottom left and top right:
               f_out = (0.5+0.5*(loy-0.5)/((loy-0.5)**2+(1-lox)**2)**0.5)*f2d(2,2)*lox + &
                       (0.5-0.5*(loy-0.5)/((loy-0.5)**2+lox**2)**0.5)*f2d(1,1)*(1-lox)
            endif
         endif
      endif
      
            
      return
end subroutine

! Implement the exponential (exact for linear 1D velocity) integration
! of trajectories
subroutine expstep (dt, uo, vo, wo, ui, vi, wi, u0din, v0din, w0din, &
                    u0dout, v0dout, w0dout)
      REAL(wp), INTENT(in) :: dt, uo, vo, wo, ui, vi, wi, u0din, v0din, w0din
      REAL(wp), INTENT(out) :: u0dout, v0dout, w0dout

      REAL(wp) :: dxn, dyn, dzn ! Normalized direction of change of velocity
      REAL(wp) :: votrack, vitrack ! Along-track velocities at origin, estimated foot
      REAL(wp) :: uip, vip, wip ! Components of ui perpendicular to track
      REAL(wp) :: tmag ! Track distance
      REAL(wp) :: ee, ea, dtprime ! Helper variables
      REAL(wp) :: dvdx ! Change of along-track velocity

      ! The basic idea here is to integrate an *exact* trajectory of a particle in
      ! a velocity field given by *linear* interpolation between the "here" velocity
      ! (uo, vo, wo), and the "foot" velocity (ui, vi, wi) at relative coordinates
      ! -(u0din, v0din, w0din)

      ! The velocities are assumed to vary along the direction given by <uo,vo,wo>,
      ! which normalized is:

      votrack = (uo**2+vo**2+wo**2)**0.5
      ! Early exit: if votrack is zero, no interpolation
      if (votrack == 0) then
         u0dout = 0
         v0dout = 0
         w0dout = 0
         return
      endif

      dxn = uo/votrack
      dyn = vo/votrack
      dzn = wo/votrack

      ! And at the foot:
      vitrack = ui*dxn + vi*dyn + wi*dzn
      uip = ui - dxn*vitrack
      vip = vi - dyn*vitrack
      wip = wi - dzn*vitrack

      ! This provides one axis of the coordinate system, the axis along which we
      ! assume velocities vary.  The distance spanned by this trajectory is then:

      tmag = -(u0din*dxn + v0din*dyn + w0din*dzn)

      ! with the negative sign because of the convention for interpolation

      ! Early exit: if tmag is zero, then we don't have interpolation happening:
!CS 2018/07/04      if (tmag == 0) then
      if (tmag >= 0) then
         u0dout = uo*dt
         v0dout = vo*dt
         w0dout = wo*dt
         return
      endif

      ! Over this trajectory, the along-track velocity changes as

      dvdx = (vitrack - votrack)/tmag

      ! Early exit: if dvdx is zero, then apply a trapezoidal rule:
      if (dvdx == 0) then
         u0dout = 0.5*(uo+ui)*dt
         v0dout = 0.5*(vo+vi)*dt
         w0dout = 0.5*(wo+wi)*dt
         return
      endif

      ! Restricted case: if dvdx*dt is small, then the exponential formulas below
      ! will lose some accuracy and maybe be weird.

      if (abs(dvdx*dt) < 1e-3) then
         ! Compute the along-track distance covered by the particle over (-dt) time
         ee = votrack*(-dt + dvdx*dt**2/2)

         ! The across-track distance is given by ignoring the effects of along-track
         ! acceleration, as <d> = ([uvw]p)/tmag*votrack*(-dt^2/2), limited by dt to avoid
         ! extrapolation
         ea = min(dt,votrack/tmag*(-dt**2))
         u0dout = dxn*(-ee) + uip*ea
         v0dout = dyn*(-ee) + vip*ea
         w0dout = dzn*(-ee) + wip*ea

         return
      endif

      ! General case, where we can take an exponential:

      ! The location of this particle at time (-dt) along-track will be given by:
      if (dt*dvdx > -20) then
         ee = votrack/dvdx*(exp(-dt*dvdx)-1)
      else ! Catch overflow with strongly accelerating flows
         ee = -dt*vitrack
      endif

      ! Note that this should be negative.

      ! Which we want to limit to prevent extrapolation:

      if (vitrack == 0) then ! Ensure trajectory shrinkage if we're in the wall
         ee = max(0.95*tmag,ee)
      end if
      ! Also cap trajectory growth
      !ee = max(1.25*tmag,ee)
      ! And forbid extrapolation beyond the maximum along-track velocity seen:
      ee = max(ee,-dt*max(votrack,vitrack))

      ! In the case of limiting, we've effectively locally reduced the timestep; invert
      ! this formula to find out what timestep was actually used:
      if (dt*dvdx < 0) then
         if (abs(ee*dvdx/votrack+1) < 1e-3) then
            ! Taylor series to avoid roundoff-related problems
            dtprime = ee/votrack + dvdx*(ee/votrack)**2
         else
            dtprime = log(ee*dvdx/votrack+1)/dvdx
         endif
      else
         dtprime = -dt
      endif
      
      ! The across-track distance is given by [uvw]ip multiplied by:

      !ea = max(-dt,votrack/tmag/dvdx*((exp(dvdx*(-dt))-1)/dvdx-votrack*dt))
      if (abs(dvdx*dtprime) < 1e-3) then
         ! Low-order expansion to avoid roundoff problems, which reduces
         ! to the trapezoidal rule:
         ea = max(-dt,votrack*dtprime**2/2/tmag)
      else
         ea = max(-dt,1.0/dvdx/tmag*(ee-votrack*dtprime))
      endif

      ! Giving:

      u0dout = -(dxn*ee + uip*ea)
      v0dout = -(dyn*ee + vip*ea)
      w0dout = -(dzn*ee + wip*ea)

      return
end subroutine

end module semilag_adv
