      module outflow_mod

      use par_kind

      implicit none

      INTEGER      LStoSWP
!                  Which Lake Superior:SW Pier relationship to use for 
!                  calculating gated flow.  Used to determine sidechannel (i.e.
!                  non-gated flow in SupCriterions(), and to compute gated
!                  flow for routing in SuperiorRoute.

      real(wp) :: OldSWPA(12), OldSWPB(12), OldSWPC
      real(wp) :: EC2000SWPA, EC2000SWPB, EC2000SWPC(12)
      real(wp) :: EC2008SWPA, EC2008SWPB, EC2008SWPC(12)
!                  Constants for SW Pier/Pt Iroquois relationship

      real(wp) :: SWPGateEqA(16), SWPGateEqB(16)  
!                  Constants for SW Pier gate equations, used for gated
!                  flow calculation.
!
      INTEGER      SideChanMaxCapacity(12)
      real(wp) :: CWOvertopElev, CWMinimumFreeboard(12)
!                  See explanations in RouteSuperior() in Superior.for
!
      INTEGER P77ASup05NBS(12), P77ASup50NBS(12), P77ASup95NBS(12),    &
     &        P77ALLOgokiDiv(12),P77AStMarysRetrd(12),                 &
     &        P77AMH05NBS(12), P77AMH50NBS(12), P77AMH95NBS(12),       &
     &        P77AChicagoDiv(12), P77AStClairRetrd(12),                &
     &        P77AStC50NBS(12), P77ADetroitRetrd(12),                  &
     &        P77AErie50NBS(12), P77AWellandFlow(12),                  &
     &        P77ANiagaraRetrd(12), P77ABaseFlow(12),                  &
     &        P77AMidLakesInc, P77AMidLakesTol, P77AWinterMax,         &
     &        P77AMonthRound, P77AFlowRound, P77ALevelRound,           &
     &        P77AMonthsToAve(12), P77AMaxChange, P77AMinimumFlow(12)

      real(wp) ::  P77ACGIP, P77ALowTrigger, P77AHighTrigger,          &
     &        P77ASPK, P77ASPYM, P77ASPA, P77ASPB, P77ASPWT, P77ASPC,  &
     &        P77AMHK, P77AMHYM, P77AMHA, P77AMHB, P77AMHWT, P77AMHC,  &
     &        P77ASCK, P77ASCYM, P77ASCA, P77ASCB, P77ASCWT, P77ASCC,  &
     &        P77AERK, P77AERYM, P77AERA, P77AERB, P77AERWT, P77AERC,  &
     &        P77ASupTargetLev(12), P77ASupRangeFact(12),              &
     &        P77AMHTargetLev(12), P77AMHRangeFact(12), P77AAValue
      LOGICAL P77AUniformMonth 
      INTEGER   SPOPNBS06(12,6), SPOPNBS07(12,6), OPLLO(12),           &
     &          MHOPNBS06(12,6), MHOPNBS07(12,6), OPChicago,           &
     &          SCRRet(12), SCOPNBS06(12,6), SCOPNBS07(12,6),          &
     &          DetRet(12), EROPNBS06(12,6), EROPNBS07(12,6),          &
     &          OPWelland(12), NiaRet(12)

      CHARACTER*24     P77ACodes
!                      Describes which Plan 1977A Requirements and 
!                      Criteria come into play, used for output.

      CHARACTER*120    P77AHeader
!                      Describes columns of Plan 1977A output

      CHARACTER        SupPreProject*6, SupGateFlow*6,                 &
     &                 SupSideChanFlow*6, SupPlanFlow*6, SWPierLevel*8
!                      More Lake Superior regulation output strings

      INTEGER      MonthsChecked
!                  which month of OpSupply sequence we are in if p77Acompat=2.
!                    

      LOGICAL      BackCompat
!                  .TRUE. if program is trying to reproduce results from
!                  older upper lakes models.

      INTEGER      SupCritRnd
!                   1 = use P77A rounding for criterion calcs
!                   2 = use general settings for criterion calcs

      CHARACTER        BalancingFlow*8, SupBOMLevel*8
!                      The flow called for by the balancing equation,
!                      and beginning-of-month level.  Used for output.

      INTEGER          CritBOMRound, CritFlowRound, CritLevelRound
!                      Rounding values for how the criterions get calculated.
!                      These are currently set to the same as Plan 1977A,
!                      to replicate existing operational practice.  There
!                      is a block of commented code for setting them to 
!                      the general parameters, if practice is changed.    

      INTEGER          Q1887Method, CritBMethod
!                      These specify which variation in computational 
!                      methods to use for Criterions B and C.
!                      If Q1887Method is 1, use the old way (get pre-project
!                      by popping the Plan 1977A forecasted level into 
!                      the 1887 equation.  If Q1887Method is 2, the sub
!                      GetSupPreProject solves for the Criterion C 
!                      iteratively, based on the BOP level and water 
!                      balance terms.  Either way, the trigger remains
!                      the Plan 1977A forecasted level.  If CritBMethod is
!                      1, then the Criterion B flow is allowed to fall below
!                      the pre-project flow (old way).  If CritBMethod is 2,
!                      then the Criterion B flow is notallowed to be less
!                      than the pre-project flow.
      real(wp) :: CritCThresholdEl
!                      Elevation at which minimum flow becomes pre-project - 
!                      specified as 183.40 in the Orders of Approval.
      CHARACTER*4      P4
!                      For some reason one compiler wanted to pad a common block
!                      by 4 bytes

      real(wp) :: MPFT, HALF, ONE, ZERO
      INTEGER      TEN, FTPMI, INPFT, SecsInDay, MinInt, MaxInt,         &
     &             MissingVal
      INTEGER      SecsInUniformMon
!                  Used for calculating dT, and converting storage to
!                  flow. The number of seconds in a uniform month,  
!                  2629746, is taken from the average month length 
!                  according to the Gregorian calendar, which calls 
!                  for 97 leap days every 400 years. So the mean 
!                  length of a month works out to:
!                  (400 * 365 + 97) days * (24 * 60 * 60) seconds/day
!                  --------------------------------------------------
!                                  (400 * 12) months    
!                   = 2629746 seconds/month 
!                  Uniform quarter-month length taken as 1/4 of this.

      DATA P77AAValue      /  18580.0_wp /
      DATA P77AMonthsToAve / 12 * 5 /
      DATA P77AMinimumFlow / 12 * 1560 /
      DATA P77AMaxChange   / 850 /
      DATA P77AWinterMax    /2410/
      DATA P77AUniformMonth /.TRUE./
      DATA P77ALowTrigger, P77AHighTrigger / 183.19_wp, 183.71_wp /
      DATA P77ASup05NBS    /   790,  1730,  3510,  6710,  8160,  7110,  &
     &                        5750,  4900,  5040,  3340,  2550,   680 /
      DATA P77ASup50NBS    /  -400,   250,  1190,  4160,  5240,  4390,  &
     &                        3620,  2780,  2010,  1080,   420,  -650 /
      DATA P77ASup95NBS    / -1470,  -850,  -590,  2070,  2580,  2240,  &
     &                        2010,   960,  -200, -1020, -1420, -2070 /
      DATA P77AMH05NBS     /  4130,  4980,  9260, 12230, 11330,  9060,  &
     &                        6340,  4530,  4930,  3680,  4020,  4050 /
      DATA P77AMH50NBS     /  1420,  2460,  5100,  7990,  7020,  5690,  &
     &                        3540,  1500,   740,  -110,   930,   710 /
      DATA P77AMH95NBS     /  -650,   400,  1640,  4530,  3680,  3200,  &
     &                        1440, -1160, -2290, -2860, -2010, -2150 /
      DATA P77AStC50NBS    /   140,   170,   250,   230,   140,    80,  &
     &                          60,    30,    30,    30,    60,   110 /
      DATA P77AErie50NBS   /   620,   960,  2010,  1900,  1250,   790,  &
     &                          80,  -370,  -570,  -680,  -200,   450 /
      DATA P77ALLOgokiDiv   / 12 *  159 /
      DATA P77AChicagoDiv         / 12 *  -91 /
      DATA P77AWellandFlow  / 12 * -261 /
      DATA P77AStMarysRetrd / 12 * 0 /
      DATA P77AStClairRetrd / 590, 480, 110, 110, 7 * 0, 110 /
      DATA P77ADetroitRetrd / 650, 510, 110, 8 * 0, 140 /
      DATA P77ANiagaraRetrd / 110, 140,  80, 140, 0, 60,                &
     &                        140, 110,  80,  60, 0,  0 /
      DATA P77ABaseFlow     / 2010, 2010, 1980, 1980, 2180, 2350,       &
     &                        2490, 2460, 2490, 2440, 2440, 2040  /
      DATA P77ASupTargetLev /                                           &
     &               183.34_wp, 183.27_wp, 183.22_wp, 183.20_wp,        &
     &               183.28_wp, 183.38_wp, 183.46_wp, 183.50_wp,        &
     &               183.52_wp, 183.51_wp, 183.47_wp, 183.42_wp /
      DATA P77ASupRangeFact /                                           &
     &               0.13_wp,   0.13_wp,   0.13_wp,   0.13_wp,          &
     &               0.15_wp,   0.14_wp,   0.14_wp,   0.13_wp,          &
     &               0.13_wp,   0.12_wp,   0.13_wp,   0.12_wp /
      DATA P77AMHTargetLev /                                            &
     &               176.38_wp, 176.35_wp, 176.35_wp, 176.40_wp,        &
     &               176.51_wp, 176.59_wp, 176.65_wp, 176.66_wp,        &
     &               176.62_wp, 176.57_wp, 176.49_wp, 176.45_wp /
      DATA P77AMHRangeFact /                                            &
     &               0.35_wp,   0.35_wp,   0.34_wp,   0.36_wp,          &
     &               0.36_wp,   0.37_wp,   0.37_wp,   0.38_wp,          &
     &               0.38_wp,   0.37_wp,   0.37_wp,   0.37_wp /
      DATA P77ASPK, P77ASPYM, P77ASPA, P77ASPB, P77ASPWT, P77ASPC /     &
     &     824.7_wp, 181.43_wp,1.5_wp, 0._wp,   1._wp,      0._wp /
      DATA P77AMHK, P77AMHYM, P77AMHA, P77AMHB, P77AMHWT, P77AMHC /     &
     &      46.4_wp,165.83_wp,2.00_wp, 0.50_wp, 0.50_wp, -0.02_wp /
      DATA P77ASCK, P77ASCYM, P77ASCA, P77ASCB, P77ASCWT, P77ASCC /     &
     &      70.7_wp, 165.94_wp,2.00_wp,0.50_wp,  1.00_wp,-0.01_wp /
      DATA P77AERK, P77AERYM, P77AERA, P77AERB, P77AERWT, P77AERC /     &
     &     616.7_wp, 169.75_wp,1.50_wp,0.00_wp,  1.00_wp,   0._wp /  
      DATA P77ACGIP / 171.16_wp /
      DATA P77AMidLakesInc, P77AMidLakesTol / 40, -2 /
      DATA P77AMonthRound, P77AFlowRound, P77ALevelRound / -2, 0, -4 /

      DATA P77ACodes  / '                        ' /  
      DATA SupBOMLevel, SWPierLevel / 2*'        ' /
      DATA SupPreProject, SupGateFlow, SupPlanFlow,                     &
     &     SupSideChanFlow, BalancingFlow / 4*'      ','        ' /
!

      DATA BackCompat / .FALSE. /


      DATA SWPGateEqA /                                                 &
     &   33.12_wp, 66.13_wp, 99.05_wp, 140.17_wp, 181.23_wp,            &
     &   227.17_wp, 272.27_wp, 315.35_wp,                               &
     &   340.25_wp, 355.40_wp, 372.39_wp,                               &
     &   383.33_wp, 397.80_wp, 413.62_wp,                               &
     &   427.08_wp, 437.69_wp /
!
      DATA SWPGateEqB / 180.59_wp, 180.47_wp, 180.54_wp, 180.70_wp,     &
     &   180.84_wp, 180.97_wp, 181.09_wp,                               &
     &   181.18_wp, 181.21_wp, 181.21_wp,                               &
     &   181.22_wp, 181.21_wp, 181.21_wp,                               &
     &   181.22_wp, 181.22_wp, 181.21_wp /

      DATA OldSWPA / 27.6027_wp, 18.9144_wp, 17.6510_wp, 11.4080_wp,    &
     &   17.2385_wp, 14.9971_wp, 8.7544_wp,                             &
     &   9.9005_wp, 11.8851_wp, 20.3023_wp,                             &
     &   16.7230_wp, 13.6718_wp /
      DATA OldSWPB / 1.1534_wp, 1.1060_wp,                              &
     &   1.0991_wp, 1.0648_wp, 1.0965_wp,                               &
     &   1.0842_wp, 1.0501_wp, 1.0564_wp,                               &
     &   1.0672_wp, 1.1132_wp, 1.0936_wp,                               &
     &   1.0771_wp /
      DATA OldSWPC / 0.27_wp /


      DATA EC2000SWPA, EC2000SWPB / 1.046385_wp, 0.10015_wp /
      DATA EC2000SWPC /                                                 &
     &   8.152_wp, 8.161_wp, 8.163_wp, 8.158_wp, 8.129_wp,              &
     &   8.129_wp, 8.125_wp, 8.124_wp,                                  &
     &   8.125_wp, 8.126_wp, 8.118_wp, 8.127_wp /

      DATA EC2008SWPA, EC2008SWPB / 1.033806_wp, 0.08622_wp /
      DATA EC2008SWPC /                                                 &
     &   5.916_wp, 5.923_wp, 5.925_wp, 5.923_wp, 5.897_wp,              &
     &   5.897_wp, 5.892_wp, 5.891_wp,                                  &
     &   5.894_wp, 5.890_wp, 5.886_wp, 5.884_wp /


      DATA CWOvertopElev /184.0_wp/
!     Assumed from actual observed intermittent overtopping at a daily level 
!     of 183.91 (from Rob Caldwell/David Fay (2008))
      DATA CWMinimumFreeboard /12 * 0.10_wp/
      DATA SideChanMaxCapacity /12 * 2320/

      integer, private :: i

      DATA (SPOPNBS07( 1,I),I=1,6) / -360,  280, 1300, 4200, 5200, 4410/
      DATA (SPOPNBS07( 2,I),I=1,6) /  260, 1280, 4180, 5180, 4390, 3680/
      DATA (SPOPNBS07( 3,I),I=1,6) / 1300, 4200, 5200, 4410, 3700, 2780/
      DATA (SPOPNBS07( 4,I),I=1,6) / 4150, 5150, 4360, 3650, 2730, 1980/
      DATA (SPOPNBS07( 5,I),I=1,6) / 5240, 4450, 3740, 2820, 2070, 1140/
      DATA (SPOPNBS07( 6,I),I=1,6) / 4410, 3700, 2780, 2030, 1100,  510/
      DATA (SPOPNBS07( 7,I),I=1,6) / 3680, 2760, 2010, 1080,  490, -660/
      DATA (SPOPNBS07( 8,I),I=1,6) / 2750, 2000, 1070,  480, -670, -390/
      DATA (SPOPNBS07( 9,I),I=1,6) / 2010, 1080,  490, -660, -380,  260/
      DATA (SPOPNBS07(10,I),I=1,6) / 1080,  490, -660, -380,  260, 1280/
      DATA (SPOPNBS07(11,I),I=1,6) /  530, -620, -340,  300, 1320, 4220/
      DATA (SPOPNBS07(12,I),I=1,6) / -630, -350,  290, 1310, 4210, 5210/

      DATA (MHOPNBS07( 1,I),I=1,6) / 1630, 2470, 5220, 8040, 7070, 5760/
      DATA (MHOPNBS07( 2,I),I=1,6) / 2530, 5280, 8100, 7130, 5820, 3710/
      DATA (MHOPNBS07( 3,I),I=1,6) / 5220, 8040, 7070, 5760, 3650, 1560/
      DATA (MHOPNBS07( 4,I),I=1,6) / 8140, 7170, 5860, 3750, 1660,  910/
      DATA (MHOPNBS07( 5,I),I=1,6) / 7110, 5800, 3690, 1600,  850,   90/
      DATA (MHOPNBS07( 6,I),I=1,6) / 5760, 3650, 1560,  810,   50, 1110/
      DATA (MHOPNBS07( 7,I),I=1,6) / 3630, 1540,  790,   30, 1090,  830/
      DATA (MHOPNBS07( 8,I),I=1,6) / 1560,  810,   50, 1110,  850, 1630/
      DATA (MHOPNBS07( 9,I),I=1,6) /  810,   50, 1110,  850, 1630, 2470/
      DATA (MHOPNBS07(10,I),I=1,6) /   30, 1090,  830, 1610, 2450, 5200/
      DATA (MHOPNBS07(11,I),I=1,6) / 1110,  850, 1630, 2470, 5220, 8040/
      DATA (MHOPNBS07(12,I),I=1,6) /  860, 1640, 2480, 5230, 8050, 7080/

      DATA (SCOPNBS07( 1,I),I=1,6) /  190,  210,  270,  230,  150,  110/
      DATA (SCOPNBS07( 2,I),I=1,6) /  220,  280,  240,  160,  120,   90/
      DATA (SCOPNBS07( 3,I),I=1,6) /  270,  230,  150,  110,   80,   50/
      DATA (SCOPNBS07( 4,I),I=1,6) /  220,  140,  100,   70,   40,   30/
      DATA (SCOPNBS07( 5,I),I=1,6) /  150,  110,   80,   50,   40,   50/
      DATA (SCOPNBS07( 6,I),I=1,6) /  100,   70,   40,   30,   40,   60/
      DATA (SCOPNBS07( 7,I),I=1,6) /   80,   50,   40,   50,   70,  150/
      DATA (SCOPNBS07( 8,I),I=1,6) /   50,   40,   50,   70,  150,  190/
      DATA (SCOPNBS07( 9,I),I=1,6) /   30,   40,   60,  140,  180,  200/
      DATA (SCOPNBS07(10,I),I=1,6) /   40,   60,  140,  180,  200,  260/
      DATA (SCOPNBS07(11,I),I=1,6) /   60,  140,  180,  200,  260,  220/
      DATA (SCOPNBS07(12,I),I=1,6) /  150,  190,  210,  270,  230,  150/
!
      DATA (EROPNBS07( 1,I),I=1,6) /  770, 1040, 2030, 1880, 1300,  860/
      DATA (EROPNBS07( 2,I),I=1,6) / 1050, 2040, 1890, 1310,  870,  150/
      DATA (EROPNBS07( 3,I),I=1,6) / 2030, 1880, 1300,  860,  140, -330/
      DATA (EROPNBS07( 4,I),I=1,6) / 1860, 1280,  840,  120, -350, -530/
      DATA (EROPNBS07( 5,I),I=1,6) / 1270,  830,  110, -360, -540, -670/
      DATA (EROPNBS07( 6,I),I=1,6) /  840,  120, -350, -530, -660, -120/
      DATA (EROPNBS07( 7,I),I=1,6) /  110, -360, -540, -670, -130,  490/
      DATA (EROPNBS07( 8,I),I=1,6) / -350, -530, -660, -120,  500,  750/
      DATA (EROPNBS07( 9,I),I=1,6) / -520, -650, -110,  510,  760, 1030/
      DATA (EROPNBS07(10,I),I=1,6) / -660, -120,  500,  750, 1020, 2010/
      DATA (EROPNBS07(11,I),I=1,6) / -110,  510,  760, 1030, 2020, 1870/
      DATA (EROPNBS07(12,I),I=1,6) /  520,  770, 1040, 2030, 1880, 1300/
!
!
!     The following data corresponds to P77ACompat=3, in use from 199?-2007
      DATA (SPOPNBS06( 1,I),I=1,6) / -380,  300, 1280, 4210, 5260, 4470/
      DATA (SPOPNBS06( 2,I),I=1,6) /  280, 1260, 4190, 5240, 4450, 3650/
      DATA (SPOPNBS06( 3,I),I=1,6) / 1280, 4210, 5260, 4470, 3670, 2850/
      DATA (SPOPNBS06( 4,I),I=1,6) / 4210, 5260, 4470, 3670, 2850, 2080/
      DATA (SPOPNBS06( 5,I),I=1,6) / 5260, 4470, 3670, 2850, 2080, 1070/
      DATA (SPOPNBS06( 6,I),I=1,6) / 4460, 3660, 2840, 2070, 1060,  510/
      DATA (SPOPNBS06( 7,I),I=1,6) / 3640, 2820, 2050, 1040,  490, -700/
      DATA (SPOPNBS06( 8,I),I=1,6) / 2830, 2060, 1050,  500, -690, -400/
      DATA (SPOPNBS06( 9,I),I=1,6) / 2060, 1050,  500, -690, -400,  280/
      DATA (SPOPNBS06(10,I),I=1,6) / 1050,  500, -690 ,-400,  280, 1260/
      DATA (SPOPNBS06(11,I),I=1,6) /  520, -670, -380,  300, 1280, 4210/
      DATA (SPOPNBS06(12,I),I=1,6) / -670, -380,  300, 1280, 4210, 5260/
!
      DATA (MHOPNBS06( 1,I),I=1,6) / 1490, 2480, 5220, 8090, 7100, 5780/
      DATA (MHOPNBS06( 2,I),I=1,6) / 2560, 5300, 8170, 7180, 5860, 3680/
      DATA (MHOPNBS06( 3,I),I=1,6) / 5220, 8090, 7100, 5780, 3600, 1560/
      DATA (MHOPNBS06( 4,I),I=1,6) / 8090, 7100, 5780, 3600, 1560,  880/
      DATA (MHOPNBS06( 5,I),I=1,6) / 7150, 5830, 3650, 1610,  930,   80/
      DATA (MHOPNBS06( 6,I),I=1,6) / 5760, 3580, 1540,  860,   10, 1010/
      DATA (MHOPNBS06( 7,I),I=1,6) / 3560, 1520,  840,  -10,  990,  770/
      DATA (MHOPNBS06( 8,I),I=1,6) / 1550,  870,   20, 1020,  800, 1480/
      DATA (MHOPNBS06( 9,I),I=1,6) /  890,   40, 1040,  820, 1500, 2490/
      DATA (MHOPNBS06(10,I),I=1,6) /   10, 1010,  790, 1470, 2460, 5200/
      DATA (MHOPNBS06(11,I),I=1,6) / 1050,  830, 1510, 2500, 5240, 8110/
      DATA (MHOPNBS06(12,I),I=1,6) /  830, 1510, 2500, 5240, 8110, 7120/

!
      DATA (SCOPNBS06( 1,I),I=1,6) /  160,  190,  270,  220,  150,  100/
      DATA (SCOPNBS06( 2,I),I=1,6) /  190,  270,  220,  150,  100,   70/
      DATA (SCOPNBS06( 3,I),I=1,6) /  270,  220,  150,  100,   70,   40/
      DATA (SCOPNBS06( 4,I),I=1,6) /  220,  150,  100,   70,   40,   30/
      DATA (SCOPNBS06( 5,I),I=1,6) /  150,  100,   70,   40,   30,   40/
      DATA (SCOPNBS06( 6,I),I=1,6) /   90,   60,   30,   20,   30,   50/
      DATA (SCOPNBS06( 7,I),I=1,6) /   70,   40,   30,   40,   60,  140/
      DATA (SCOPNBS06( 8,I),I=1,6) /   30,   20,   30,   50,  130,  150/
      DATA (SCOPNBS06( 9,I),I=1,6) /   30,   40,   60,  140,  160,  190/
      DATA (SCOPNBS06(10,I),I=1,6) /   30,   50,  130,  150,  180,  260/
      DATA (SCOPNBS06(11,I),I=1,6) /   60,  140,  160,  190,  270,  220/
      DATA (SCOPNBS06(12,I),I=1,6) /  140,  160,  190,  270,  220,  150/
!
      DATA (EROPNBS06( 1,I),I=1,6) /  700, 1000, 2040, 1870, 1300,  840/
      DATA (EROPNBS06( 2,I),I=1,6) / 1010, 2050, 1880, 1310,  850,  130/
      DATA (EROPNBS06( 3,I),I=1,6) / 2040, 1870, 1300,  840,  120, -350/
      DATA (EROPNBS06( 4,I),I=1,6) / 1860, 1290,  830,  110, -360, -530/
      DATA (EROPNBS06( 5,I),I=1,6) / 1290,  830,  110, -360, -530, -660/
      DATA (EROPNBS06( 6,I),I=1,6) /  820,  100, -370, -540, -670, -160/
      DATA (EROPNBS06( 7,I),I=1,6) /  100, -370, -540, -670, -160,  460/
      DATA (EROPNBS06( 8,I),I=1,6) / -360, -530, -660, -150,  470,  690/
      DATA (EROPNBS06( 9,I),I=1,6) / -530, -660, -150,  470,  690,  990/
      DATA (EROPNBS06(10,I),I=1,6) / -670, -160,  460,  680,  980, 2020/
      DATA (EROPNBS06(11,I),I=1,6) / -140,  480,  700, 1000, 2040, 1870/
      DATA (EROPNBS06(12,I),I=1,6) /  490,  710, 1010, 2050, 1880, 1310/


      DATA OPLLO / 113, 99, 85, 79, 181, 252, 207, 170,    &
     &   133, 127, 122, 125 / 
!
      DATA OPWelland  / -218, -215, -218, -252, -255, -255,&
     &   -249, -252, -258, -261, -258, -227 / 
!
      DATA SCRRet / 590,480,110,110,0,0,0,0,0,0,0,110 / 
!
      DATA (DETRet(I),I= 1,12)  / 650,510,110, 0, 0,0,0,0,0,0,0,140 / 
!
      DATA (NIARet(I),I= 1,12) / 110,140,80,140,0,60,230,140,80,60,0,0/
      DATA OPChicago / -91 /
      DATA MonthsChecked / 0 /

      DATA MPFT, HALF, ONE, ZERO /0.3048_wp, 0.5_wp, 1.0_wp, 0.0_wp/
      DATA TEN, FTPMI, INPFT, SecsInDay /10, 5280, 12, 86400/
      DATA SecsInUniformMon / 2629746 /
      DATA MissingVal / -9999 /

      DATA LStoSWP, SupCritRnd /2 * -9999/
      DATA CritBOMRound, CritFlowRound, CritLevelRound /3 * -9999/
      DATA Q1887Method, CritBMethod /2 * -9999/
      DATA CritCThresholdEl / 183.40_wp /

      end module outflow_mod
