!
!-----------------------------------------------------------------------
!
!
      FUNCTION Round( Value, PowerOfTen )
! 
!     Returns Value rounded to the PowerOfTen position.
!
! modules
      USE par_kind
!
      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: Round
      real(wp) :: Value
!                  The 64-bit number to be rounded.
!
      INTEGER      PowerOfTen
!                  The digit which Value is to be rounded to.  3 means
!                  round the thousand, 2 rounds to hundredth, 1 to
!                  the tens, 0 to the ones position, -1 is to the tenth,
!                  -2 to the hundredth, -3 to the thousandth, -4 to 
!                  the ten-thousandth, -5 to the hundred-thousandth, and
!                  -6 to the millionth.  No other numbers are allowed, 
!                  since it refers to an element of array ExpFactor.
!
!     Declarations regarding local variables or functions:
!
      real(wp) :: tmp
!                  scratch variable
!
      real(wp) :: Constant(0:1)
!                  The number to add for rounding purposes.  Even
!                  numbers add first element, odd numbers add 2nd.
!
      real(wp) :: ExpFactor(-6:3)
!                  This is the array containing the position which is
!                  to be rounded to.  Array index set up to correspond
!                  to values of PowerOfTen (ie, 10E-3 to 10E+3).
!
!     Declarations regarding COMMON block variables:
!
!
!     
      DATA Constant / 0.4999999_wp, 0.5000001_wp/
!
!     These values seem in reverse order, because we multiply by them
      DATA ExpFactor /1000000._wp, 100000._wp,     &
     &    10000._wp, 1000._wp, 100._wp,            &
     &    10._wp, 1._wp, 0.1_wp, 0.01_wp, 0.001_wp/
!
!
      IF( PowerOfTen .GE. -6 .AND. PowerOfTen .LE. 3) THEN
!
!        Use Lake Survey rounding technique 
         tmp = ABS(Value * ExpFactor(PowerOfTen))
!
         Round = INT( Constant(  MOD( INT(tmp),2 ) ) + tmp) /  &
     &      SIGN(ExpFactor(PowerOfTen), Value)
!
      ELSE IF ( PowerOfTen .EQ. -99 ) THEN
!        Do no rounding 
         Round = Value
      ELSE
         STOP 'Round out of range'
      ENDIF
      END

