      subroutine compute_saint_marys(NewMonth, verbose)
!      
!     This is the main Lake Superior module.  This is where we decide
!     what regulation plan to use, how to handle forced flows or 
!     gates, how to solve for EOP level, etc.  This is a somewhat less
!     dismal as of Dec08 mess.  
!
!modules
      use par_kind
      use lakesup_mod
      use outflow_mod

      implicit none

! arguements
      integer verbose

! locals
      INTEGER       I, K
      real(wp) :: ROUND
!                   Returns value rounded to a certain accuracy,
!                   according to the old Lake Survey method.
      external round

      real(wp) :: MHEndofMonthL, SCEndofMonthL, EREndofMonthL,          &
     &              SPEndofMonthL, SPMonthlyMeanF
!                   The end-of-month levels and mean monthly flow, 
!                   computed by TallyMeans, that get used as beginning
!                   of month levels and previous month's outflow by 
!                   Superior regulation plans.              
!                   Note that these storages are actually aliased to
!                   memory in FWRK.
      INTEGER       MaxIterations
!
      real(wp) :: Tolerance
!                   Threshold at which the difference in outflow between
!                   two iterations becomes close enough, in m3s.

      LOGICAL       NewMonth
!                   (input) .TRUE. if first time routing through the month
      INTEGER       SecsInMonth, NumGates

      real(wp) :: CWFreeBoard
      real(wp), EXTERNAL :: CompWorksFreeboard
!                   Returns amount of clearance before overtopping
!                   the lowest point of the Compensating Works
      INTEGER       SPNWB, MHNWB, SCNWB, ERNWB, MHRet, SCRet, ERRet
!                   Values that hold routing data appropriate to setting
!                   of P77ACompat.  Not used after overtopping calls.
      real(wp), external :: getarea
      real(wp) :: SWPier, GateFlow
!                   Level at SW Pier, and flow through gates.
      real(wp) :: TmpA
!                   Scratch variables
      CHARACTER*1   EOPType
!                	  Returns 'M' if period ends a month, or 'Q' if period
!                   ends a quarter-month, else returns 'W', or 'D'.
      INTEGER       EOILevels2Ago
!                   Used for tracking iterative convergence
      real(wp) :: SPIMLV, SWPI, GateFlowI
!                   Mean levels of Lake Superior and SW Pier for an increment,
!                   mean gated flow for an increment



      DATA MaxIterations /9999/
!
!     Start with  value of SecsInUniformMon.  If using actual month
!     length, will get overridden each month.
      DATA SecsInMonth / 2629746 /


      NewMonth      = .true.
! FD      SPNWB=-247_wp !m3/s ?

      SPEndofMonthL = SPBOMLev
      MHEndofMonthL = MHBOMLev
      SCEndofMonthL = SCBOMLev  
      EREndofMonthL = ERBOMLev


!     Consider equations solved if iterate to one FlowRound (ie, if
!     if FlowRound set to 1, must be 10 m3s or less difference between
!     iterations).  If FlowRound == -99, use .1 m3s.
      Tolerance = 10.0_wp**FlowRound
      IF( Tolerance .LT. 0.1_wp ) Tolerance = 0.1_wp
!
!     Initialize BOP levels as the EOP levels from previous period
      SPBOP = SPEOP

!     Let's save ourselves some grief by checking a little bit here:
      IF( NINT(SPBOP) .LE. ZERO .OR. SPBOP .GT. 1000) THEN
         WRITE(*,4289) 'Superior BOP level', PeriodEnd(1,1),            &
     &       MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1), SPBOP
         STOP
      ENDIF
 4289 FORMAT(' Unbelievable ', A, ' for period ending ', I2, A3, I5,    &
     &   ': ', F17.5)

!     Initialize mean lake level registers for period.  See 
!     function RouteMiddleLakes() for an explanation of the 
!     arithmetic. 
      SPMLV = SPBOP * HALF
!
!     Initialize EOI values with BOP values; they will be rolled 
!     into BOI values at beginning of loop through increments.
      SPEOI = SPBOP

!     Initialize period-average discharge values with 0.0D.  They 
!     will be computed as the mean of the outflows computed for 
!     each increment.  (The outflow for each increment is 
!     calculated using the mean level for the increment).
      STMRIVFLOW = ZERO

      IF( Verbose .GT. 0 ) THEN
         WRITE(*,'(2X,3(A,F15.8),A,F15.3,A,F9.3,A,F9.3)')               &
     &      ' SPBOP=', SPBOP, ' Superior NTS=', SPNTS,                  &
     &      ' Ice=', SMICE, ' dT=', dT(1), ' SupGatesOpen=',            &
     &      SupGatesOpen, ' ComputedNonGate=',ComputedNonGate
         WRITE(*,'(2X,5(A,F15.8))')' SideChannel=',SupSideChannel,      &
     &      ' Forced Flow=', SupForceFlow, ' Forced Gates=',            &
     &      NumSupForceGates, ' Deviation Flow=', SupDevFlow,           &
     &      ' SupForceEOPLev=', SupForceEOPLev
         WRITE(*,'(2X,3(A,F15.8))')                                     &
     &         ' SPEOP=', SPEOP, ' SPMLV=', SPMLV, ' StMRivFlow=',      &
     &         StMRivFlow
      ENDIF

!        Numerous Lake Superior reg plans need to know BOM levels and 
!        the length of the month.  Just figure them out here whether 
!        really needed or not to avoid duplicate effort later.
!        
         IF(NewMonth) THEN
            IF(MonthLength .EQ. 2)  THEN
!              Actual Month length.  (Don't need number of days in year, 
!              so just use K as scratch)
               CALL NUMDAYS(PeriodEnd(4,1), PeriodEnd(3,1), K, I)
               SecsInMonth = I * SecsInDay
            ENDIF
!        
!           TallyMeans populates SPEndofMonthL (i.e., FWRK(30,5)) and
!           similar values for the middle lakes.  For daily, qmonthly,
!           or monthly routing we can just use the value available there
!           when we hit a new month.  The weekly boundary will not quite
!           match up most of the time, and needs to be adjusted.
!           XX Monthly-Weekly Boundary Fudge:
!           XX If its weekly routing, just use level at end of last
!           XX week as EOM level.  Could estimate based on BOP, EOP
!           XX of last week. 
            IF(RoutingInterval(1) .EQ. 2) SPEndofMonthL = SPEOP
            IF(RoutingInterval(2) .EQ. 2) THEN
               MHEndofMonthL = MHEOP 
               SCEndofMonthL = SCEOP 
               EREndofMonthL = EREOP 
            ENDIF
         ENDIF

         IF(NewMonth) THEN
!
            IF(EREndofMonthL .LE. 99 .OR. EREndofMonthL .GE. 999)  &
     &         WRITE(*,4289) ' Erie BOM level', PeriodEnd(1,1),    &
     &            MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),     &
     &            EREndofMonthL
            IF(SCEndofMonthL .LE. 99 .OR. SCEndofMonthL .GE. 999)  &
     &         WRITE(*,4289) ' St. Clair BOM level',               &
     &            PeriodEnd(1,1), MonthAbbrev(PeriodEnd(3,1)),     &
     &            PeriodEnd(4,1), SCEndofMonthL
            IF(MHEndofMonthL .LE. 99 .OR. MHEndofMonthL .GE. 999)  &
     &         WRITE(*,4289) ' Michigan-Huron BOM level',          &
     &            PeriodEnd(1,1), MonthAbbrev(PeriodEnd(3,1)),     &
     &            PeriodEnd(4,1), MHEndofMonthL
            IF(SPEndofMonthL .LE. 99 .OR. SPEndofMonthL .GE. 999)  &
     &         WRITE(*,4289)' Superior BOM level', PeriodEnd(1,1), &
     &            MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),     &
     &            SPEndofMonthL
            IF( NINT(NominalStMarys) .LE. ZERO)                    &
     &         WRITE(*,4289) 'monthly mean St. Marys River flow',  &
     &            PeriodEnd(1,1), MonthAbbrev(PeriodEnd(3,1)),     &
     &            PeriodEnd(4,1), SPMonthlyMeanF

!           Now sort out ways to remain compatible with existing
!           procedures.  
            CALL SetUpSupOperationalSupplies(P77ACompat, PeriodEnd(3,1),&
     &         SPNTS, SPNWB, MHNWB, SCNWB, ERNWB, MHRet, SCRet, ERRet)
!
!           XX Modeling/Operational Compatibility Issue:
!           XX    Operational practice sets outflow from the previous
!           XX month to the 'nominal regulated flow', calculated using
!           XX standardized supplies, diversions, and retardations, 
!           XX assuming the outflow is constant over the month, and
!           XX rounded to 10m3s.  On rare occasions (such as when the 
!           XX lake level changes dramatically during a month), the
!           XX regulators use engineering judgement to come up with a
!           XX better estimate of the previous month's outflow.  When
!           XX this situation occurs and how the better flow is 
!           XX determined are not defined adequately to be modeled.
!           XX So we'll simply take last month's 'nominal regulated'
!           XX (i.e. 'plan') flow, and round it to 10m3s.
!    
!
            CALL Plan1977A( PeriodEnd(3,1), PeriodEnd(4,1),       &
     &         Round(NominalStMarys, 1), SPNWB,                   &
     &         SPEndofMonthL, MHEndofMonthL, SCEndofMonthL,       &
     &         EREndofMonthL, NominalStMarys, SupGatesOpen,       &
     &         verbose)
! 
            CALL SupCriterions( PeriodEnd(4,1), PeriodEnd(3,1),   &
     &         SPNWB, MHNWB, SCNWB, ERNWB, MHRet, SCRet, ERRet,   &
     &         SPEndofMonthL, MHEndofMonthL, SCEndofMonthL,       &
     &         EREndofMonthL, DBLE(SecsInMonth),NominalStMarys,   &
     &         SWPier, ComputedNonGate, SupGatesOpen, P77ACodes,  &
     &         verbose)


!           At this point, the number of gates and non-gated flow
!           for the month are established.
            IF( verbose .GT. 0 )THEN
               WRITE( *, '(A,A,I5,A,F7.2,A,D18.10)')               &
     &               ' Final Plan1977A results for month of ',     &
     &               MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),  &
     &               ': desired gates=', SupGatesOpen,             &
     &               ': desired regulated flow=', NominalStMarys
            ENDIF            
            SWPier = ZERO
            GateFlow = ZERO
            LastComputedNonGate = ComputedNonGate 

         ELSE
!           Running under Plan 1977A, but interval does not include a new month.  Keep previous settings
            IF( verbose .GT. 0 )THEN
               WRITE( *, '(A,A,D18.10,A,F9.2)')                   &
     &               ' Still in same month, so using previous ',  &
     &               'regulated flow=', LastComputedNonGate,      &
     &               ' and previous gates=', PreviousGates
            ENDIF
            SupGatesOpen = PreviousGates 
            ComputedNonGate = LastComputedNonGate 
         ENDIF



!     Forced Gates 
      IF( NINT(NumSupForceGates) .NE. MissingVal) THEN
!
!        Dump at run-time if computation not supported yet
         IF( NINT(SupForceFlow) .NE. MissingVal ) THEN 
            WRITE(*,'(A,I2,A3,I5,A)')                                 &
     &         ' Unsupported computation for ', PeriodEnd(1,1),       &
     &         MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),           &
     &         ': Forcing gates and total flow.'
            WRITE(*,'(A,I2,A3,I5,A)')                                 &
     &         ' Forced Sidechannel: ', SupSideChannel,               &
     &         MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),           &
     &         ': Forcing gates and either total flow or sidechannel.'
            stop 'pb in forced gates'
         ENDIF
!        Issue a warning or log entry if overriding a real value
         IF( NINT(SupGatesOpen) .NE. MissingVal ) THEN
            WRITE( *, '(A,I2,A3,I5,A,F5.2,A,F5.2,A)')             &
     &         ' Warning for ', PeriodEnd(1,1),                   &
     &         MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),       &
     &         ': forcing ', NumSupForceGates,                    &
     &         ' over reg plan setting of ', SupGatesOpen
         ENDIF
         P77ACodes = '   Forced Gates         ' 
         SupGatesOpen = NumSupForceGates
      ENDIF
!
!     Check for forced flow condition
      IF(NINT(SupForceFlow) .NE. MissingVal)THEN
         STMRIVFLOW = SupForceFlow
!        XX Should we make a stab here regarding how many gates
!        XX are required to achieve the forced flow?  Not Yet.
!
!        Dump at run-time if computation not supported yet
         IF( NINT(SupSideChannel) .NE. MissingVal .OR.                  &
     &      NINT(NumSupForceGates) .NE. MissingVal ) THEN 
            WRITE(*,'(A,I2,A3,I5,A)')                                   &
     &         ' Unsupported computation for ', PeriodEnd(1,1),         &
     &         MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),             &
     &         ': Forcing total flow and either gates or sidechannel.'
            WRITE(*,'(2(A,F9.2))') ' NumSupForceGates=',                &
     &         NumSupForceGates, '    SupSideChannel',SupSideChannel
            stop 'pb in forced gates 2'
         ENDIF
         SupGatesOpen = -1.0
         ComputedNonGate = MissingVal
         WRITE(SupSideChanFlow,'(I6)')NINT(SupSideChannel)
         P77ACodes = '   Forced Flow          ' 
         WRITE(SupGateFlow, '(I6)') MissingVal
         IF( verbose .GT. 0 ) THEN
            WRITE( *, 2256) SupForceFlow
         ENDIF
      ENDIF
 2255 FORMAT( 'Now forget whatever Superior outflow may have been ',    &
     &   'computed using a regulation plan!')
 2256 FORMAT( 'Forcing St Marys River flow to specified value:',        &
     &   F12.3, ' m3s') 
!
      IF( verbose .GT. 0 )THEN
         WRITE( *, '(A,D18.10,A,I18)')                                  &
     &         ' Check 1: SupSideChannel= ', SupSideChannel,            &
     &         '  MissingVal=', MissingVal
      ENDIF
      IF( NINT(SupSideChannel) .EQ. MissingVal) THEN 
!        Not forcing a sidechannel flow for this period
         SupSideChannel = ComputedNonGate
         IF( verbose .GT. 0 )THEN
            WRITE( *, '(A,A,D18.10)')                                   &
     &         ' Not forcing a sidechannel flow for this period - ',    &
     &         ' take SupSideChannel as ComputedNonGate = ',            &
     &         SupSideChannel
         ENDIF
      ELSE
         WRITE( *, '(A,I2,A3,I5,2(A,F7.2),A)')                          &
     &      ' Warning for ', PeriodEnd(1,1),                            &
     &      MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),                &
     &      ': forcing ', SupSideChannel,                               &
     &      ' over reg plan sidechannel of ', ComputedNonGate
         IF ( P77ACodes(1:8) .EQ. '   Force' ) THEN
            P77ACodes(17:24) = 'SideChan' 
         ELSE
            P77ACodes = '   Forced Sidechannel   ' 
         ENDIF
      ENDIF

!     Check to see if outflows from a regulation plan jeopardize 
!     the structure. All plan flows are known and we now have to verify
!     these with physical capacities.  (See notes from 8 Oct 2008)
!------------------------------------------------------------------------
! Bookmark:PChecks

!        See if Lake Superior level causes overtopping Comp Works.
!        Just use BOP level for simplicity.
         CWFreeBoard = CompWorksFreeboard(                              &
     &      PeriodEnd(3,1), SPBOP, CWOvertopElev)

         IF (CWFreeBoard .LT. CWMinimumFreeboard(PeriodEnd(3,1))) THEN
            WRITE( *, '(A,I2,A3,I5,A,F10.6,A,F10.6,A)')                 &
     &         ' Warning - on ', PeriodEnd(1,1),                        &
     &         MonthAbbrev(PeriodEnd(3,1)), PeriodEnd(4,1),             &
     &         ' estimate only ',CWFreeBoard,                           &
     &         'm of freeboard but need ',                              &
     &         CWMinimumFreeboard(PeriodEnd(3,1)),                      &
     &         'm, so overriding regulated flow'
!
!
            CALL AdjustDueToCompOvertop(PeriodEnd(1,1), PeriodEnd(3,1), &
     &         PeriodEnd(4,1), dT(1), 3, DFLOAT(SPNWB),                 &
     &         SideChanMaxCapacity(PeriodEnd(3,1)), SupGatesOpen,       &
     &         ComputedNonGate, P77ACodes)
!
!           Now update the final value of side channel flow
            SupSideChannel = ComputedNonGate
!  
         ELSE
            IF( verbose .GT. 0 )THEN
               WRITE( *, '(4(A,F10.6))')                                &
     &           ' Superior level of ', SPBOP, 'm leaves ', CWFreeBoard,&
     &           'm of freeboard, which meets requirement of ',         &
     &           CWMinimumFreeboard(PeriodEnd(3,1)),                    &
     &           'm before overtop at ', CWOvertopElev
            ENDIF
            SupSideChannel = ComputedNonGate
         ENDIF

      CALL CheckSideChannelLimits(SupSideChannel, PeriodEnd(3,1),       &
     & verbose)

      WRITE(SupSideChanFlow,'(I6)')NINT(SupSideChannel)
      IF( verbose .GT. 0 )THEN
         WRITE( *, '(A,A,A,D18.10,A)')                                  &
     &      ' Setting SupSideChanFlow= ', SupSideChanFlow, ' (',        &
     &      SupSideChannel, ')'
      ENDIF


!     Beginning of ACTUAL routing through Superior
!---------------------------------------------------------------------
!     See how flows are calculated.  
!
!     If outflow is allowed to vary within a routing period as a
!     function of lake level, then change-in-storage isn't linear,
!     and need to integrate over a number of increments while solving
!     simultaneous equations.
!
!     If assumed constant over the period (such as Plan77A/linear
!     certain gate setting will result in assumes a constant outflow 
!     over the month), then storage change is linear, and StmRivFlow 
!     equals NominalStMarys (or SupForceFlow).  
!
!     Traditional Superior regulation practice was to make the
!     simplifying assumption that the regulated outflow remains
!     constant during the month.  With outflow not a function
!     of head, the change-in-storage over the month is a straight
!     line, so knowing the outflow we can plug in the storage 
!     equation and solve for EOP level, then mean level.  This 
!     case (i.e., 1 increment) follows through with the general
!     logic in this loop, but occasionally fails to reproduce the 
!     exact sequence of rounds necessary to reproduce the results of 
!     previous models.  So if the relevant parameters imply that the
!     modeler is trying to replicate the errors of the older models,
!     the computation is handled separately as follows:
!   
      IF ( BackCompat .OR. NINT(SupForceFlow) .NE. MissingVal) THEN
!        Assume that need full backward compatibility with earlier 
!        models.  Linear change-in-storage, just compute EOP and 
!        mean levels, no increment loop needed.   Also applies to
!        forced flows, since constant outflow.
!          
         IF( verbose .GT. 0 ) THEN
            WRITE( *, '(A)') ' Performing'//                         &
     &         ' constant outflow calculation for Lake Superior'
         ENDIF
!
!        Forget gate settings, monthly average outflow defined as
!        the plan flow, so StmRivFlow = NominalStMarys, unless
!        forced flow or gates.
         IF(NINT(SupForceFlow)     .EQ. MissingVal .AND.             &
     &      NINT(NumSupForceGates) .EQ. MissingVal)                  &
     &      STMRIVFLOW = NominalStMarys
!
!        Sabotage significant digits in keeping with existing practice
         IF( verbose .GT. 0 ) THEN
            TmpA = GetArea(1,SPMLV)
            WRITE( *, 6234) Round(SPBOP, BOMRound), SPNTS,           &
     &         STMRivFlow, RoutingIncrement(1), dT(1), TmpA
         ENDIF
 6234    FORMAT( '   Linear change in storage: BOP=', F9.5,          &
     &      ' NTS=',F8.2, ' St Marys Flow=', F7.1, ' num inc=', I3,  &
     &      ' dT=', F10.1, ' Area=', D27.14)
!
         SPEOP=Round(SPBOP, BOMRound) + Round((SPNTS-STMRIVFLOW)*    &
     &      RoutingIncrement(1) *dT(1)/GETAREA(1,SPBOP), BOMRound)
         SPMLV = Round((SPEOP + SPBOP) * HALF, BOMRound)
         CALL GetSWPier( SPMLV, PeriodEnd(3,1), SupGatesOpen,        &
     &      NINT(SupSideChannel), BOMRound, LStoSWP, SWPier,         &
     &      GateFlow, verbose)
!
         IF( verbose .GT. 0 ) THEN
            WRITE( *, '(A,I5,A)') ' Re-computed gate flow '//        &
     &         'with levels rounded to ', FlowRound, ' places:' 
            WRITE( *, 2305) GateFlow, SPMLV, SWPier
         ENDIF
 2305    FORMAT( '   Gated flow computed as ', F9.4,                 &
     &      ', based on Sup/SW Pier levels of ', 2F9.4)
!
!        Update report fields
         WRITE(SWPierLevel, '(F8.3)') Round(SWPier, BOMRound)
         WRITE(SupGateFlow, '(I6)') NINT(Round(GateFlow, 0))
!
!        Do a continuity check to see if creating/destroying water
         SPSolutionCheck = ROUND(ABS( STMRIVFLOW - SPNTS +           &
     &      ((SPEOP - SPBOP) * GETAREA(1,SPMLV))/dT(1)), FlowRound)
         IF(SPSolutionCheck .GT. SPErrorFlow)                        &
     &      SPErrorFlow=SPSolutionCheck 
         IF( verbose .GT. 0 ) THEN
            WRITE(*, 8033) PeriodEnd(5,1), 1,'SP',                   &
     &         SPSolutionCheck, SolutionTest
         ENDIF
!        Value of SolutionTest assigned in CONTROL.FOR 
         IF(SPSolutionCheck .GT. SolutionTest ) THEN
               SPSolveErrors = SPSolveErrors + 1
            IF( verbose .GT. 0 ) THEN
               WRITE(*, 8072) PeriodEnd(5,1), 1, PeriodEnd(1,1),     &
     &            PeriodEnd(3,1), PeriodEnd(4,1)
            ENDIF
         ELSE
            IF( verbose .GT. 0 ) THEN
               WRITE(*, 8071) 1, PeriodEnd(5,1)
            ENDIF
         ENDIF
!        End of constant Superior outflow condition
!        Short-circuit the looping through increments and jump in to
!        spot after SPEOP and SPMLV get computed for non-linear
!        change in storage.
         GOTO 1233
      ENDIF

!     At this point, the outflow varies with lake level.  So have 
!     non-linear change-in-storage, and need to integrate outflow 
!     over RoutingIncrement(1) sub-divisions of a routing interval.
!     Within each increment solve simultaneous equations for 
!     outflow, mean increment level, and end-of-increment level.
!
      IF( verbose .GT. 0) THEN
         WRITE(*, '(A,I3,A)') ' Lake Superior outflow varies '//        &
     &      'during an interval; integrating over ',RoutingIncrement(1),&
     &      ' increments.'
      ENDIF
!
      SWPier = ZERO
      GateFlow = ZERO

      DO 1222 I=1, RoutingIncrement(1)            
!
!        Will do iterative solution of 3 simultaneous eqns:
!           mean level for increment = (SPBOI + SPEOI)/2
!           outflow= f(mean level) or SPOutI = f(SPBOI + SPEOI)/2
!           (SPEOI-SPBOI)*Area/dT = Net Total Supply - outflow
!          
!        Initialize BOI levels as EOI levels from prev increment
         SPBOI = SPEOI
!        Initialize the outflows with bogus number
         SPOutI = MissingVal
!        Initialize the mean level value with the initial level
         SPIMLV = SPBOI
!
         EOILevels2Ago = -999
!
!        Dump some debugging info if desired:
         IF( verbose .GT. 0 ) THEN
            TmpA = GetArea(1,SPIMLV)
            WRITE(*,'(A,I3,3(A,F15.8),A,D27.17)')                       &
     &         ' Increment ', I, ' SPBOI=', SPBOI, ' SPEOI=', SPEOI,    &
     &         ' SPIMLV=', SPIMLV, ' Area=', TmpA
         ENDIF

!           The number of gates and the non-gated flow are established.
!           Not assuming that outflow during a period is constant.
!           Must compute actual gated flow within a period, and add to 
!           non-gated flow to get St Marys flow.
!
            IF(verbose .GT. 0 ) THEN
               WRITE(*,'(4X,A,A,F5.2,A,I12,A,F12.4,A,I2)')              &
     &            '. . . Calling SolveGatesSide(',                      &
     &            'SupGatesOpen=', SupGatesOpen,                        &
     &            ', NINT(SupSideChannel)=', NINT(SupSideChannel),      &
     &            ', SPNTS=', SPNTS,                                    &
     &            ', LStoSWP=', LStoSWP
            ENDIF
            CALL SolveGatesSide(SupGatesOpen, NINT(SupSideChannel),     &
     &         PeriodEnd(3,1), SPBOI, SPNTS,                            &
     &         dT(1), LStoSWP, SPEOI, SPIMLV, SWPI, GateFlowI, SPOutI)
!           Accumulate SWPier means for increment, for later averaging
            SWPier = SWPier + SWPI
            GateFlow = GateFlow + GateFlowI
!
!           End of increment solution using Plan 1977A

!        Accumulate outflow for increment, for later averaging
         STMRIVFLOW = STMRIVFLOW + SPOutI

!
!        Now check that the simultaneous equations were solved
!        correctly.  Verify that the inflow plus NBS, minus the
!        average rate of change in storage during the period,
!        nearly equals the outflow for the period.  Store the 
!        difference in the SolutionCheck variables for debugging
!        info.  Note that this implemenation uses surface area as
!        BOI value.  Can easily fix this by introducing a new 
!        increment mean level if want to check variable surface 
!        area runs.
!----&------------------------------------------------------------------
         SPSolutionCheck = ROUND(ABS( SPOutI -  SPNTS +              &
     &      ((SPEOI - SPBOI) * GETAREA(1,SPEOI))/dT(1)), FlowRound)
         IF(SPSolutionCheck .GT. SPErrorFlow)SPErrorFlow=SPSolutionCheck
         IF(verbose .GT. 0 ) THEN
            WRITE(*, 8033) PeriodEnd(5,1), I,'SP',             &
     &         SPSolutionCheck, SolutionTest
         ENDIF
!        Value of SolutionTest assigned in CONTROL.FOR 
         IF(SPSolutionCheck .GT. SolutionTest ) THEN
            SPSolveErrors = SPSolveErrors + 1
            WRITE(*, 8072) PeriodEnd(5,1), I, PeriodEnd(1,1),  &
     &         PeriodEnd(3,1), PeriodEnd(4,1)
         ELSE
            IF(verbose .GT. 0 ) THEN
               WRITE(*, 8071) I, PeriodEnd(5,1)
            ENDIF
         ENDIF

!        Accumulate the EOI levels into the mean level variables
 1222    SPMLV = SPMLV + SPEOI
!            
!     Ok, have looped through each increment of a period.  The EOP 
!     values are the final EOI levels.
      SPEOP = SPEOI
!           
!     Must adjust the mean level values to subtract half of the final
!     EOI values (was only supposed to add a half inside the
!     loop, but added a whole).  Then divide by number of increments.
      SPMLV = (SPMLV - (SPEOP * HALF)) / DBLE(RoutingIncrement(1))
!
!     Now calculate mean outflow for the period.
      STMRIVFLOW = Round(STMRIVFLOW / DBLE(RoutingIncrement(1)),    &
     &   FlowRound)
!
!     Calculations for the entire routing period finished at this point.
!     Send the results to the superior output file
 1233 continue
!     Plan 1977A results
!
         SWPier = Round(SWPier / DBLE(RoutingIncrement(1)), LevelRound)
!        Since SupSideChannel is constant, these are mathematically equivalent
!        GateFlow = StMRivFlow - SupSideChannel
         GateFlow= Round(GateFlow / DBLE(RoutingIncrement(1)),FlowRound)
!        Update report fields
         WRITE(SWPierLevel, '(F8.3)') SWPier
         WRITE(SupGateFlow, '(I6)') NINT(GateFlow)
!
!        If not a monthly routing, some values have meaning only at BOM.
         IF(.NOT. NewMonth) THEN
            BalancingFlow = '        '
            SupPlanFlow = '      '
            SupPreProject = '      '
            SupBOMLevel = '        '
            IF(NINT(SupForceFlow)     .EQ. MissingVal .AND.     &
     &         NINT(NumSupForceGates) .EQ. MissingVal)          &
     &         P77ACodes = '                        '
         ENDIF

         IF( verbose .GT. 0 ) THEN
            WRITE(*, '(A)') P77AHeader
         ENDIF


      IF(verbose .GT. 0 ) THEN
         WRITE(*, 8089) PeriodEnd(5,1), Day(1), PeriodEnd(1,1), &
     &      PeriodEnd(3,1), PeriodEnd(4,1), SPBOP, SPMLV,       &
     &      SPEOP, STMRIVFLOW  
      ENDIF
!
!     Force the EOP level, if necessary, for use as next period's BOP
!     level:
      IF( SupForceEOPLev .NE. MissingVal )THEN
         IF(verbose .GT. 0 ) THEN
            WRITE(*,'(2X,A,F8.4,A,F12.7,A)')                       &
     &         ' Replacing modeled Superior EOP level (', SPEOP,   &
     &         ') with forced EOP level:', SupForceEOPLev, 'm.'
         ENDIF
         SPEOP = SupForceEOPLev
      ENDIF


 8080 FORMAT( 3X,3(I3), I5, 3F9.4, F9.1)
! 8081 FORMAT( 1X, 2(I3), I5, 1X, a1, 1x, A8, F8.3, F8.3, A8, F6.2, A8, 2A7, 
!     &    I7, 2A7, 1X, A21)
 8083 FORMAT( 3X,3(I3), I5, 3F9.4, 1X, F6.1, 21X, F9.1)
 8089 FORMAT(' Superior computation ending ', I10, 3(I3), I5,  &
     &   ' BOP/mean/EOP elevs: ',3F10.4,' mean outflow=',F9.1)
!
  614 FORMAT(' Exceeded max of ', I5,' iterations for increment ',I2)
  605 FORMAT(' Superior increment ',I5,                             &
     &            ' beginning- and end-of-increment levels: ',2F14.8)
  606 FORMAT(' Superior increment ',I5,                             &
     &            '    mean outflow and mean level for inc: ',2F14.8)
  612 FORMAT(I3, ' Required ', I5,                                  &
     &               ' iterations to converge within ',F7.3,' m3s.')
 8033 FORMAT(' End day: ',I9,' increment:', I3, A5,'Check=',F9.1,   &
     &   '  vs ',F10.2)
 8071 FORMAT(' Successfully solved increment ', I5,                 &
     &   ' of routing interval ending on day number', I9)
 8072 FORMAT(' Suspicious Superior routing for end day ',I9,        &
     &    ' increment', I5, ' (ending d/m/y= ',I2,'/',I2,'/',I5,')')

      END
 
!
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------

      SUBROUTINE SetUpSupOperationalSupplies(CompatIndex, MonthIndex,   &
     &    SPNTS, SPNWB, MHNWB, SCNWB, ERNWB, MHRet, SCRet, ERRet)
!      
!     This module assigns the desired "operational" supplies and
!     retardations to the variables used for Criterion B and C
!     checks. 
!
!modules
      use par_kind
      use outflow_mod

      IMPLICIT NONE 
!
!     input arguments
      INTEGER       CompatIndex
!                   aka P77ACompat
!                   1=outflow eqn, 2=1955 mod rule, 3=plan77a
!                   -1 = virtcarl, 0 = read-in, 1 = 6-month operational
      INTEGER       MonthIndex
!                   1=Jan
      real(wp) :: SPNTS
!                   sum of Superior water balance terms for current routing 
!                   period read from tsd file.
!
!     arguments used for output
      INTEGER       SPNWB, MHNWB, SCNWB, ERNWB, MHRet, SCRet, ERRet
!                   Values that hold routing data appropriate to setting
!                   of P77ACompat.  Not used after overtopping calls.
!
!
!           Now sort out ways to remain compatible with existing
!           procedures.  
!
            IF ( CompatIndex .EQ. -1 ) THEN
!              Virtcarl-type supplies.  Use latest operational values for
!              month 1 of the operational values for final flow average, 
!              and criterion checks
               SPNWB=OPLLO(MonthIndex)+ SPOPNBS07(MonthIndex,1)
               MHNWB = OPChicago + MHOPNBS07( MonthIndex, 1)
               SCNWB = SCOPNBS07( MonthIndex, 1)
               ERNWB = OPWelland( MonthIndex ) +   &
     &                   EROPNBS07( MonthIndex, 1)
               MHRet = SCRRet(MonthIndex)
               SCRet = DetRet(MonthIndex)
               ERRet = NiaRet(MonthIndex)
!
            ELSE IF( CompatIndex .EQ. 0 ) THEN
!              Replicating existing models.  Use observed supplies read
!              in for routing for final flow average, and criterion
!              checks.
               SPNWB = NINT(SPNTS)
!              We actually can't replicate Dumont/Larson model methods
!              without looking ahead to this month's supplies on the
!              middle lakes.  But this only comes into play regarding 
!              Criterion B, which is a mess anyway.  So we'll just
!              fake it here, and use the first month operationals.
               MHNWB = OPChicago + MHOPNBS07( MonthIndex, 1)
               SCNWB = SCOPNBS07( MonthIndex, 1)
               ERNWB = OPWelland( MonthIndex ) +   &
     &                   EROPNBS07( MonthIndex, 1)
               MHRet = SCRRet(MonthIndex)
               SCRet = DetRet(MonthIndex)
               ERRet = NiaRet(MonthIndex)
!  
            ELSE IF( CompatIndex .EQ. 1 ) THEN
!              Using Plan 1977A Supplies instead of operational supplies
               SPNWB = P77ALLOgokiDiv(MonthIndex) +  &
     &            P77ASup50NBS(MonthIndex)
               MHNWB = P77AChicagoDiv(MonthIndex) +  &
     &            P77AMH50NBS(MonthIndex)
               SCNWB = P77AStC50NBS( MonthIndex)
               ERNWB = P77AWellandFlow( MonthIndex ) +  &
     &                   P77AErie50NBS( MonthIndex)
               MHRet = P77AStClairRetrd(MonthIndex)
               SCRet = P77ADetroitRetrd(MonthIndex)
               ERRet = P77ANiagaraRetrd(MonthIndex)
!  
            ELSE IF( CompatIndex .EQ. 2 ) THEN
               STOP ' No P77A=2' 
!
            ELSE IF ( CompatIndex .EQ. 3 ) THEN
!              Virtcarl-type supplies.  Same as option -1, but use 
!              older operational values (only for month 1 of the  
!              6-month array for final flow average, and criterion checks).
               SPNWB=OPLLO(MonthIndex)+ SPOPNBS06(MonthIndex,1)
               MHNWB = OPChicago + MHOPNBS06( MonthIndex, 1)
               SCNWB = SCOPNBS06( MonthIndex, 1)
               ERNWB = OPWelland( MonthIndex ) +  &
     &                   EROPNBS06( MonthIndex, 1)
               MHRet = SCRRet(MonthIndex)
               SCRet = DetRet(MonthIndex)
               ERRet = NiaRet(MonthIndex)
!
            ELSE
               STOP 'Datsyuk'
            ENDIF
      END

!
!----&------------------------------------------------------------------
!
!
      SUBROUTINE Plan1977A(Month, Year, PreviousFlow,       &
     &   SupNTSfromTS, SupBOM, MHBOM, StClairBOM, ErieBOM,  &
     &   Plan1977AFlow, GatesOpen,                          &
     &         verbose)
!      
!modules
      use par_kind
      use lakesup_mod
      use outflow_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER          Month, Year
!                      (input) The month number and year to compute 
!                      outflow for.
!
      real(wp) :: PreviousFlow
!                      (input) last month outflow.
!
      INTEGER          SupNTSfromTS
!                      (input) Net total supply for the month, from the
!                      time-series input.
!    
      real(wp) :: SupBOM, MHBOM, StClairBOM, ErieBOM
!                      (input) Beginning-of-Month lake levels 
!    
      real(wp) :: Plan1977AFlow 
!                      (output) The flow for Month
!
      REAL(sp)         GatesOpen
!                      (output) The number of gates open in Month.
      integer verbose
!
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I, L
!                      Scratch variables
!
      CHARACTER*(*)    IDString
!                      Used as prefix on lines of detailed.log 
!
      INTEGER          ForecastPeriod, ForecastMonth(2)
!                      Identify the month for which forecasted 
!                      outflow is being computed.  ForecastPeriod
!                      refers to sequence in P77AMonthsToAve, while
!                      ForecastMonth(1) contains the month number from
!                      1-12, ForecastMonth(2) contains the year.  So if
!                      computing Plan 1977A for October 1998, with 
!                      5-month averaging, values get updated 5 times: 
!                       
!                          ForecastPeriod   ForecastMonth 
!                               1             10   1998
!                               2             11   1998
!                               3             12   1998
!                               4              1   1999
!                               5              2   1999
!
!      real(wp) :: MHNTS 
!                      The Net Total Supply to use for Lakes 
!                      Michigan-Huron (considering trigger levels, 
!                      and diverions)
!
      real(wp) :: ForecastSPEOP, ForecastSPBOP
      real(wp) :: ForecastMHEOP, ForecastMHBOP
      real(wp) :: ForecastSCEOP, ForecastSCBOP
      real(wp) :: ForecastEREOP, ForecastERBOP
      real(wp) :: ForecastSTMFlow, ForecastPrevFlow,ForecastBalance
!  ForecastSPMLV, ForecastMHMLV, ForecastERMLV, ForecastSCMLV,
!   ForecastSCRFlow, ForecastDETFlow, ForecastNIAFlow,
      real(wp) :: SecsInPeriod
!                      The number of seconds in routing period (month).
      real(wp) :: Round
!                      Function for rounding.

      PARAMETER (IDString='    Plan 1977A==> ')
!
      DATA SecsInPeriod     /2629746._wp/
!
!
      P77ACodes= '                        '
!
!     Log the data entering Plan 1977A    
!
      IF (verbose .eq. 1 ) THEN
         WRITE(*, 6523) IDString, MonthAbbrev(Month), Year
         WRITE(*, 6524) IDString,SupBOM,MHBOM,StClairBOM,ErieBOM
         WRITE(*, 6525) IDString, PreviousFlow, PreviousGates,   &
     &SupNTSfromTS
         WRITE(*, 6526) IDString, P77AMonthsToAve(Month)
      ENDIF
!
      Plan1977AFlow = 0.0_wp
!
!     Initialize BOP/EOP levels, and month/year counters
!
      ForecastSPEOP = Round(SupBOM,P77AMonthRound)
      ForecastMHEOP = Round(MHBOM,P77AMonthRound)
      ForecastSCEOP = Round(StClairBOM,P77AMonthRound)
      ForecastEREOP = Round(ErieBOM,P77AMonthRound)
      ForecastSTMFlow = PreviousFlow
      ForecastMonth(1) = Month - 1
      ForecastMonth(2) = Year
!
!
!     Write interesting data to character buffers for later output
!
      WRITE(SupBOMLevel,'(F8.3)') Round(ForecastSPEOP, -3)
!
      WRITE(BalancingFlow, '(I8)') NINT(Round( P77ABaseFlow(Month) +  &
     &      P77AAValue *                                              &
     &         (ForecastSPEOP -                                       &
     &             (P77ASupTargetLev(Month) +                         &
     &                (ForecastMHEOP - P77AMHTargetLev(Month))        &
     &                * P77ASupRangeFact(Month) /                     &
     &                P77AMHRangeFact(Month))), 0))
!
!
!     Loop through each month of the Plan 1977A forecast    
!
      DO 5555, ForecastPeriod = 1, P77AMonthsToAve(Month)
!
!
!        Increment month/year counters
!
         ForecastMonth(1) = ForecastMonth(1) + 1
         IF( ForecastMonth(1) .GT. 12) THEN
            ForecastMonth(1) = 1
            ForecastMonth(2) = ForecastMonth(2) + 1
         ENDIF
!
!
!        Determine length of timestep (unless uniform month)
!
         IF( .NOT. P77AUniformMonth )THEN
!           Get number of days in month in L (I is unused)
            CALL NumDays(ForecastMonth(2), ForecastMonth(1), I, L)
            SecsInPeriod = L * 86400
         ENDIF
!
!
!        Roll over the EOP/BOP values for this forecast period
!
         ForecastSPBOP = Round(ForecastSPEOP, P77AMonthRound)
         ForecastMHBOP = Round(ForecastMHEOP, P77AMonthRound)
         ForecastSCBOP = Round(ForecastSCEOP, P77AMonthRound)
         ForecastERBOP = Round(ForecastEREOP, P77AMonthRound) 
         ForecastPrevFlow = ForecastSTMFlow
!
         IF( Verbose .GT. 0) THEN
            WRITE(*, 6527) IDString, ForecastPeriod,                   &
     &         P77AMonthsToAve(Month), MonthAbbrev(ForecastMonth(1)),  &
     &         ForecastMonth(2)
         ENDIF
!
!
!        Compute balancing equation flow:
!
!
         ForecastBalance = Round( P77ABaseFlow(ForecastMonth(1)) +      &
     &      P77AAValue *                                                &
     &         (ForecastSPBOP -                                         &
     &             (P77ASupTargetLev(ForecastMonth(1)) +                &
     &                (ForecastMHBOP -P77AMHTargetLev(ForecastMonth(1)))&
     &                * P77ASupRangeFact(ForecastMonth(1)) /            &
     &                P77AMHRangeFact(ForecastMonth(1)))),              &
     &      P77AFlowRound)
!
         IF( Verbose .GT. 0) THEN
            WRITE(*, 2456) IDString, ForecastMonth(1), P77AAValue,      &
     &         ForecastMHBOP, ForecastSPBOP,                            &
     &         P77ABaseFlow(ForecastMonth(1)),                          &
     &         P77AMHTargetLev(ForecastMonth(1)),                       &
     &         P77AMHRangeFact(ForecastMonth(1)),                       &
     &         P77ASupTargetLev(ForecastMonth(1)),                      &
     &         P77ASupRangeFact(ForecastMonth(1))
         ENDIF
! 
         ForecastSTMFlow = ForecastBalance
!
         P77ACodes(1:3) = 'BE '
!
         CALL P77ALimitations( SecsInPeriod, ForecastPrevFlow, -9999,   &
     &      ForecastSPBOP, ForecastMHBOP, ForecastSCBOP, ForecastERBOP, &
     &      ForecastSPEOP, ForecastMHEOP, ForecastSCEOP, ForecastEREOP, &
     &      ForecastSTMFlow, GatesOpen, ForecastMonth(1),               &
     &      P77ACodes, IDString//' (fcast flow adjust) ',verbose)
!
!
         IF( Verbose .GT. 0) THEN
            WRITE(*, 6528) IDString, ForecastBalance
            WRITE(*, 6522) IDString, 'forecast', ForecastSTMFlow,       &
     &         GatesOpen
            WRITE(*, 6521) IDString, ForecastSPBOP, ForecastMHBOP,      &
     &         ForecastSCBOP, ForecastERBOP
            WRITE(*, 6520) IDString, ForecastSPEOP, ForecastMHEOP,      &
     &         ForecastSCEOP, ForecastEREOP
         ENDIF 
!
!
!
!
!
!
!
!        Accumulate forecast flows in Plan1977AFlow for later averaging
 5555    Plan1977AFlow = Plan1977AFlow + ForecastSTMFlow
!
!
!     Compute mean of forecast outflows
      Plan1977AFlow = Round(Plan1977AFlow / (ForecastPeriod - 1),  &
     &   P77AFlowRound)
!
!
!
      IF( Verbose .GT. 0) THEN
         WRITE(*, 6529) IDString, Plan1977AFlow
      ENDIF
!  
!
      IF( .NOT. P77AUniformMonth )THEN
!        Get number of days in month in L (I is unused)
         CALL NumDays(Year, Month, I, L)
         SecsInPeriod = L * 86400
      ENDIF
!
!
!     Do final adjustments of averaged forecast flow.  
!     ForecastSPEOP and ForecastSPMLV are not used (Superior actually takes
!     place in SuperiorRoute).  
!     
!
      P77ACodes(1:3) = 'AF '
!
      CALL P77ALimitations( SecsInPeriod, PreviousFlow, SupNTSfromTS,   &
!     XXXX Another dumb hack for backward compatibility.  Round levels
!     XXXX here to 2 places instead of usual four.  Would like to change
!     XXXX this!  Consider it a bug in Carl's model
     &   Round(SupBOM, P77AMonthRound), Round(MHBOM, P77AMonthRound),   &
     &   Round(StClairBOM,P77AMonthRound),Round(ErieBOM,P77AMonthRound),&
     &   ForecastSPEOP, ForecastMHEOP, ForecastSCEOP, ForecastEREOP,    &
     &   Plan1977AFlow, GatesOpen, Month,                               &
     &   P77ACodes, IDString//' (final flow adjust) ', verbose)
!
!
      WRITE(SupPlanFlow,'(I6)') NINT(Plan1977AFlow) 
!
      IF( Verbose .GT. 0) THEN
         WRITE(*, 6522) IDString, 'Plan 1977A', Plan1977AFlow,  &
     &      GatesOpen
      ENDIF 
!
!
!
 6522 FORMAT( A,' Computed St. Marys ', A,' flow of ', F6.1,          &
     &   ' m3s (',  F5.2, ' gates)')
 6521 FORMAT( A,' Beginning-of-Period levels:', 4F11.5)
 6520 FORMAT( A,'       End-of-Period levels:', 4F11.5)
 6523 FORMAT( A,'Entering Plan1977A to compute nominal outflow for ', &
     &   A3, I5, '.') 
 6524 FORMAT( A,'Beginning of month levels:', 4F16.11)
 6525 FORMAT( A,'Previous month outflow=', F20.3, ', gates open=',    &
     &   F8.2, '   Sup NTS=',I13) 
 6526 FORMAT( A,'Averaging over ', I2, ' forecasted months.')
 6527 FORMAT( A,' Calculating forecasted outflow for month ', I2,     &
     &   ' of ', I2, ' (', A3, I5, ').')
 6528 FORMAT( A,' Balancing equation flow: ', F12.3)
 6529 FORMAT( A,' Average of forecasted outflows: ', F12.3)

 2456       FORMAT(A,' Month=', I2, ' A=',F9.2,' MHBOP=', F9.4,       &
     &         ' SupBOP=', F9.4, ' Qavg=', I7, ' MH stats=', F9.4,    &
     &         F7.4, ' Sup stats=', F9.4, F7.4)
! 
      END
!
!
!    

